#
# akira_mpi.m4
#
# SYNOPSIS
#
#   ACX_AKIRA_MPI()
#   _ACX_AKIRA_MPI_DETECT()
#
# DESCRIPTION
#
#   ACX_AKIRA_MPI determines whether MPI parallelization is requested.
#
#   _ACX_AKIRA_MPI_DETECT checks for the compiler/linker flags required
#   to build with MPI support.
#

#
# _ACX_AKIRA_MPI_DETECT()
#
AC_DEFUN([_ACX_AKIRA_MPI_DETECT], [
AC_PREREQ(2.61)

AC_ARG_VAR([MPIFC], [MPI Fortran compiler command])
# Virtually every MPI implementation provides mpif{c,90} which takes care
# of everything. We thus solely rely on the presence of this binary.
AC_CHECK_PROGS( [MPIFC], [mpif90 mpxlf95_r mpxlf90_r mpxlf95 mpxlf90 mpf90 cmpif90c])
AS_IF([test -z "$ac_cv_prog_MPIFC"],
	[AC_MSG_ERROR([MPI Fortran compiler not found])]
)

])dnl _ACX_AKIRA_MPI_DETECT



#
# ACX_AKIRA_MPI()
#
AC_DEFUN([ACX_AKIRA_MPI], [
AC_PREREQ(2.61)

AC_MSG_CHECKING([whether to activate MPI support])
AC_ARG_WITH(
	[mpi],
	[AS_HELP_STRING([--with-mpi], [build a parallel version using MPI])],
	[AS_CASE($with_mpi,
		[no], [with_mpi=no],
		[with_mpi=yes])
	],
	[with_mpi=no]
)
AC_MSG_RESULT([$with_mpi])

AS_IF([test x"$with_mpi" != xno], [_ACX_AKIRA_MPI_DETECT()])

])dnl ACX_AKIRA_MPI
