#
# akira_parallel.m4
#
# SYNOPSIS
#
#   ACX_AKIRA_PARALLEL()
#
# DESCRIPTION
#
#   ACX_AKIRA_PARALLEL determines whether a serial or parallel version
#   should be build.
#
#
AC_DEFUN([ACX_AKIRA_PARALLEL], [
AC_PREREQ(2.61)
AC_REQUIRE([ACX_AKIRA_PVM])
AC_REQUIRE([ACX_AKIRA_MPI])

AC_MSG_CHECKING([for parallelization requested])

# this construction prefers PVM over MPI and ensures
# that only one of them is activated
AS_IF(
	[test x"$with_pvm" != xno], [acx_akira_parallel="pvm"],
	[test x"$with_mpi" != xno], [acx_akira_parallel="mpi"],
	[acx_akira_parallel="serial"]
)

AM_CONDITIONAL([COND_PARALLEL], [test x"$acx_akira_parallel" != x"serial"])

AS_CASE([$acx_akira_parallel],
	[serial], [
		AC_MSG_RESULT([none])
		AC_DEFINE([AKIRA_SERIAL], 1, [Define to 1 if building serial version])
	],
	[pvm], [
		AC_MSG_RESULT([PVM])
		AC_DEFINE([AKIRA_PVM], 1, [Define to 1 if building PVM version])
	],
	[mpi], [
		AC_MSG_RESULT([MPI])
		AC_DEFINE([AKIRA_MPI], 1, [Define to 1 if building MPI version])
		# always use MPI_SSEND if using MPI
		AC_DEFINE([AKIRA_MPI_SSEND], 1, [Define to 1 for using MPI_SSEND instead of MPI_SEND])
	],
	[AC_MSG_ERROR([Internal error: Unknown parallelization method])]
)

])dnl ACX_AKIRA_PARALLEL

