#
# akira_pvm.m4
#
# SYNOPSIS
#
#   ACX_AKIRA_PVM()
#   _ACX_AKIRA_PVM_DETECT()
#
# DESCRIPTION
#
#   ACX_AKIRA_PVM determines whether PVM parallelization is requested.
#
#   _ACX_AKIRA_PVM_DETECT checks for the compiler/linker flags required
#   to build with PVM support.
#

#
# _ACX_AKIRA_PVM_DETECT()
#
AC_DEFUN([_ACX_AKIRA_PVM_DETECT], [
AC_PREREQ(2.61)
AC_REQUIRE([AC_PROG_FC])

# check for libraries

acx_akira_pvm_detect_save_LIBS="$LIBS"

AC_ARG_VAR([PVM_ROOT], [Location of the PVM installation])
AC_MSG_CHECKING([whether PVM_ROOT is set])
AS_IF([test -z "$PVM_ROOT"],
	[AC_MSG_ERROR([environment variable PVM_ROOT is not set])],
	[AC_MSG_RESULT([yes, $PVM_ROOT])]
)

AC_ARG_VAR([PVM_ARCH], [PVM architecture])
AC_MSG_CHECKING([whether PVM_ARCH is set])
AS_IF([test -z "$PVM_ARCH"],
	[AC_MSG_ERROR([environment variable PVM_ARCH is not set])],
	[AC_MSG_RESULT([yes, $PVM_ARCH])]
)

acx_akira_pvm_detect_L="-L$PVM_ROOT/lib/$PVM_ARCH"
LIBS="$LIBS $acx_akira_pvm_detect_L"

AC_LANG_PUSH([Fortran])
AC_SEARCH_LIBS([pvmfexit],
	[fpvm3],
	[],
	[AC_MSG_FAILURE([Unable to find PVM Fortran library])],
	[-lpvm3]
)
AC_LANG_POP([Fortran])

LIBS="$acx_akira_pvm_detect_save_LIBS"

PVM_LIBS="$acx_akira_pvm_detect_L $ac_cv_search_pvmfexit -lpvm3"

# check for headers
acx_akira_pvm_detect_save_FCLAGS="$FCLAGS"

acx_akira_pvm_detect_I="-I$PVM_ROOT/include"
FCFLAGS="$FCFLAGS $acx_akira_pvm_detect_I"
AC_LANG_PUSH([Fortran])
ac_ext=F
# We cannot use AC_CHECK_HEADER since this does not work with Fortran (yet)
AC_MSG_CHECKING([for fpvm3.h])
AC_COMPILE_IFELSE(
	[AC_LANG_PROGRAM([],[
#include <fpvm3.h>
      integer i
      i=PVMHOST])
	],
	[AC_MSG_RESULT([yes])],
	[AC_MSG_FAILURE([Unable to find (working) fpvm3.h])]
)
AC_LANG_POP([Fortran])

FCLAGS="$acx_akira_pvm_detect_save_FCLAGS"

AM_CPPFLAGS="$AM_CPPFLAGS $acx_akira_pvm_detect_I"

AC_SUBST([PVM_LIBS])
AC_SUBST([AM_CPPFLAGS])
])dnl _ACX_AKIRA_PVM_DETECT



#
# ACX_AKIRA_PVM()
#
AC_DEFUN([ACX_AKIRA_PVM], [
AC_PREREQ(2.61)

AC_MSG_CHECKING([whether to activate PVM support])
AC_ARG_WITH(
	[pvm],
	[AS_HELP_STRING([--with-pvm], [build a parallel version using PVM])],
	[AS_CASE($with_pvm,
		[no], [with_pvm=no],
		[with_pvm=yes])
	],
	[with_pvm=no]
)
AC_MSG_RESULT([$with_pvm])

AS_IF([test x"$with_pvm" != xno], [_ACX_AKIRA_PVM_DETECT()])

])dnl ACX_AKIRA_PVM
