#
# snf_flags.m4
#
# SYNOPSIS
#
#   ACX_SNF_CFLAGS()
#   ACX_SNF_FCFLAGS()
#   ACX_SNF_FFLAGS()
#
# DESCRIPTION
#
#   ACX_SNF_CFLAGS sets CFLAGS if not set in the environment
#   ACX_SNF_FCFLAGS sets FCFLAGS if not set in the environment
#   ACX_SNF_FFLAGS sets FFLAGS if not set in the environment
#

#
# ACX_SNF_CFLAGS()
# 
AC_DEFUN([ACX_SNF_CFLAGS], [
AC_PREREQ(2.61)

AC_MSG_CHECKING([for CFLAGS])
acx_snf_cflags_message=""

# Test whether CFLAGS was set in the environment. If so, use this
# value, if not check whether we know the compiler. If it is known,
# substitute our CFLAGS, otherwise leave them unchanged.
AS_IF([test "$ac_cv_env_CFLAGS_set" = set],
	[acx_snf_cflags_message="(by user-request)"],
	[AS_CASE([$CC],
		[: # default dummy]
	)]
)
AC_MSG_RESULT([$CFLAGS $acx_snf_cflags_message])
])dnl ACX_SNF_CFLAGS

#
# ACX_SNF_FCFLAGS()
#
AC_DEFUN([ACX_SNF_FCFLAGS], [
AC_PREREQ(2.61)

AC_MSG_CHECKING([for FCFLAGS])
acx_snf_fcflags_message=""

# Test whether FCFLAGS was set in the environment. If so, use this
# value, if not check whether we know the compiler. If it is known,
# substitute our FCFLAGS, otherwise leave them unchanged.
AS_IF([test "$ac_cv_env_FCFLAGS_set" = set],
	[acx_snf_fcflags_message="(by user-request)"],
	[AS_CASE([$FC],
		[gfortran], [FCFLAGS=""],
		[ifc|ifort], [FCFLAGS="-O2 -mp"],
		[lf9?], [FCFLAGS="-O --ntrace --dal"],
		[pgf9?], [FCFLAGS="-fast -Munroll=c:3,n:1 -pc 64"],
		[xlf9?], [FCFLAGS="-O3 -qstrict -q64 -qsuffix=f=f90 -qextname=flush"],
		[: # default dummy]
	)]
)

AC_MSG_RESULT([$FCFLAGS $acx_snf_fcflags_message])
])dnl ACX_SNF_FCFLAGS

#
# ACX_SNF_FFLAGS()
#
AC_DEFUN([ACX_SNF_FFLAGS], [
AC_PREREQ(2.61)

AC_MSG_CHECKING([for FFLAGS])
acx_snf_fflags_message=""

# Test whether FFLAGS was set in the environment. If so, use this
# value, if not check whether we know the compiler. If it is known,
# substitute our FFLAGS, otherwise leave them unchanged.
AS_IF([test "$ac_cv_env_FFLAGS_set" = set],
	[acx_snf_fflags_message="(by user-request)"],
	[AS_CASE([$F77],
		[g77], [FFLAGS="-fforce-addr -w -funix-intrinsics-hide -fno-f2c -ffast-math"],
		[gfortran], [FFLAGS="-O2"],
		[ifc|ifort], [FFLAGS="-O2 -mp"],
		[pgf77], [FFLAGS="-O"],
		[xlf], [FFLAGS="-O -qextname=flush"]
	)]
)

AC_MSG_RESULT([$FCFLAGS $acx_snf_fflags_message])
])dnl ACX_SNF_FFLAGS
