#
# snf_ps.m4
#
# SYNOPSIS
#
#   ACX_SNF_PS()
#
# DESCRIPTION
#
#   ACX_SNF_PS detects the operation system and from
#   that guesses the syntax and output format of the 
#   ps(1) command.
#
AC_DEFUN([ACX_SNF_PS], [
AC_PREREQ(2.61)
AC_REQUIRE([AC_CANONICAL_HOST])

AC_MSG_CHECKING([for syntax and output format of ps])

acx_snf_ps_notrap="no"

AS_CASE($host_os,
	[linux*], [
		acx_snf_ps_os="linux"
		acx_snf_ps_options="axl"
		acx_snf_ps_ppid="4"
		acx_snf_ps_interval="10"
		],
	[irix*], [
		acx_snf_ps_os="irix"
		acx_snf_ps_options="-aef"
		acx_snf_ps_ppid="3"
		acx_snf_ps_interval="60"
		],
	[aix*], [
		acx_snf_ps_os="aix"
		acx_snf_ps_options="axl"
		acx_snf_ps_ppid="5"
		acx_snf_ps_interval="10"
		],	
	[hpux*], [
		acx_snf_ps_os="hpux"
		acx_snf_ps_options="-ael"
		acx_snf_ps_ppid="5"
		acx_snf_ps_interval="10"
		],
	[hiuxwe*], [
		acx_snf_ps_os="hiuxwe"
		acx_snf_ps_options="axl"
		acx_snf_ps_ppid="3"
		acx_snf_ps_interval="60"
		acx_snf_ps_notrap="yes"
		],
	[osf*], [
		acx_snf_ps_os="osf"
		acx_snf_ps_options="axl"
		acx_snf_ps_ppid="3"
		acx_snf_ps_interval="10"
		],
	[AC_MSG_ERROR([unknown operating system $host_os])]
)

AC_MSG_RESULT([$acx_snf_ps_os])

AC_DEFINE_UNQUOTED([SNF_PS_OS], ["$acx_snf_ps_os"], [Define to the operating system detected for ps(1)])
AC_DEFINE_UNQUOTED([SNF_PS_OPTIONS], ["$acx_snf_ps_options"], [Define to the command line options for ps(1)])
AC_DEFINE_UNQUOTED([SNF_PS_PPID], ["$acx_snf_ps_ppid"], [Define to the column in ps(1) output containing the PPID])
AC_DEFINE_UNQUOTED([SNF_PS_INTERVAL], [$acx_snf_ps_interval], [Define to the interval of ps(1) calls])

AS_IF([test "$acx_snf_ps_notrap" = "yes"],
	[AC_DEFINE([SNF_PS_NOTRAP], 1, [Define to 1 to suppress timer trap in fsystem.c])]
)

])
