      program akiradefine
*-----------------------------------------------------------------------
* prepare akira run
*
* by J. Neugebauer, M. Reiher, Uni Erlangen, 2002
*
* nbm = no. of basis modes
*
* modified by C. Herrmann, Uni Bonn, 2004-2005, 
* for Gaussian and ADF interface
* modified by C. Herrmann, Uni Jena, 04/2005,
* for excited state gradients
*
* f90 memory management by J. Neugebauer, 2007
*-----------------------------------------------------------------------
      use CoordModule

#include "defs.h"
#include "param.h"

      real*8, allocatable :: ortmod(:,:),normm(:,:)
      integer, allocatable :: istepfl(:)

      integer i,j,dmode
      integer numderiv,lbuf,rpaconv,disptype,scfconv,nbm
      integer nnbm(mxit),dalmw,nbmo
      real*8 escf,temp,theta,convcr,zt,co(3,maxatoms),el(maxatoms)
      logical tmpcl,logcl,anffast,ex,crap,proj,apphess
      logical mpmode,rimpmode,valmod,goback,direct
      logical excoord, excontrl
      character*10 progtype,calctype,dalfunc
      character*132 buf
      character*3 defprog
      external valmod
      integer iclean,icleanl
      
*---------------------------
*  initialize some variables
*---------------------------

      call clean(mydir)
      lm         = 0
      nnbm(1)    = 1
      iomeg      = 0
      ipol       = 0
      disptype   = 1
      anffast    = .false.
      tmpcl      = .false.
      logcl      = .false.
      genstart   = .false.
      proj       = .false.
      cstep      = 0.01000d0
      dalmw      = 0
      numderiv   = 1
      iosl       = idbg
      scfconv    = 8
      maxnbm     = 150
      direct     = .true.
      mode       = 0
      lastqmatom = 0
      sleeptime  = 5

*-----------------------------------------------
*  print header
*-----------------------------------------------
      call akiradefinehead
*------------------------------------------------------
*  'new' program selection
*
CCH  ... this generates an integer dummy mode variable 
CCH  'dmode' which will be used as fake screen input 
CCH  in the 'old' program selection later.
CCH  (if 'mode' were read in here, it would be
CCH  overwritten in the following code)
*------------------------------------------------------
      dmode = 0
      ex=.false.
      defprog='tm'
      inquire(file='molecule.mol',exist=ex)
      if(ex) then
        defprog = 'dalton'
        dmode = 20
      end if
      inquire(file='akira.com',exist=ex)
      if(ex) then 
        defprog = 'g03'
        dmode = 32
      end if
      inquire(file='adf.in',exist=ex)
      if(ex) then 
        defprog = 'adf' 
        dmode = 40
      end if
      inquire(file='comqum.dat',exist=ex)
        if(ex) then
        defprog = 'comqum'
        dmode = 60
        mode=60
      end if

340   write(*,*)
      write(*,*)' Which program would you like to use ? '
      write(*,*)
      write(*,*)  'tm        : use TURBOMOLE for single points'
      write(*,*)  'd[alton]  : use DALTON for single points'
      write(*,*)  'g98       : use GAUSSIAN 98 for single points'
      write(*,*)  'g03       : use GAUSSIAN 03 for single points'
      write(*,*)  'a[df]     : use ADF for single points'
      write(*,*)  'c[omqum]  : use comqum QM/MM interface for '
     *                         //'single points'
      write(*,*)
      write(*,*)  '(<return> = default = '//defprog//')'
      write(*,*)
      read(*,'(A)') buf
      call slen(buf,lbuf)
      if(lbuf.ne.0) then
        if(buf(1:2).eq.'tm') then
          continue
        else if(buf(1:1).eq.'d') then
          dmode=20
        else if(buf(1:3).eq.'g98') then
          dmode=30
        else if(buf(1:3).eq.'g03') then
          dmode=32
        else if(buf(1:1).eq.'a') then
          dmode=40
        else if(buf(1:1).eq.'c') then
          dmode=60
          mode=60
        else
          write(*,*)
          write(*,*) ' unknown command'
          write(*,*)
          goto 340
        end if
      end if
      write(*,*)


*------------------------------------------------------
*  check existence of all mandatory files
*------------------------------------------------------
      if(dmode.eq.20) then
        call testfile('molecule.mol')
        call testfile('molecule.dal')
      else if(dmode.ge.30.and.dmode.lt.33) then
        call testfile('akira.com')
      else if(dmode.eq.40.or.dmode.eq.41) then
        call testfile('adf.in')
        call bfile('adf.in')
      else if(dmode.eq.60) then
CCH not sure whether pointcharges, sander.in3 always needed
        call testfile('comqum.dat')
        call testfile('prmcrd1')
        call testfile('prmcrd3')
        call testfile('prmtop1')
        call testfile('prmtop2')
        call testfile('prmtop3')
        call testfile('sander.in1')
        call testfile('sander.in2')
        call testfile('sander.in3')
        call testfile('pointcharges')
      end if
      if(dmode.eq.0.or.dmode.eq.60) then
        call testfile('control')
        call bfile('control')
        call rkeyint('control','$scfconv',scfconv)
      end if
*------------------------------------------------------
* Gaussian/ADF: read in atoms from input file
* and write fake Turbomole coord file
*------------------------------------------------------
        if(dmode.eq.30.or.dmode.eq.32)  then
          call wgausscontrol(co,natoms,maxatoms)
          call chkgnosymm
        else if(dmode.eq.40) then
          call gadflastqm(lastqmatom,qmmmatomtype)
          call wadfcontrol(co,natoms,irescp,lastqmatom,el)
          if(lastqmatom.eq.0) lastqmatom=natoms
          if(lastqmatom.ne.natoms) dmode=41
          call checkadfinput(co,el)
        end if
*------------------------------------------------------
*  read default values for "menu" from 
*  (akira_control and) control
*  Note: convcr is not used so far ...
*------------------------------------------------------
      call rdefcontrol(tmpcl,anffast,logcl,cstep,numderiv,mode,
     * rpaconv,convcr)
*------------------------------------------------------
*  read basic input from control and coord
*------------------------------------------------------
      write(*,*) 'calling akiraini'
      call akiraini(6,1)
      if(rpast.ne.0) then
        write(*,*)
        write(*,*) '*********************************************'
        write(*,*) '*** ATTENTION!                            ***'
        write(*,*) '*** Excited state calculation specified ! ***'
        write(*,*) '*** Make sure that TURBOMOLE V >= 5.6     ***'
        write(*,*) '*** is available in your PATH             ***'
        write(*,*) '*********************************************'
        write(*,*)
      end if

CCH TURBOMOLE SETTING CHECK MIGHT BE EXTENDED TO COMQUM ONE DAY
      if(dmode.ge.30) goto 350
    
*------------------------------------------------------
*  check program mode
*------------------------------------------------------

      mode=0
      call rkeyint('akira_control','$anfmode',mode)
      call rkeywd('control','$cbas',rimpmode)
      call rkeywd('control','$traloop',mpmode)
      if(mpmode) call rnscrfi
      if(mode.ne.10) then
        if(rimpmode.or.mpmode) then
          mode=5
        else
          mode=4
        end if
      end if
      if(.not.valmod(mode)) 
     * call serror('akiradefine',2,'mode not valid',6)

350   continue
      if(dmode.eq.20) then
          mode=20
          call rkeyint('akira_control','$dalmw',dalmw)
          call dalopt(direct,dalmw,mode,dalfunc,goback)
          if(goback) goto 350
      else if(dmode.ge.30) then
          mode = dmode
      end if
      call prog_type(mode,progtype,calctype)
      if(index(progtype,'DALTON').ne.0) then
        call wdalinp(icharge,coord,natoms,direct,dalfunc,mode)
      end if
*------------------------------------------------------
*  read equilibrium energy from gradient
* ADF: check (and modify) input file
*------------------------------------------------------
      ex = .false.
      if(mode.lt.30) then
        inquire(file='gradient',exist=ex)
        if(ex) then
          call getescf('gradient',escf)
        else
        call serror('akiradefine',0,
     *    'Reading energy from file gradient failed',jtape)
        end if
      else if(mode.lt.40.and.mode.ge.30) then
          write(*,*) 'Reading equilibrium energy not yet implemented for
     * Gaussian'
      else if(mode.ge.40.and.mode.lt.50) then
        inquire(file='adf.out',exist=ex)
        if(ex) then
          call gadfenergy('adf.out',escf)
      else if(mode.ge.60.and.mode.lt.70) then
        write(*,*) 'Reading equilibrium energy not yet implemented for'
     *              //' Comqum'
      else
         call serror('akiradefine',0,
     *    'Reading energy from file adf.out failed',jtape)
        end if
      end if
      if(.not.ex) then
30      write(*,*) 
        write(*,*)' Enter name of output file to read total energy from'
        write(*,*)' (<return> to set energy to zero)'
        write(*,*) 
        read(*,'(A)') buf
        call slen(buf,lbuf)
        if(lbuf.eq.0) then
          call serror('akiradefine',0,'Do not use mode 3 with E_scf=0',
     * jtape)
          escf=0.0d0
        else
          inquire(file=buf(1:lbuf),exist=ex)
          if(ex) then
            if(index(buf,'gradient').eq.0) then
              call getonlye(buf(1:lbuf),escf)
            else if(mode.ge.40.and.mode.lt.50) then
              call gadfenergy(buf(1:lbuf),escf)
            else
              call getescf(buf(1:lbuf),escf)
            end if
          else
            call serror('akiradefine',0,'No such file!',jtape)
            goto 30
          end if
        end if
      end if
      write(*,*) 
      write(*,'(A,f24.12)') 
     * ' Energy of equilibrium structure: ',escf
 
*------------------------------------------------------
*  interactive input part
*------------------------------------------------------
      call rkeyint('akira_control','$disptype',disptype)
      call rkeyint('akira_control','$maxnbm',maxnbm)
      if(nvar.lt.maxnbm) maxnbm=nvar
100   call progsetmenu(tmpcl,anffast,logcl,cstep,numderiv,mode,temp,
     * theta,rpaconv,disptype,maxnbm,proj,masstype,scfconv,nvar)
      jscfc=scfconv

      naus=2*numderiv
      nstept=naus*maxnbm
      maxnstept=naus*maxnbm

*-------------------------------------------------------
*  convergence criteria
*-------------------------------------------------------
      goback=.false.
200   call convcmenu(rthres,rabsthres,dvthres,ecthres,rchthres,
     * iorth,goback)
      if(goback) goto 100

*-------------------------------------------------------
*  output settings
*-------------------------------------------------------
      goback=.false.
300   call outmenu(g98bs,g98it,tmout,xmolout,snfout,printall,goback)
      if(goback) goto 200

*-----------------------------------------------------------
*  select basis modes (basis vectors for subspace iteration)
*-----------------------------------------------------------
      allocate(ortmod(nvar,maxnbm),istepfl(nvar*maxnbm))
      allocate(normm(nvar,maxnbm+1))
      call selectstp(normm,nnbm(1),naus,natoms,istepfl,icharge,
     * coord,mcharge,maxnbm,apphess,ortmod,nbmo)
*------------------------------------------------------
*  modify control according to the user's settings
*------------------------------------------------------
      inquire(exist=ex, file='akira_control')
      if(.not.ex) then
      call wakiractrl('akira_control',naus,nvar,nnbm,mxit,1,rthres,
     *  apphess,rabsthres,dvthres,ecthres,roothmpr,g98bs,g98it,
     *  xmolout,tmout,printall,precond,genstart,rchthres,minnbt,
     *  iorth,snfout,natoms,mode,cstep,iclean,icleanl,idbg,ipol,
     *  lastqmatom,irescp,restadf,sleeptime)
      end if
       call modcontrol(tmpcl,anffast,logcl,cstep,numderiv,mode,
     * rpaconv,disptype,proj,masstype,maxnbm,scfconv,dalmw,nbmo)

      if(mode.eq.2) then
        ipol=1
      end if
!      call normalize(normm,nvar)
      write(*,*) 
      write(*,*) '----------------------------------------------------'
      write(*,*) 'basis modes are written to akira_control ...'
      call wrnctm('akira_control',natoms,normm,maxnbm,
     *   '$vibrational basis modes')
      write(*,*) '... done'

*--------------------------------------------------------------------------
* write set of orthonormal modes to control
*--------------------------------------------------------------------------

      if(nbmo.gt.0) then
        write(*,*) 'orthogonal subspace is written to control ...'
        call wrnctm('akira_control',natoms,ortmod,nbmo,
     *   '$orthonormal basis modes')
        write(*,*) '... done'
      end if

*-------------------------------------------------------
*  write initial restart file
*-------------------------------------------------------
      write(*,*) 'initial restart file is written ...'
      call winirestart(istepfl,escf,nnbm(1))
      write(*,*) '... done'
      write(*,*) '----------------------------------------------------'
      write(*,*) 
      deallocate(ortmod,istepfl,normm)

*-------------------------------------------------------
*  root homing and preconditioning
*-------------------------------------------------------
      goback=.false.
400   call rootmenu(roothmpr,precond,apphess,genstart,minnbt,goback)
      if(goback) goto 300
      call wakiractrl('akira_control',naus,nvar,nnbm,mxit,1,rthres,
     *  apphess,rabsthres,dvthres,ecthres,roothmpr,g98bs,g98it,
     *  xmolout,tmout,printall,precond,genstart,rchthres,minnbt,
     *  iorth,snfout,natoms,mode,cstep,iclean,icleanl,idbg,ipol,
     *  lastqmatom,irescp,restadf,sleeptime)


*-------------------------------------------------------
*  write DCINPUT
*-------------------------------------------------------
      call clean(cnopt)
      call rkeystr('DCINPUT','CHOOSENOPT',cnopt)
      call llen(cnopt,lnopt)
      write(*,*) 
      write(*,*) ' =================================== '
      write(*,*) ' Input completed, calling prepdc ... '
      write(*,*) ' =================================== '
      write(*,*) 
      write(*,*) ' Options for choose_nodes call: '
      write(*,'(1x,A)') cnopt
133   write(*,*)
      write(*,*) ' Specify alternative options for choose_nodes or'//
     *           ' press <return>'
      read(*,'(A)') buf
      call llen(buf,lbuf)
      crap=.false.
      if(cstep.lt.0.01.or.cstep.gt.0.05.or.scfconv.lt.8) then
         write(*,*) 'CAUTION: unsafe settings for frequency calculation'
         write(*,*) '         chosen; I will add option'
         write(*,*) '         CRAP_OK yes              '
         write(*,*) '         in DCINPUT file          '
         write(*,*) '(press <return> to continue)'
        read(*,'(A)') buf
        crap=.true.
      end if
      call prepdc(crap,cnopt)
      end
      
**************************************************************************

     
      subroutine akiradefinehead
*
* write header
* written by J. Neugebauer, Uni Erlangen, 2002
*
#include "defs.h"
#include "version.h"
      write(*,'(A)')
C        +1234567890123456789012345678+8765432109876543210987654321+
      write(*,'(A)')
     * ' +----------------------------------------------------------+'
      write(*,'(A)')
     * ' |                                                          |'
      write(*,'(A)')
     * ' |                        AKIRADEFINE                       |'
      write(*,'(A)')
     * ' |                                                          |'
      write(*,'(A)')
     * ' |                            by                            |'
      write(*,'(A)')
     * ' |                                                          |'
      write(*,'(A)')
     * ' |                J. Neugebauer, C. Herrmann,               |'
      write(*,'(A)')
     * ' |                      and M. Reiher                       |'
      write(*,'(A)')
     * ' |                                                          |'
      write(*,'(A)')
     * ' |             www.reiher.ethz.ch/software/akira            |'
      write(*,'(A)')
     * ' |                                                          |'
      write(*,'(A)')
     * ' |                  ETH Zurich, 2006-2007                   |'
      write(*,'(A)')
     * ' |                                                          |'
      write(*,'(A)')
     * ' | cite this work as:                                       |'
      write(*,'(A)')
     * ' | M. Reiher, J. Neugebauer, J. Chem. Phys., 118 (2003) 1634|'
      write(*,'(A)')
     * ' |                                                          |'
      write(*,'(A)')
     * ' +----------------------------------------------------------+'
C        +1234567890123456789012345678+8765432109876543210987654321+
      write(*,'(A)')
      write(*,'(A)') version
      write(*,'(A)')
      end
      
**************************************************************************

      subroutine prepdc(crap,cnopt)
*-----------------------------------------------------------------------
* call mkrdcinput script for preparation of DCINPUT
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*-----------------------------------------------------------------------
      implicit none
      integer info,system,info2,lnopt,li,ncpu,i,lbuf
      logical crap,parafreq,ex
      character*(*) cnopt
      character*255 buf
      parameter(li=1)

      parafreq=.false.
      call rkeywd('control','$parafreq',parafreq)
      if(parafreq) then
        info=system('mkparainput')
      else
        info=system('mkrdcinput')
      end if
      if(crap) info2=system('echo CRAP_OK yes >> DCINPUT')
      write(*,*) 
      write(*,*) '>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<'
      write(*,*) '====================================================='
      write(*,*) '== IF YOU INTEND TO USE AKIRA WITH A BATCH QUEUING =='
      write(*,*) '== SYSTEM PLEASE ADOPT THE EXAMPLE RUNSCRIPT IN    =='
      write(*,*) '== THE scripts DIRECTORY. AKIRA MUST THEN GET THE  =='
      write(*,*) '== NODE INFO FROM SUCH A RUNSCRIPT. IN THAT CASE,  =='
      write(*,*) '== YOU CAN IGNORE ANY FOLLOWING ERROR MESSAGES.    =='
      write(*,*) '====================================================='
      write(*,*) '>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<'
      write(*,*) 
      call serror('prepdc',5,'(no message)',6)
      call llen(cnopt,lnopt)

C     --------------------------------------------------
C     create USE_NODES file for counting no. of CPUs
C     --------------------------------------------------
      if(lnopt.gt.1) then
        call clean(buf)
        buf='echo CHOOSENOPT '//cnopt(1:lnopt)//' >> DCINPUT'
        info=system(buf)
        call clean(buf)
        write(buf,'(A)') 'choose_nodes '//cnopt(1:lnopt)
        call llen(buf,lbuf)
        info=system(buf)
      else
        info=system('choose_nodes')
      end if
      write(*,*) 'called choose_nodes'

C     --------------------------------------------------
C     count no. of CPUs
C     --------------------------------------------------
      open(unit=li,file='USE_NODES')
      do i=1,3
        read(li,'(A)',err=999,end=999) buf
      end do
10    read(li,'(A)',err=999,end=999) buf
      if(index(buf,'# chosen').ne.0) goto 20
      goto 10
999   call serror('prepdc',1,'Could not determine number of nodes !',6)
      goto 30
20    read(buf(index(buf,'with')+4:index(buf,'proc')-1),*) ncpu
30    close(li)

C     --------------------------------------------------
C     create MACHINES file for MPI run
C     --------------------------------------------------
      if(lnopt.gt.1) then
        call clean(buf)
        write(buf,'(A)') 'choose_nodes -mpi '//cnopt(1:lnopt)
        call llen(buf,lbuf)
        info2=system(buf)
        write(*,*) 'called choose_nodes'
      else
        info2=system('choose_nodes -mpi')
      end if
      info2=system('mv USE_NODES MACHINES')

      if(info.ne.0.or.info2.ne.0) call serror('prepdc',1,'choose_nodes f
     *ailed. Make sure that choose_nodes is available in your path.',6)
      write(*,*) '... creating file mpi.sub for MPI-MACHINERY'
      open(unit=li,file='mpi.sub')
      write(li,'(A,i3,A)') 'mpirun -machinefile MACHINES -np ',ncpu,' ak
     *ira'
      close(li)
      info=system('chmod u+x mpi.sub')
      if(info.ne.0) call serror('prepdc',1,'Error while preparing mpi.su
     *b',6)

C     --------------------------------------------------
C     create USE_NODES file for PVM run
C     --------------------------------------------------
      write(*,*) '... creating file USE_NODES for PVM-MACHINERY'
      if(lnopt.gt.1) then
        call clean(buf)
        buf='echo CHOOSENOPT '//cnopt(1:lnopt)//' >> DCINPUT'
        info2=system(buf)
        call clean(buf)
        write(buf,'(A)') 'choose_nodes '//cnopt(1:lnopt)
        call llen(buf,lbuf)
        info=system(buf)
      else
        info=system('choose_nodes')
      end if
      if(info.ne.0) call serror('prepdc',1,'choose_nodes failed. Make su
     *re that choose_nodes is available in your path.',6)
      if(info.ne.0) then
       call serror('prepdc',1,'Could not properly execute mkrdcinput',6)
      end if
      end
      
**************************************************************************

      subroutine progsetmenu(tmpcl,anffast,logcl,cstep,numderiv,mode,
     *  temp,theta,rpaconv,disptype,maxnbm,proj,masstype,scfconv,nvar)

*-----------------------------------------------------------------------
* interactive program settings menu
* 
* written by J. Neugebauer, Uni Erlangen, 2002
*-----------------------------------------------------------------------
      implicit none
      integer numderiv,numderiv2,lrow,mode,rpaconv,disptype,masstype
      integer maxnbm,scfconv,nvar,oldmaxnbm
      real*8 cstep,temp,theta
      logical tmpcl,logcl,anffast,valid,valdtype,proj,normlog
      character*3 tag
      character*40 dtype
      character*132 row
      external valdtype

      numderiv2=2*numderiv+1
      valid=.false.
      normlog=.false.

*-----------------------------------------------------------------------
* print current settings
*-----------------------------------------------------------------------
100   if(valid) return
      write(*,*)
      write(*,*) ' Program settings menu:'
      write(*,*) ' ----------------------'
      write(*,*) ' Current settings:'
      write(*,'(1x,A,f14.8)') ' cstep    = ',cstep
      write(*,'(1x,A,i5)') ' numderiv = ',numderiv2
      write(*,'(1x,A,i5)') ' scfconv  = ',scfconv
      if(scfconv.lt.8) then
        call serror('menu',0,'scfconv value is critically small',6)
      end if
      write(*,'(1x,A,i5)') ' maxnbm   = ',maxnbm
      if(mode.eq.2) write(*,'(1x,A,i5)') ' rpaconv  = ',rpaconv
      write(*,'(1x,A,i5)') ' mtype    = ',masstype
C      write(*,'(1x,A,i5)') ' mode     = ',mode
      tag='off'
      if(tmpcl) tag='on '
      write(*,*) ' tmpcl    = '//tag
      tag='off'
      if(logcl) tag='on '
      write(*,*) ' logcl    = '//tag
      if(anffast) write(*,*) ' testrun chosen by user'

*-----------------------------------------------------------------------
* print possible commands
*-----------------------------------------------------------------------
      write(*,*) 
      write(*,*) ' Choose one of the following commands:'
      write(*,*) ' -------------------------------------'
      write(*,*) '  cstep <real>          : set cstep [<real> in bohr]'
      write(*,*) 
     * '  maxnbm <int>          : choose max. no. of basis modes'
      write(*,*) '  numderiv <int>        : choose <int> point central'
     * //' differences bickley formula'
      write(*,*) 
     * '  scfconv <int>         : choose scfconv parameter'
      write(*,*) '  tmpcl                 : switch on/off removal of'
     * //' temporary directories' 
      write(*,*) '  logcl                 : switch on/off removal of'
     * //' log directories' 
C      write(*,*) '  mode <int>            : choose mode no. <int>'
      write(*,*) '  mtype <int>           : choose type of masses to be'
     * //' used '
      write(*,*) '                          '//
     *'(1: most abundant isotopes, 2: average atomic masses,'
      write(*,*) '                          '//
     *' 3: most abundant isotopes, but deuterium mass for hydrogen)'
      write(*,*) '  <return>              : leave this menu'
      write(*,*)

*-----------------------------------------------------------------------
* read command
*-----------------------------------------------------------------------
      call clean(row)
      read(*,'(A)') row
      call slen(row,lrow)
      if(lrow.eq.0) then
        valid=.true.
      else if(row(1:3).eq.'rpa') then
        read(row(4:),*,err=100) rpaconv
        if(rpaconv.le.0) then
          write(*,*) ' rpaconv must be greater or equal to zero !'
          rpaconv=5
        else if(rpaconv.ge.10) then
          write(*,*) ' rpaconv value to large!'
          rpaconv=8
        end if
      else if(row(1:4).eq.'mode') then
        read(row(5:),*,err=100,end=100) mode
      else if(row(1:5).eq.'mtype') then
        read(row(6:),*,err=100,end=100) masstype
        if(masstype.lt.1.or.masstype.gt.3) masstype = 1
      else if(row(1:7).eq.'scfconv') then
        read(row(8:),*,err=100,end=100) scfconv
      else if(row(1:6).eq.'maxnbm') then
        oldmaxnbm=maxnbm
        read(row(7:),*,err=100,end=100) maxnbm
        if(maxnbm.gt.nvar) then
          write(*,*) ' maxnbm cannot exceed no. of degrees of freedom'
          maxnbm = nvar
        else if (maxnbm.lt.1) then
          write(*,*) ' so you don''t want to do a calculation at all ?'
          maxnbm=oldmaxnbm
        end if
      else if(row(1:5).eq.'cstep') then
        read(row(6:),*,err=100,end=100) cstep
      else if(row(1:8).eq.'numderiv') then
        read(row(9:),*,err=100,end=100) numderiv2
        if(mod(numderiv2,2).ne.1) then
          write(*,*) 'only odd numderiv numbers allowed !'
          numderiv2=numderiv*2+1
          goto 100
        end if
        numderiv=(numderiv2-1)/2
      else  if(row(1:4).eq.'proj') then
        if(proj) then
          proj=.false.
        else
          proj=.true.
        end if
      else  if(row(1:5).eq.'tmpcl') then
        if(tmpcl) then
          tmpcl=.false.
        else
          tmpcl=.true.
        end if
      else  if(row(1:5).eq.'logcl') then
        if(logcl) then
          logcl=.false.
        else
          logcl=.true.
        end if
      else  if(row(1:7).eq.'anffast') then
        if(anffast) then
          anffast=.false.
        else
          anffast=.true.
        end if
      else
        write(*,*) 'Unknown command'
      end if
      goto 100
      end

*******************************************************************************
  
      subroutine rootmenu(roothmpr,precond,apphess,genstart,minnbt,
     * goback)
*
* set type of root homing and preconditioner
* 
* written by J. Neugebauer, Uni Erlangen, 2003
*
      implicit none
      integer lrow,mxitem,minnbt
      parameter(mxitem=12)
      character*132 row
      character*132 item(mxitem)
      character*7 precond,roothmpr
      logical goback,valid,apphess,genstart

      precond='unitmat'
      if(apphess) precond='jacobid'
      valid=.false.
      goback=.false.

*-----------------------------------------------------------------------
* print current settings
*-----------------------------------------------------------------------
100   if(valid) return
      write(*,*)
      write(*,*) ' Root homing and preconditioning menu:'
      write(*,*) ' -------------------------------------'
      write(*,*) ' current settings:'
      if(roothmpr.eq.'trackgv') then
        write(*,'(1x,A)') ' root homing by overlap'
     &  //' with initial guess vector'
      else if(roothmpr.eq.'testmin') then
        write(*,'(1x,A)') ' no root homing; optimize lowest eigenvalue'
      else if(roothmpr.eq.'maxfreq') then
        write(*,'(1x,A)') ' no root homing; '
     *  //'optimize largest eigenvalues'
      else 
        write(*,'(1x,A)') ' root homing by overlap'
     &  //' with selected vector of previous iteration'
      end if
      write(*,'(1x,A)') ' preconditioning scheme: '//precond(1:7)
      if(precond.eq.'backtra') 
     &  write(*,'(1x,a,i7)') '    nbt                : ',minnbt
      if(genstart) write(*,'(1x,A)') 
     & ' Generation of start vectors selected => No root homing, '//
     & ' no preconditioning'
*-----------------------------------------------------------------------
* print possible commands
*-----------------------------------------------------------------------
      write(*,*) 
      write(*,*) ' choose one of the following commands:'
      write(*,*) '  root <string>     : set root homing scheme;'//
     * ' possible values:'
      write(*,*)   '                      lastsel = '//
     * 'default; overlap with last eigenvector'
      write(*,*)   '                      trackgv = '//
     * 'overlap with guess vector'
      write(*,*)   '                      minresi = '//
     * 'choose vector with minimal residuum'
      write(*,*)   '                      testmin = '//
     * 'test for minimum; optimize lowest root'
      write(*,*)   '                      maxfreq = '//
     * 'choose vectors with largest eigenvalues'
      write(*,*) '  prec <string>     : set preconditioner;'//
     * ' possible values:'
      write(*,*)   '                      backtra = '//
     * 'backtransformation of Davidson Matrix'
      if(apphess) then
        write(*,*) '                      jacobid = '//
     * 'Jacobi-Davidson with Hessian guess'
        write(*,*) '                      aphsinv = '//
     * 'Davidson with Hessian guess'
        write(*,*) '                      aphsdia = '//
     * 'Davidson with diagonal'
      end if
      write(*,*)   '                      unitmat = '//
     * 'Davidson with unit matrix'
      write(*,*)   '                      lanczos = '//
     * 'Lanczos type diagonalization'
      if(precond.eq.'backtra') then
        write(*,*) '  nbt  <int>        : min. no. of vectors'//
     *   ' for backtransformation'
      end if
      if(genstart) then
        write(*,*) '  genstart off      : switch off genstart option'
      else 
        write(*,*) '  genstart on       : only one iteration for '//
     *  'generation of start vectors'
      end if
      write(*,*) '  d                 : default settings'
      write(*,*) '  &                 : go back to convergence '//
     * 'criteria menu'
      write(*,*) '  <return>          : leave this menu'
      call clean(row)
      read(*,'(A)') row
      call slen(row,lrow)
      if(lrow.eq.0) then
        valid=.true.
      else if(row(1:4).eq.'prec') then
        call split(row,item,mxitem)
        read(item(2),*,end=100,err=100) precond
      else if(row(1:4).eq.'root') then
        call split(row,item,mxitem)
        read(item(2),*,end=100,err=100) roothmpr
      else if(row(1:3).eq.'nbt') then
        read(row(4:),*,err=100,end=100) minnbt
      else if(row(1:8).eq.'genstart') then
        if(index(row(9:),'on').ne.0) then
          genstart=.true.
        else if(index(row(9:),'off').ne.0) then
          genstart=.false.
        end if
      else if(row(1:1).eq.'d') then
        roothmpr='lastsel'
        precond='backtra'
        if(apphess) precond='jacobid'
      else if(row(1:1).eq.'&') then
        goback=.true.
        return
      end if
      goto 100
      end
*******************************************************************************
  
      subroutine convcmenu(rthres,rabsthres,dvthres,ecthres,rchthres,
     *  iorth,goback)
*
* set thresholds for convergence criteria
* 
* written by J. Neugebauer, Uni Erlangen, 2003
*
      implicit none
      integer lrow,iorth
      real*8 rthres,rabsthres,dvthres,ecthres,rchthres
      character*132 row
      logical goback,valid,lselect

      valid=.false.
      goback=.false.
      lselect=.false.

*-----------------------------------------------------------------------
* print current settings
*-----------------------------------------------------------------------
100   if(valid) return
      if(.not.lselect) rchthres=rthres*1.0d-4
      write(*,*)
      write(*,*) ' Convergence critera menu:'
      write(*,*) ' -------------------------'
      write(*,*) ' current thresholds:'
      write(*,'(1x,A,f14.8)') ' |max. component of residuum vector| : ',
     *  rthres
      write(*,'(1x,A,e14.8)') ' change of max. component of r       : ',
     *  rchthres
      if(rabsthres.gt.0.0d0) 
     *write(*,'(1x,A,f14.8)') ' norm of residuum vector/(3*natoms)  : ',
     * rabsthres
      if(dvthres.gt.0.0d0) 
     *write(*,'(1x,A,f14.8,a)') 
     *' change in wavenumber                : ',dvthres,' [1/cm]'
      if(ecthres.gt.0.0d0) 
     *write(*,'(1x,A,f14.8)') ' expansion coef. of last basis vector: ',
     * ecthres
      write(*,'(1x,A,i14)')   ' orthonormalization parameter        : ',
     * iorth
*-----------------------------------------------------------------------
* print possible commands
*-----------------------------------------------------------------------
      write(*,*) 
      write(*,*) ' choose one of the following commands:'
      write(*,*) '  rthres <real>     : set threshold for max. '//
     *  'component of residuum vector'
      write(*,*) '  rabsthres <real>  : set threshold for norm '//
     *  'of residuum vector/(3*natoms)'
      write(*,*) '  dvthres <real>    : set threshold for change '//
     *  'in wavenumber [1/cm]'
      write(*,*) '  ecthres <real>    : set threshold for expansion'//
     *  ' coef. of last basis vector'
      write(*,*) '  rchthres <real>   : set threshold for change of'//
     *  ' max. component of r'
      write(*,*) '  iortho <int>      : set orthogonalization parameter'
      write(*,*) '  d                 : default settings'
      write(*,*) '  &                 : go back to program settings'//
     *  ' menu'
      write(*,*) '  <return>          : leave this menu'
      call clean(row)
      read(*,'(A)') row
      call slen(row,lrow)
      if(lrow.eq.0) then
        valid=.true.
      else if(row(1:6).eq.'iortho') then
        read(row(7:),*,err=100,end=100) iorth
      else if(row(1:6).eq.'rthres') then
        read(row(7:),*,err=100,end=100) rthres
      else if(row(1:9).eq.'rabsthres') then
        read(row(10:),*,err=100,end=100) rabsthres
      else if(row(1:8).eq.'rchthres') then
        read(row(9:),*,err=100,end=100) rchthres
        lselect=.true.
      else if(row(1:7).eq.'dvthres') then
        read(row(8:),*,err=100,end=100) dvthres
      else if(row(1:7).eq.'ecthres') then
        read(row(8:),*,err=100,end=100) ecthres
      else if(row(1:1).eq.'d') then
        rthres=0.0005d0
        rabsthres=-1.0d0
        dvthres=-1.0d0
        ecthres=-1.0d0
        lselect=.false.
      else if(row(1:1).eq.'&') then
        goback=.true.
        return
      end if
      goto 100
      end

*==================================================================

      subroutine outmenu(g98bs,g98it,tmout,xmolout,snfout,printall,
     *                   goback)
*
* set output options
* 
* written by J. Neugebauer, Uni Erlangen, 2003
c modified by C. Herrmann, Uni Jena, 2005
*
      implicit none
      integer lrow
      character*132 row
      character*3 tag
      logical goback,valid,g98bs,g98it,tmout,xmolout,snfout,printall

      valid=.false.
      goback=.false.

*-----------------------------------------------------------------------
* print current settings
*-----------------------------------------------------------------------
100   if(valid) return
      write(*,*)
      write(*,*) ' Output menu:'
      write(*,*) ' ------------'
*-----------------------------------------------------------------------
* print possible commands
*-----------------------------------------------------------------------
      write(*,*) ' choose one of the following commands:'
      tag='on '
      if(g98it) tag='off'
      write(*,*) '  g98it '//tag//'         : switch '//tag
     * //' g98 normal mode output'
      tag='on '
      if(g98bs) tag='off'
      write(*,*) '  g98bs '//tag//'         : switch '//tag
     * //' g98 basis vector output'
      tag='on '
      if(tmout) tag='off'
      write(*,*) '  tmout '//tag//'         : switch '//tag
     * //' TM output of final normal modes'
      tag='on '
      if(xmolout) tag='off'
      write(*,*) '  xmout '//tag//'         : switch '//tag
     * //' xmol output of final normal modes'
      tag='on '
      if(snfout) tag='off'
      write(*,*) '  snfout '//tag//'        : switch '//tag
     * //' SNF output of final normal modes'
      tag='on '
      if(printall) tag='off'
      write(*,*) '  prall '//tag//'         : switch '//tag
     * //' detailled output of subspace iteration'
      write(*,*) '  d                 : default settings'
      write(*,*) '  &                 : go back to program settings'//
     *  ' menu'
      write(*,*) '  <return>          : leave this menu'
      call clean(row)
      read(*,'(A)') row
      call slen(row,lrow)
      if(lrow.eq.0) then
        valid=.true.
      else if(row(1:5).eq.'xmout') then
        if(index(row(6:),'on').ne.0) then
          xmolout=.true.
        else if(index(row(6:),'off').ne.0) then
          xmolout=.false.
        end if
      else if(row(1:6).eq.'snfout') then
        if(index(row(7:),'on').ne.0) then
          snfout=.true.
        else if(index(row(7:),'off').ne.0) then
          snfout=.false.
        end if
      else if(row(1:5).eq.'prall') then
        if(index(row(6:),'on').ne.0) then
          printall=.true.
        else if(index(row(6:),'off').ne.0) then
          printall=.false.
        end if
      else if(row(1:5).eq.'tmout') then
        if(index(row(6:),'on').ne.0) then
          tmout=.true.
        else if(index(row(6:),'off').ne.0) then
          tmout=.false.
        end if
      else if(row(1:5).eq.'g98it') then
        if(index(row(6:),'on').ne.0) then
          g98it=.true.
        else if(index(row(6:),'off').ne.0) then
          g98it=.false.
        end if
      else if(row(1:5).eq.'g98bs') then
        if(index(row(6:),'on').ne.0) then
          g98bs=.true.
        else if(index(row(6:),'off').ne.0) then
          g98bs=.false.
        end if
      else if(row(1:1).eq.'d') then
        g98it=.true.
        g98bs=.true.
        tmout=.false.
        xmolout=.false.
        snfout=.false.
        printall=.false.
      else if(row(1:1).eq.'&') then
        goback=.true.
        return
      end if
      goto 100
      end
