      SUBROUTINE basic_input

!*************************************************
! akira version by J. Neugebauer, Uni Erlangen, 2001
! sligthly adapted for excited state gradients by
! C. Herrmann, Uni Jena 04/2005
!*************************************************

      use CoordModule
#include "defs.h"
#include "param.h"

      integer i,nn
      character*255 b1
      real*8 charge
      character*10 calctype,progtype
      integer iclean,icleanl
      logical valmod,normlog
      external valmod

!---------------------------------------------------
! read control file
!---------------------------------------------------

       call testfile('akira_control')
       call rakiractrl('akira_control',rthres,rabsthres,dvthres,ecthres,
     *  roothmpr,g98it,g98bs,xmolout,tmout,printall,precond,genstart,
     *  rchthres,minnbt,iorth,snfout,natoms,mode,cstep,naus,iclean,
     *  icleanl,idbg,ipol,lastqmatom,irescp,restadf,sleeptime)
      if(mode.lt.20.and.mode.gt.0) then
       call rcontrol('control',rpast,smult,natoms,jscfc,
     * isuhf,riflag) 
      else if(mode.eq.60) then
       call rcontrol('control',rpast,smult,lastqmatom,jscfc,
     * isuhf,riflag)
      end if
      if (.not.valmod(mode)) then
        write(jtape,*) 'basic_input: mode no. does not exist: ',mode
        stop
      end if

!---------------------------------------------------
! set okok: one '1' for each input file to be copied
! (synchronize with islave.r)
! 1) akira_control, coord, program-specific input file
!---------------------------------------------------
      
      okok = 111

!---------------------------------------------------------
! 2) additional program- and run-type specific input files
!---------------------------------------------------------
      if(mode.gt.0.and.mode.lt.20.or.mode.eq.60) then

! TM: control file
        okok = 10*okok+1

! TM: mos or alpha (basis is counted above)
        okok = 10*okok+1

! TM: auxbasis
        if(riflag.eq.1) okok = 10*okok+1

! TM: beta
        if(isuhf.gt.0) okok = 10*okok+1

      else if(mode.ge.40.and.mode.lt.50) then

! ADF: restart-file
         if(irescp.gt.0) okok = 10*okok+1

CCH* ADF: mm.restart
CCH         if(lastqmatom.ne.natoms) okok = 10*okok+1

! COMQUM: various files
      end if
      if(mode.eq.60) then
        okok2=111111111
      end if

      ipol=0
      if(mode.eq.2) then
        ipol=1
        if(iomeg.ne.0) ipol=iomeg
      end if
      IF(iclean.eq.1) 
     *WRITE(*,'('' remove directories for single point calculations'')')
      IF(icleanl.eq.1) 
     *WRITE(*,'('' remove log files from slave processes'')')
      WRITE (*,'('' step sizes (bohr)'',f10.5)') cstep
      IF (icrap.EQ.0 .AND. jscfc.LT.iscfc.and.mode.lt.20) THEN
       WRITE (jtape,*)
     *  'The present parameter setting of AKIRA enforces'
       WRITE (jtape,*)
     *  'a higher accuracy for dscf/ridft than is provided'
       WRITE (jtape,*) 'in the current <control> file:'
       WRITE (jtape,*) 'SCFCONV too small on <control>'
       WRITE (jtape,*)
     *  'it is ',jscfc,', and it should be at least ',iscfc
       WRITE (jtape,*)
     *  'set "CRAP_OK yes" in the AKIRA input, if lower accuracy'
       WRITE (jtape,*)
     *  'is OK; note that this may give much less accurate results'
       WRITE (jtape,*) 'for frequency and intensities.'
       CALL errex('SCFCONV too small')
      ENDIF
      iscfc=jscfc
      IF (icrap.EQ.0 .AND. (cstep.LT.0.01.OR.cstep.GT.0.05)) THEN
       WRITE (jtape,*)
     *  'The present parameter setting of AKIRA enforces'
       WRITE (jtape,*) 'a cstep value 0.01 <= cstep <= 0.05.'
       WRITE (jtape,*) '$cstep has been given as ',cstep
       WRITE (jtape,*)
     *  'set CRAP_OK in the AKIRA input, if this is OK'
       WRITE (jtape,*)
     *  'note that this may give much less accurate results'
       WRITE (jtape,*) 'for frequency and intensities.'
       CALL errex('SCFCONV too small')
      ENDIF
      
!* read coordinates

      allocate(icharge(natoms),coord(3,natoms))
      normlog=.false.
      call rkeywd('akira_control','$vibrational basis',normlog)
      if(.not.normlog) then
        call errex('basis modes not present in control')
      end if
      if(mode.ne.60) then
        call rcoord('coord',nn,coord,icharge)
      else
        call readcomqum('prmcrd3','prmtop3',natoms,coord,icharge,
     *  maxatoms) 
      end if

      if (nn.ne.natoms.and.mode.ne.60) then
       call errex('natoms from control and coord differ!')
      endif

      call prog_type(mode,progtype,calctype)
      write (jtape,*) 
      write (jtape,'('' AKIRA: NUMBER OF ATOMS      '',i5)') natoms
      write (jtape,'('' degrees of freedom          '',i5)') natoms*3
      write (jtape,'('' program for single points   '',a8)')
     *  progtype
      write (jtape,'('' type of calculations        '',a8)')
     *  calctype
      write (jtape,'('' SCF convergence parameter   '',i5)') jscfc
      write (jtape,*) ' '
      write (jtape,*)
     *  '        ORIENTATION OF MOLECULE IN FORCE CALCULATION'
      write (jtape,*) ' '
      write (jtape,*)
     *  '   NO.       ATOM         X         Y         Z'
      write (jtape,*) ' '
      call wcoo(natoms,icharge,coord)

      nvar  =natoms*3
      nstept=nvar*maxnbm
      return
      end

c***************************************************************

      subroutine wcoo(natoms,ilab,co)
#include "fconstants.h"
      integer natoms
      integer ilab(natoms)
      real*8 co(3,natoms)
      integer i
      do i=1,natoms
       write (*,110) i,ilab(i),
     *  co(1,i)*bohr2ang,co(2,i)*bohr2ang,co(3,i)*bohr2ang
110    format(i6,i10,4x,3f10.4)
      end do
      return
      end
