      subroutine checkadfinput(coord,el)
c
c F77 version of check_adf_input in SNF program package
c (see www.reiher.ethz.ch/software/snf)
c
c  - read adf.in and check whether SCF-convergence criteria and
c    integration grid are sufficient. 
c  - write block requesting geometry optimization which is stopped
c    after one iteration (if not present yet)    
c
c  written by C. Herrmann, Uni Jena 06/2005
c  modified by J. Neugebauer, 10/2005:
c
c  - include RESPONSE block at this stage in case of Raman calculations
c  - check for SYMMETRY key and replace it by SYMMETRY NOSYM
c  - check for any STOPAFTER keys and replace them by STOPAFTER GGRADS
c  - check for settings that are important for well-converged SCF:
c    converge, occupations keeporbitals, occupations smearq
c
c  removed check for length units, since this will be done in 
c  routine radfcoord of src/snfdc/rwadf.r anyway (CH 24.04.07) 
c--------------------------------------------------------------------

#include "param.h"
#include "fconstants.h"
    
      integer :: ir1,mxbuf,i,ilast,ihalt,iatomstart,iatomend
      integer :: lscr,j,icount,ii
      parameter(mxbuf=10000)
      character(len=132) :: buf(mxbuf),item(mxitem),scr                 
      character(len=2) :: el(maxatoms)
      real*8 :: rint1,rint2,rconv1,rconv2,string2real,coord(3,maxatoms)
      logical :: intchanged,intfound,scffound,convchanged,geofound
      logical :: qmmmoptfound
      logical :: occfound
      integer :: lbuf(mxbuf)

      ir1=10
      intfound=.false.
      intchanged=.false.
      scffound=.false.
      geofound=.false.
      convchanged=.false.
      occfound=.false.
      qmmmoptfound=.false.
       open(unit=ir1,file='adf.in')
       i=1
10       read (ir1,'(a)',ERR=900,END=900) buf(i)
         call llen(buf(i),lbuf(i))
         call split(buf(i),item,mxitem)
         call upper(item(1))
         call upper(item(2))
c skip comments
         if(index(buf(i),'::').ne.0) then
           goto 100
         else if(((item(1)(1:3).eq.'END').and.(item(2)(1:5).eq.'INPUT'))
     *       .or.item(1)(1:8).eq.'ENDINPUT') then
             ihalt=i
         else if(item(1)(1:11).eq.'INTEGRATION') then
             goto 20
         else if(item(1)(1:3).eq.'SCF') then 
             goto 30
         else if(item(1)(1:8).eq.'GEOMETRY') then 
             goto 40
         else if(item(1).eq.'OCCUPATIONS') then
             write(buf(i),'(a)') 'OCCUPATIONS  KEEPORBITALS=1000000 '//
     *       'SMEARQ=0.0d0'
             occfound=.true.
!------------------------------------------------------------------
! QM/MM: check for  QMMM-specific optimization block and
!        insert/replace options for QM/MM-gradient-only calculation
! attention: problems if OPTIMIZE subblock contains already
! other subblocks such as CONJGRAD => will write one subend too much
! any idea how to fox this? (CH 01.06.2007)
!------------------------------------------------------------------
        else if(item(1)(1:4).eq.'QMMM') then
         i=i+1
4        read (ir1,'(a)',ERR=900,END=900) buf(i)
           call llen(buf(i),lbuf(i))
           call split(buf(i),item,mxitem)
           call upper(item(1))
            if(index(item(1),'OPTIMIZE').ne.0) then
              qmmmoptfound=.true.
              scr=(buf(i))
              call upper(scr)
              ii=index(scr,'OPTIMIZE')
5             read (ir1,'(a)',ERR=900,END=900) scr
                call upper(scr)
                if(index(scr,'SUBEND').ne.0) then
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii+2:)='METHOD CONJGRAD'
                  call llen(buf(i),lbuf(i))
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii+2:)='MM_NOTCONVERGED 0'
                  call llen(buf(i),lbuf(i))
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii+2:)='MAX_STEPS 1'
                  call llen(buf(i),lbuf(i))
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii+2:)='CONJGRAD'
                  call llen(buf(i),lbuf(i))
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii+2:)='  MAX_ESTEPS 1'
                  call llen(buf(i),lbuf(i))
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii+2:)='  MAXSTEP 0.0d0'
                  call llen(buf(i),lbuf(i))
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii+2:)='SUBEND'
                  call llen(buf(i),lbuf(i))
                  i=i+1
                  call clean(buf(i))
                  buf(i)(ii:)='SUBEND'
                  call llen(buf(i),lbuf(i))
                  goto 15
                end if
                goto 5
15              continue
            end if
            if(index(buf(i),'END').ne.0.and.index(buf(i),'SUBEND').eq.0)
CCH still to come: insert QMMM-Optimize subblock if not found in QMMM block
CCH (not so important, since it should exist from preceding geometry
CCH  optimization anyway).
CCH               if(.not.qmmmoptfound) ...
     *         goto 14
            i=i+1
          goto 4
14        continue
c---------------------------------------------------------------
c skip UNITS block - coordinates will be written in ADF's
c standard unit Angstrom anyway.
c---------------------------------------------------------------
         else if(item(1)(1:5).eq.'UNITS') then
           i=i-1
3            read(ir1,'(a)',ERR=900,END=900) scr
             call llen(scr,lscr)
             call split(scr,item,mxitem)
             call upper(item(1))
             if(item(1)(1:3).eq.'END') goto 13
           goto 3
13         continue
         else if(item(1)(1:5).eq.'ATOMS') then
c----------------------------------------------------------------
c just skip ATOMS block. The new ATOMS block will be added below
c----------------------------------------------------------------
           i=i-1
1            read(ir1,'(a)',ERR=900,END=900) scr
             call llen(scr,lscr)
             call split(scr,item,mxitem)
             call upper(item(1))
             if(item(1)(1:3).eq.'END') goto 11
           goto 1
11         continue
         else if(item(1)(1:8).eq.'RESPONSE') then
c----------------------------------------------------------------
c just skip response block: either, it is not needed in this
c calculation, or it will be added by snfdefine below
c----------------------------------------------------------------
           i=i-1
2            read(ir1,'(a)',ERR=900,END=900) scr
             call llen(scr,lscr)
             call split(scr,item,mxitem)
             call upper(item(1))
             if(item(1)(1:3).eq.'END') goto 12
           goto 2
12         continue
         else if(item(1)(1:8).eq.'SYMMETRY') then
c----------------------------------------------------------------
c skip SYMMETRY key
c----------------------------------------------------------------
           i=i-1
         else if(item(1)(1:9).eq.'STOPAFTER') then
c----------------------------------------------------------------
c skip STOPAFTER keys
c----------------------------------------------------------------
           i=i-1
         end if
        goto 100
c----------------------------
20      rint1=string2real(item(2))
        rint2=string2real(item(3))
       if(rint1.lt.6.0d0) then
         rint1 = 6.0d0 
         intchanged=.true.
       else if(rint2.lt.6.0d0) then
         rint2 = 6.0d0
         intchanged=.true.
       end if
       write(buf(i),'(a12,2(f3.1,2x))') 'INTEGRATION ', rint1,rint2
       call llen(buf(i),lbuf(i))
        intfound=.true.
           goto 100
c----------------------------
30    i=i+1
       scffound=.true.
31    read(ir1,'(a)',ERR=900,END=900) buf(i)
      call llen(buf(i),lbuf(i))
      call split(buf(i),item,mxitem)
      call upper(item(1))
      if(item(1)(1:3).eq.'END') goto 100 
      if(item(1)(1:8).eq.'CONVERGE') then
         read(item(2),*) rconv1
         read(item(3),*) rconv2
        if(rconv1.gt.1.0d-6) then
          item(2)(1:6) = '1.0e-6'
          convchanged=.true.
        else if(rconv2.gt.1.0d-6) then
          item(3)(1:6) = '1.0e-6'
          convchanged=.true.
        end if
        write(buf(i),'(a10,2x,a6,2x,a6)') 
     *     ' converge ', item(2)(1:6),item(3)(1:6) 
        call llen(buf(i),lbuf(i))
      end if
      i=i+1
      goto 31
c----------------------------
40    read(ir1,'(a)',ERR=900,END=900) scr
      call split(scr,item,mxitem)
      if(item(1)(1:3).eq.'end'.or.item(1)(1:3).eq.'END'.or.
     *   item(1)(1:3).eq.'End') then
         buf(i) =  'Geometry'
         call llen(buf(i),lbuf(i))
         buf(i+1) = '  GO'
         call llen(buf(i+1),lbuf(i+1))
         buf(i+2) = '  iterations 1'
         call llen(buf(i+2),lbuf(i+2))
         buf(i+3) = 'End'  
         call llen(buf(i+3),lbuf(i+3))
         i=i+3
         geofound=.true.
         goto 100
      end if
      goto 40
100   i=i+1
      goto 10
900   ilast=i-1
      rewind(ir1)
c-----------------------------------------
c skip empty lines at the end of the input
c-----------------------------------------
      icount=0
      do i=ihalt-1,1,-1
        if(len_trim(buf(i)).ne.0) exit
        icount=icount+1
      end do
      do i = 1,ihalt-1-icount
        call llen(buf(i),lbuf(i))
        write(ir1,'(a)') buf(i)(1:lbuf(i))
      end do
      write(ir1,'(a)')
      write(ir1,'(a)') 'SYMMETRY NOSYM'
      write(ir1,'(a)')
      write(ir1,'(a)') 'ATOMS'
111   format (1x,a2,4x,3(f20.14,2x))
      do i=1,natoms
        write(ir1,111) el(i),(coord(j,i)*bohr2ang,j=1,3)
      end do
      write(ir1,'(a)') 'END'
      write(ir1,'(a)')
      write(ir1,'(a)') 'STOPAFTER GGRADS'
      write(ir1,'(a)')
      if(.not.intfound) then
        write(ir1,'(a12,2(f3.1,2x))') 'INTEGRATION ', rint1,rint2
        intchanged=.true.
      end if
      if(.not.scffound) then 
        write(ir1,'(a)')
        write(ir1,'(a)') 'SCF'
        write(ir1,'(a)') '  converge 1.0e-6 1.0e-6'
        write(ir1,'(a)') 'End' 
        write(ir1,'(a)')
        convchanged=.true.
      end if
      if(.not.geofound) then
        write(ir1,'(a)')
        write(ir1,'(a)') 'Geometry'
        write(ir1,'(a)') '  GO'
        write(ir1,'(a)') '  iterations 1'
        write(ir1,'(a)') 'End'
        write(ir1,'(a)')
      end if
      if(.not.occfound) then
        write(ir1,'(a)')
        write(ir1,'(a)') 'OCCUPATIONS KEEPORBITALS=1000000 SMEARQ=0.0d0'
        write(ir1,'(a)')
      end if
      do i = ihalt,ilast
        write(ir1,'(a)') buf(i)(1:lbuf(i))
      end do
      close(ir1)
      if(intchanged) then
        write(*,*) 
        write(*,*) '***************************************************'
        write(*,*) ' WARNING: The integration grid found in adf.in '
        write(*,*) ' is not sufficiently fine. It has been changed to' 
        write(*,'(a14,2(f3.1,2x))') '  INTEGRATION ', rint1,rint2 
        write(*,*) '***************************************************'
        write(*,*)
      end if
      if(convchanged) then
        write(*,*)
        write(*,*) '***************************************************'
        write(*,*) 
     * ' WARNING: The scf convergence criteria found in adf.in'
        write(*,*) ' are too loose. They have been changed to '
        write(*,'(a)') '  converge 1.0e-6 1.0e-6 '
        write(*,*) '***************************************************'
        write(*,*)
      end if
      if(intchanged.or.convchanged) then
        write(*,*) ' It is recommended to redo a geometry optimization '
        write(*,*) ' using the new input-settings '
        write(*,*)
        write(*,*) ' Please type <Enter> to confirm'
        write(*,*)
        read(*,'(a)') scr
      end if
      end subroutine

