/***************************************************************************
 replacement for system call with enhanced functionality:
 The called task is periodically checked for its state; start of task,
 end of task, uptime record and ps record for the task are communicated
 to the task that called fsystem by means of a call to MNOTIFY
 29.05.2008 make mkstemp work correctly and do not leave file open (St. Schenk)
 15.05.2007 change of mkstemp call (J. Neugebauer and St. Schenk)
 15.01.2007 gcc-4.x compatibility (J. Neugebauer and St. Schenk)
 060801 - IRIX
 190501 - stat
 280101 - SR8000
 BAH, May 16, 1999
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_TIME_H
#  include <time.h>
#endif

#ifdef HAVE_SIGNAL_H
#  include <signal.h>
#endif

#ifdef HAVE_STDIO_H
#  include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
#  if !defined STDC_HEADERS && defined HAVE_MEMORY_H
#    include <memory.h>
#  endif
#  include <string.h>
#endif

#ifdef HAVE_ERRNO_H
#  include <errno.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#ifdef HAVE_SYS_WAIT_H
#  include <sys/wait.h>
#endif

#include "fsystem.h"
#include "mnotify.h"

#ifdef __cplusplus
extern "C" { /* prevent C++ name mangling */
#endif

/*
 * global variables
 */

pid_t child_pid;	/* pid of child task */
char command[256];	/* command to be executed */

/* the following variables are set by archive() */
time_t timestamp;	/* MO file time stamp */

/* the following variables are set by archiving() */
char tmpdi[256];	/* temporary directory */
char file1[256];	/* mo filename ('mos','alpha')*/
char file2[256];	/* mo filename ('beta') */
char apath[256];	/* archive path */
int astepno = 0;	/* archive step number */
int archno = 0;		/* number of files to archive */



/*
 * fsystem_trap()
 */

RETSIGTYPE fsystem_trap( int n)
{
  /* printf("fsystem_trap %d child_pid=%d\n",n, child_pid); */
  chk_task( child_pid);
#ifndef SNF_PS_NOTRAP
  if ( signal( SIGALRM, fsystem_trap) == SIG_ERR) {
    fprintf( stderr, "fsystem_trap: cannot establish timer trap\n");
    perror( "establishing timer trap");
    exit(7);
  }
#endif

  alarm( SNF_PS_INTERVAL); /* set up new alarm */
}



/*
 * termpid() - terminates child task; called by slshutdown and master
 */

int TERMPID_FC (pid)
  int *pid;
{
  int p;
  p = *pid;
  if (p == 0) return( kill( child_pid, SIGTERM));
  return( kill( p, SIGTERM));
}



/*
 * fsystem()
 */

int FSYSTEM_FC (l,x)
  char*x;
  int *l;
{
  int one=1, two=2;

  int status, len;
  pid_t pid;

  len = *l;

  if (len > 255) {
    fprintf( stderr, "fsystem: command length (%d) exceeds maximum of 255", len);
    exit(9);
  }

  strncpy( command, x, len);
  command[len] = '\0';

  pid = fork();
  if (pid == -1) return -1;

  /* the child process */
  if (pid == 0) {
    char *argv[4];
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    execv( "/bin/sh", argv);
    exit(127);
  }

  status=0;
  MNOTIFY_FC( &one, &pid, &status, x, l); /* notify calling task */
  child_pid = pid;                         /* store child pid globally */

  if (signal( SIGALRM, fsystem_trap) == SIG_ERR) {
    fprintf( stderr, "fsystem: cannot establish timer trap\n");
    perror( "establishing timer trap");
    exit(8);
  }

  alarm( SNF_PS_INTERVAL);

  do {
    if (waitpid( pid, &status, 0) == -1) {
      if (errno != EINTR) return -1;
    } else {
      MNOTIFY_FC( &two, &pid, &status, x, l); /* notify calling task */
      return status;
    }
  } while(1);
}



/*
 * chk_task()
 */

int chk_task( ppp)
  int ppp;
{
#ifndef SNF_SMP
  char cmd[512];
  int pid, status, len;
  FILE *f;
  char buf[255];
  char filename[255];
  int three=3, four=4;
  char *p,*q;
  int fd;

  strcpy( filename, "/tmp/akira_XXXXXX");
  fd = mkstemp( filename);
  if (fd == -1) {
    fprintf( stderr, "fsystem: cannot create temporary file");
/*
 * 2008/05/14 (St. Schenk)
 * It is unclear to me what we should return in this case. Clearly, its
 * a problem with the system like failure of fork() so we can return -1.
 * On the other hand, below we exit(7) (EXIT!) the whole program if the
 * file can not be read.
 * I vote for returning -1 since mkstemp() may still succeed later on.
 */
    return -1;
  }
  close( fd);

/* ps command and CPU reporting */

  sprintf( cmd, "uptime > %s; ps %s | awk '{if (%s==\"%d\") print $0}' >> %s",
    filename, SNF_PS_OPTIONS, SNF_PS_PPID, ppp, filename);
  pid = fork();
  if (pid == -1) return -1;

/* the child process */
  if (pid == 0) {
    char *argv[4];
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = cmd;
    argv[3] = 0;
    execv("/bin/sh", argv);
    exit(127);
  }

  status=0;
  do {
    if (waitpid( pid, &status, 0) == -1) {
      if (errno != EINTR) return -1;
    } else {

/* give data back to calling task */
      pid = ppp;
      f = fopen( filename, "r");
      if (!f) {
        fprintf( stderr, "fsystem: cannot open file %s\n", filename);
        exit(7);
      }

      fgets( buf, 255, f);
      p = buf;
/* load average */
      while (*p && strncmp( p, "age:", 4)) p++;
      if (*p) {
        p += 5;
        while (*p && *p == ' ') p++;
        q = p;
        while (*q && *q != ',') q++;
        *q = '\0';
        len = strlen( p);
        MNOTIFY_FC( &three, &pid, &status, p, &len);
      }

/* cpu time */
      fgets( buf, 255, f);
      p = buf;
      while (*p && *p != ':') p++;
      if (*p) {
        while (*p != ' ') p--;
        len = strlen(p);
        MNOTIFY_FC( &four, &pid, &status, p, &len);
      }

      fclose( f);
      unlink( filename);
      break;
     }
   } while(1); /* end of wait loop for uptime command */
#endif

   return 0;
}



