#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      SUBROUTINE getres(gra,stepfl,vmach,escf,pol,nbm,machnumb)

* =====================================================
* akira version by J. Neugebauer, Uni Erlangen, 2001
* =====================================================
* adapted to Gaussian and ADF interface by C. Herrmann, 
*            Uni Bonn, 2004-05
* modified by C. Herrmann, Uni Jena 02.05.2005 
* (exc. state gradients)
*
* migrate to MachineModule, J. Neugebauer, 2007
* =====================================================
     
      use MachineModule

*
* get results for node i
* dictionary of nodefl:
*                        -2          waiting to be spawned
*                        -1          terminated
*                         0          not used
*                         1          working
*                         2          waiting for work

#include "defs.h"
#include "param.h"

#ifdef AKIRA_PVM
#include "fpvm3.h"
#endif

#ifdef AKIRA_MPI
#include "mpif.h"
#endif

      type(type_machine), pointer  :: mach,cmach
      integer stepfl(*)
      integer ibuf,m,info,icmp,iaus,now,nse,tid,ia,ib,i,j,ll1,ll2
      integer ll3,jomeg,xeq,nbm,machnumb
      integer*4 nit
      character*24 date
      character*40 msg,name,myname
      character*255 b1,stempdir
      real*8, allocatable :: ggrad(:)
      real*8 gra(nvar,naus,maxnbm)
      real*8 escf(nstept+1),pol(mxomeg,8,naus,maxnbm)
      real*8 e,d(3),po(8,mxomeg)
      real*8 cpu,wall,gcpu,gwall
      real*8 gsum,se,vmach(*)

#ifdef AKIRA_MPI
      integer st(MPI_STATUS_SIZE)
      integer w
      logical flag
      data w /MPI_COMM_WORLD/
#endif

      allocate(ggrad(nvar))

*     ------------------------------------------------------------
*     set current task id and check whether results are available 
*     from this task ("tid"); if not, then return
*     ------------------------------------------------------------
      call findnode_by_num(mach,machnumb)
      tid=mach%tid

#ifdef AKIRA_PVM
      call pvmftrecv(tid,0,1,0,ibuf)
      if (ibuf.eq.0) return
#endif

#ifdef AKIRA_MPI
      call mpi_iprobe(tid,fsys_restag,w,flag,st,info)
      call mpierr('mpi_probe','getres',info)
      if (.not.flag) return
#endif

      mach%nodf='chk'
      call getmach_name(mach,name)

#ifdef AKIRA_MPI
       call mpi_recv(nit,1,mpi_integer,tid,fsys_restag,w,st,info)
       call mpierr('mpi_recv nit','getres',info)
       call mpi_recv(cpu,1,mpi_double_precision,tid,801,w,st,info)
       call mpierr('mpi_recv cpu','getres',info)
       call mpi_recv(wall,1,mpi_double_precision,tid,802,w,st,info)
       call mpierr('mpi_recv wall','getres',info)
       call mpi_recv(gcpu,1,mpi_double_precision,tid,803,w,st,info)
       call mpierr('mpi_recv gcpu','getres',info)
       call mpi_recv(gwall,1,mpi_double_precision,tid,804,w,st,info)
       call mpierr('mpi_recv gwall','getres',info)
       call mpi_recv(e,1,mpi_double_precision,tid,805,w,st,info)
       call mpierr('mpi_recv escf','getres',info)
       call mpi_recv(ggrad,nvar,mpi_double_precision,tid,807,w,st,info)
       call mpierr('mpi_recv grad','getres',info)
#endif

#ifdef AKIRA_PVM
      call pvmfunpack(integer4,nit,1,1,info)
      call pvmfunpack(real8,cpu,1,1,info)
      call pvmfunpack(real8,wall,1,1,info)
      call pvmfunpack(real8,gcpu,1,1,info)
      call pvmfunpack(real8,gwall,1,1,info)
      call pvmfunpack(real8,e,1,1,info)
      call pvmfunpack(real8,ggrad,nvar,1,info)
#endif


#ifdef DEBUG
      write (jtape,'(4d18.8)') e
      write (jtape,*) ' '
      write (jtape,'(4d18.8)') (ggrad(j),j=1,nvar)
      write (jtape,*) ' '
#endif
            
      gsum=0.0d0
      do j=1,nvar
        gsum=gsum+ggrad(j)**2
      end do
      se=wall+gwall
      nse=se
      call secs(now)
      if (mach%last > 0) then
        i=mach%time+mach%last
        write (idbg,*) 'getres: time,last: ',mach%time,mach%last
        write (idbg,*) 'getres: now ',now,
     *                 ' node ',mach%numb,
     *                 ' expected ',i,
     *                 ' used     ',nse,
     *                 ' on ',name(1:8),'[D=',i-now,']'
      end if
      mach%last = now-mach%time
      mach%time = 0
      ia=mach%stpa
      ib=mach%stpb
       
      write(*,'(A)')       '----------------------------------------'
      write(*,'(A,i15)')   'STEP NO.            : ',ia
      write(*,'(A,f15.2)') 'CPU TIME            : ',cpu+gcpu
      if(mode.lt.30) then
        write(*,'(A,f15.2)') 'WALL TIME           : ',wall+gwall
      end if
      write(*,'(A,i15)')   'NO. OF ITERATIONS   : ',nit
      if(mode.ge.40.and.mode.lt.50) then
        write(*,'(A,f15.8)') 'BONDING ENERGY      : ',e
      else 
        write(*,'(A,f15.8)') 'ELEC. ENERGY        : ',e
      end if
      write(*,'(A)')       '----------------------------------------'


*     -------------------------------------------------------------
*     JN: removed gsum check for gsum.le.1.0d-16 (gsum is 0.0d0 for
*         certain modes
*     CH: removed energy check for excited state calculations 
*     (energy > 0 is acceptable, since reference is ground state)  
*     CH271106: energy > 0 acceptable for QM/MM calculations,
*     since only MM energy can be read out, which may be positive.
*     -------------------------------------------------------------

CJN      if (gsum.gt.1.or.e.ge.0.d0) then
      if (e.ge.0.d0.and.rpast.eq.0) then
        if (e.ge.0.d0.and.mode.ne.41) then
          write (msg,'(a20,d20.10)') ' invalid energy:',e
          call remnode(msg,mach_fai1,stepfl,mach%numb)
        else if(e.eq.0.d0.and.mode.eq.41) then
          write (msg,'(a20,d20.10)') ' invalid energy:',e
          call remnode(msg,mach_fai1,stepfl,mach%numb)
CJN          write (msg,'(a20,d20.10)') ' warning! invalid gsum:',gsum
CJN          call remnode(msg,mach_fai2,stepfl,mach%numb)
        endif
       
*       -----------------------
*       shut down the process
*       -----------------------
        call remnode('wrong results',mach_term,stepfl,mach%numb)
        return
      else
        if(gsum.gt.1.0d0) then
          write(*,*) 'WARNING! invalid gsum: ',gsum
          write(*,*) 
     &      'unless you have specified a very large cstep value'
          write(*,*) 'this is a reason to worry'
        end if
        call fdate(date)
        write (jtape,109)
     &     date,'<- ',tid,name(1:10),ia,
     &     nit,cpu,wall,gcpu,gwall
109       format(a,a,i10,' ',a,' return  step ',i3/,
     &     (30x,i2,'i',f10.1,'c',f10.1,'w',f10.1,'c',f10.1,'w'))
        call flush(jtape)
        call findnode_by_name(pvmmachlist,cmach,mach%name)
        if (associated(cmach)) cmach%usag = cmach%usag + 1
      end if

      stepfl(ia)=1
      iaus=mod(ia,naus)
      if (iaus.eq.0) iaus=naus
      m=ia/naus
      if(mod(ia,naus).ne.0) m=m+1
      do icmp=1,nvar
        gra(icmp,iaus,m)=ggrad(icmp)
      end do

      escf(ia)=e
      do jomeg=1,ipol
        do icmp=1,8
          pol(jomeg,icmp,iaus,m)=po(icmp,jomeg)
        end do
      end do

      deallocate(ggrad)

      mach%nodf = 'rdy '
      mach%tpid = 0

*     ----------------------------------------------------------------------
*     write data to restart file
*     ----------------------------------------------------------------------
      call wrestart(vmach,gra,stepfl,escf,pol,nbm)
      call print_node(mach,idbg)

*     -------------------------------------------------
*     stop host if a shutdown command has been received
*     -------------------------------------------------
      if(mach%cmd == 'shut')
     * call remnode('shutdown cmd',0,stepfl,mach%numb)

#ifdef AKIRA_PVM
      write(idbg,*) '** getres: calling choose_nodes'
      call updvm(updopt)
#endif

      return
      end
