#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      SUBROUTINE islave(nbm)

****************************************************
* akira version by J. Neugebauer, Uni Erlangen, 2001
****************************************************
* modified by C. Herrmann, Uni Bonn, 2004-2005,
* for Gaussian and ADF interface
* slightly adapted for excited state gradients by
* C. Herrmann, Uni Jena 04/2005
* and for ADF-interface 07/2005
* cleanup of file copy section by
* J. Neugebauer, 08/2005
****************************************************

#include "defs.h"
#include "param.h"

#ifdef AKIRA_PVM
#include "fpvm3.h"
#endif

#ifdef AKIRA_MPI
#include "mpif.h"
#endif

      integer l,ibuf,info,mbuf,ierr,irc,xeq,now,w,lf
      integer intgr,iclean,icleanl,ncalc,info2,nbm
#ifdef AKIRA_MPI
      integer st(MPI_STATUS_SIZE)
      character*(MPI_MAX_PROCESSOR_NAME) mpiname
#endif
      integer*4 iready,iwhat,kvers,ia,i4,iready2
      character*255 b1,b2,filename,fname
      character*40 mynode,pnode
      logical valmod,ex,parafreq
      integer lres
      external valmod

#include "version.h"

**
** slave processes
**
      ncalc=0

c=========================================================================
* receive tempdir, jobdir, parent node name, current node name from master
c=========================================================================

#ifdef AKIRA_MPI
        call MPI_GET_PROCESSOR_NAME(mpiname,l,info)
        call mpierr('MPI_GET_PROCESSOR_NAME','islave',info)
        call clean(mynode)
        mynode=mpiname(1:l)
#endif

        call pgetstr('islave: logdir',logdir,tid0,255,401,info)
        call slen(logdir,ll)
        call clean(fname)
        call secs(now)
        call augnum(fname,logdir(1:ll)//'dclog.',now)
        call slen(fname,lf)
        call clean(filename)
        call augnum(filename,fname(1:lf)//'_',mytid)
        call slen(filename,lf)
        OPEN (unit=iosl,file=filename(1:lf),STATUS='UNKNOWN')
        WRITE (iosl,*) 'islave: logdir=',logdir

        WRITE (iosl,*) '>>> AKIRA slave log file for tid ',mytid
        WRITE (iosl,*) version
        call pgetstr('islave: turbodir',turbodir,tid0,255,402,info)
        call pgetstr('islave: arpath',arpath,tid0,255,403,info)
        call pgetstr('islave: masterid',masterid,tid0,255,404,info)
        call pgetstr('islave: tmpnode',b1,tid0,255,405,info)
        call pgetstr('islave: mydir',mydir,tid0,255,406,info)
        call pgetstr('islave: pnode',pnode,tid0,40,407,info)
        WRITE (iosl,*) 'islave: turbodir=',turbodir
        call flush(iosl)
#ifdef AKIRA_MPI
        call pputstr('islave: mynode',mynode,tid0,40,408,info)
#endif

#ifdef AKIRA_PVM
        call pgetstr('islave: mynode',mynode,tid0,40,408,info)
#endif
        call slen(b1,lt)
        call slen(mydir,lm)
        call slen(arpath,lar)
        call slen(turbodir,lb)
        call slen(masterid,li)
CCH140605C#      now=10*now+mytid

        call clean(b2)
        call fdate(b2)
        write (iosl,*) b2(1:40)
        call flush(iosl)
        write (iosl,*) 'TMPDIR=',b1(1:lt)
        write (iosl,*) 'LOGDIR=',logdir(1:ll)
        write (iosl,*) turbodir(1:10)
        if (lb.gt.0) write (iosl,*) 'PROGDIR=',turbodir(1:lb)
        write (iosl,*) 'MYDIR =',mydir(1:lm)
        write (iosl,*) 'MYNODE=',mynode
        write (iosl,*) 'PNODE =',pnode

*
* St. Schenk
* We also have to suffix the name of the temporary directory with the
* tid because otherwise we have a race condition. On present day
* hardware it is not completely unlikely that we may spawn two processes
* in a second (irony off). By the way, why is TMPDIR stored in a
* variable intuitively called b1 ?
*
        call clean(fname)
        call augnum(fname,b1(1:lt)//'/TMP',now)
        call slen(fname,lf)
        call augnum(tempdir,fname(1:lf)//'_',mytid)
        call slen(tempdir,lt)

        write (iosl,*) 'PE0 TID=',tid0
        call flush(iosl)
        info=xeq('mkdir '//tempdir(1:lt))
        info=0
c
c=========================================================
c get input files
c CAUTION: always synchronize with serstuff.r, spawn.r, basic_input
c=========================================================

c-----------------
** akira_control
c-----------------

100     WRITE (iosl,*) 'islave: waiting for <akira_control>'
        call flush(iosl)
        okok = 1
#ifdef SNF_SCP
        WRITE (iosl,*) 'get akira_control from ',mynode
        call getfile('akira_control',mynode,pnode,info)
        WRITE (iosl,*) 'return form getfile, info=',info
#else
        call gfile('akira_control',tid0,info)
#endif

        WRITE (iosl,*) 'islave: received <akira_control>, ncalc is',
     *                  ncalc
        call flush(iosl)

        call rakiractrl(tempdir(1:lt)//'/akira_control',rthres,
     *  rabsthres,dvthres,ecthres,
     *  roothmpr,g98it,g98bs,xmolout,tmout,printall,precond,genstart,
     *  rchthres,minnbt,iorth,snfout,natoms,mode,cstep,naus,iclean,
     *  icleanl,idbg,ipol,lastqmatom,irescp,restadf,sleeptime)  

        WRITE (iosl,*) 'islave: read akira_control, mode is ',mode
        call flush(iosl)

       if (.not.valmod(mode)) then
         write(iosl,*) 'islave: No such program mode: ',mode
         stop
       end if

c----------------------------------------
** MACHINES (for parallel turbomole run)
c----------------------------------------

        parafreq=.false.
        call rkeywd(tempdir(1:lt)//'/control','$parafreq',parafreq)
        WRITE (iosl,*) 'islave: parafreq is: ',parafreq
        call flush(iosl)
        if(parafreq) then
          okok=10*okok+1
#ifdef SNF_SCP
          call getfile('MACHINES',mynode,pnode,info)
#else
          call gfile('MACHINES',tid0,info)
#endif
        end if

*=======================================================================
*=======================================================================
* HERE COME THE INPUT FILES
* CAUTION: always synchronize with serstuff.r, spawn.r, basic_input.r (okok) !
*          Even if program runs with --initial-scp option, it might not
*          work without that option
*=======================================================================
*=======================================================================

**=============================================
** modes 1-19 reserved for: TURBOMOLE 
** and COMQUM (Turbomole + Amber QM/MM interface)
** (mode 10 is a test mode for parallelization)
**=============================================

        if(mode.gt.0.and.mode.lt.20.or.(mode.ge.60.and.mode.lt.70)) then

**------------------
** file no. 1: control
**------------------

          okok=10*okok+1
#ifdef SNF_SCP
          call getfile('control',mynode,pnode,info)
#else
          call gfile('control',tid0,info)
#endif
        if(mode.lt.60) then
          call rcontrol(tempdir(1:lt)//'/control',rpast,smult,natoms,
     *    jscfc,isuhf,riflag)
        else
          call rcontrol(tempdir(1:lt)//'/control',rpast,smult,
     *    lastqmatom,jscfc,isuhf,riflag)
        end if

**------------------
** file no. 2: basis
**------------------

          okok=10*okok+1
#ifdef SNF_SCP
          call getfile('basis',mynode,pnode,info)
#else
          call gfile('basis',tid0,info)
#endif

**---------------------
** file no. 3: auxbasis
**---------------------

          if(riflag.eq.1) then
            okok=10*okok+1
#ifdef SNF_SCP
            call getfile('auxbasis',mynode,pnode,info)
#else
            call gfile('auxbasis',tid0,info)
#endif
          end if

**------------------------------------
** file no. 4: mos or (alpha and beta)
**------------------------------------

          if(isuhf.eq.0) then
            okok=10*okok+1
#ifdef SNF_SCP
            call getfile('mos',mynode,pnode,info)
#else
            call gfile('mos',tid0,info)
#endif
          else
            okok=10*okok+1
            okok=10*okok+1
#ifdef SNF_SCP
            call getfile('alpha',mynode,pnode,info)
            call getfile('beta',mynode,pnode,info)
#else
            call gfile('alpha',tid0,info)
            call gfile('beta',tid0,info)
#endif
          end if

**=============================================
** modes 20-29 reserved for: DALTON
**=============================================

        else if(mode.ge.20.and.mode.lt.30) then
          okok=10*okok+1
#ifdef SNF_SCP
          call getfile('molecule.dal',mynode,pnode,info)
#else
          call gfile('molecule.dal',tid0,info)
#endif

**=============================================
** modes 30-39 reserved for: GAUSSIAN
**=============================================

        else if(mode.ge.30.and.mode.lt.40) then
          okok=10*okok+1
#ifdef SNF_SCP
          call getfile('akira.com',mynode,pnode,info)
#else
          call gfile('akira.com',tid0,info)
#endif

**=============================================
** modes 40-49 reserved for: ADF
**=============================================

        else if(mode.ge.40.and.mode.lt.50) then
          okok=10*okok+1
#ifdef SNF_SCP
          call getfile('adf.in',mynode,pnode,info)
#else
          call gfile('adf.in',tid0,info)
#endif
          call radf(tempdir(1:lt)//'/adf.in',natoms,qmmmatomtype)
          if(irescp.gt.0) then 
             okok=10*okok+1
             call llen(restadf,lres)
#ifdef SNF_SCP
             call getfile(restadf(1:lres),mynode,pnode,info)
#else
             call gfile(restadf(1:lres),tid0,info)
#endif
          end if
CCH          if(lastqmatom.ne.natoms) then
CCH             okok=10*okok+1
CCHC&           call getfile('mm.restart',mynode,pnode,info)
CCHC//SKIPC&1
CCH             call gfile('mm.restart',tid0,info)
CCH          end if
**=============================================
** modes 50-59 reserved for: MOLPRO
** (to be implemented one day)
**=============================================

        end if
**=============================================
** modes 60-69 reserved for: COMQUM
**=============================================
        if(mode.ge.60.and.mode.lt.70) then 
          info2=0
          okok2=1
#ifdef SNF_SCP
          call getfile('comqum.dat',mynode,pnode,info2)
#else
          call gfile('comqum.dat',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('pointcharges',mynode,pnode,info2)
#else
          call gfile('pointcharges',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('prmcrd3',mynode,pnode,info2)
#else
          call gfile('prmcrd3',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('prmtop1',mynode,pnode,info2)
#else
          call gfile('prmtop1',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('prmtop2',mynode,pnode,info2)
#else
          call gfile('prmtop2',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('prmtop3',mynode,pnode,info2)
#else
          call gfile('prmtop3',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('sander.in1',mynode,pnode,info2)
#else
          call gfile('sander.in1',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('sander.in2',mynode,pnode,info2)
#else
          call gfile('sander.in2',tid0,info2)
#endif
          okok2=10*okok2+1
#ifdef SNF_SCP
          call getfile('sander.in3',mynode,pnode,info2)
#else
          call gfile('sander.in3',tid0,info2)
#endif
          call flush(iosl)
        end if

        WRITE (iosl,*) '** islave: got all program-specific input files'
        call flush(iosl)

**=============================================
** now copy coord file
**=============================================

        WRITE (iosl,*) 'islave: waiting for <coord> '
        call flush(iosl)
        okok=10*okok+1
#ifdef SNF_SCP
        call getfile('coord',mynode,pnode,info)
#else
        call gfile('coord',tid0,info)
#endif
        write(iosl,*) 'islave: coord copied, info=',info
        call flush(iosl)

c======================================================
c make backup copies in temp. directory on slave nodes
c======================================================
        if (info.EQ.okok) then
          if(mode.lt.20.or.(mode.ge.60.and.mode.lt.70)) then
            if (isuhf.EQ.0) then
              call clean(b1)
              b1=tempdir(1:lt)//'/mos'
              call llen(b1,l)
              irc=xeq('cp '//b1(1:l)//' '//b1(1:l)//'.ori')
            else
              call clean(b1)
              b1=tempdir(1:lt)//'/alpha'
              call llen(b1,l)
              irc=xeq('cp '//b1(1:l)//' '//b1(1:l)//'.ori')
              call clean(b1)
              b1=tempdir(1:lt)//'/beta'
              call llen(b1,l)
              irc=xeq('cp '//b1(1:l)//' '//b1(1:l)//'.ori')
            end if
          end if
          call clean(b1)
          b1=tempdir(1:lt)//'/coord'
          call llen(b1,l)
          irc=xeq('mv '//b1(1:l)//' '//b1(1:l)//'.ori')
          if(mode.ge.60.and.mode.lt.70) then
            call clean(b1)
            b1=tempdir(1:lt)//'/prmcrd3'
            call llen(b1,l)
            irc=xeq('mv '//b1(1:l)//' '//b1(1:l)//'.ori')
          end if
        end if

c=======================================
** all input files copied
c=======================================
        WRITE (iosl,*) '>>> input files copied, result is ',info
        call flush(iosl)
        iready=info
        iready2=info2
        WRITE (b1,'(I5)') ivers
#ifdef DEBUG
        call mnotify(9,mytid,iready,b1,5)
        call mnotify(9,mytid,iready2,b1,5)
#endif


c=======================================
** sending information on copying
c=======================================
      write (iosl,*) '>>> sending iready: ',iready
#ifdef AKIRA_MPI
      w=MPI_COMM_WORLD
#ifdef AKIRA_MPI_SSEND
      call MPI_SSEND(iready,1,MPI_INTEGER,tid0,407,w,info)
      call mpierr('MPI_SSEND iready','islave',info)
      if(mode.eq.60) then
        call MPI_SSEND(iready2,1,MPI_INTEGER,tid0,407,w,info2)
        call mpierr('MPI_SSEND iready2','islave',info2)
      end if
      call MPI_SSEND(ivers,1,MPI_INTEGER,tid0,408,w,info)
      call mpierr('MPI_SSEND ivers','islave',info)
#else
      call MPI_SEND(iready,1,MPI_INTEGER,tid0,407,w,info)
      if(mode.eq.60) then
        call mpierr('MPI_SEND iready2','islave',info2)
        write (iosl,*) '>>> sent iready2: ',iready2
      end if
      call MPI_SEND(ivers,1,MPI_INTEGER,tid0,408,w,info)
      call mpierr('MPI_(S)SEND ivers','islave',info)
      write (iosl,*) '>>> sent ivers: ',ivers
#endif
      if (iready.ne.okok.or.(mode.eq.60.and.iready2.ne.okok2)) then
        write (iosl,*) '>>> calling termcomm: iready =',iready
        write (iosl,*) '>>>                     okok =',okok
        call flush(iosl)
        call termcomm('islave: not ready')
      else 
        write (iosl,*) '>>> input completed, waiting for task ...'
      end if
  42  call MPI_RECV(iwhat,1,MPI_INTEGER,tid0,409,w,st,info)
      call mpierr('MPI_RECV iwhat','islave',info)
#endif

#ifdef AKIRA_PVM
      call flush(iosl) 
      call PVMFINITSEND(PVMDATADEFAULT,MBUF)
      call flush(iosl) 
      call PVMFPACK(INTEGER4, iready, 1, 1, INFO )
      call flush(iosl) 
      IF (info.NE.0) call pvmfperror('pack iready',info)
      call flush(iosl) 
      if(mode.eq.60) then
        call PVMFPACK(INTEGER4, iready2, 1, 1, INFO2 )
        call flush(iosl) 
        IF (info.NE.0) call pvmfperror('pack iready2',info2)
        call flush(iosl) 
      end if
      kvers=ivers
      call PVMFPACK(INTEGER4, kvers, 1, 1, INFO )
      call flush(iosl) 
      IF (info.NE.0) call pvmfperror('pack ivers',info)
      call PVMFSEND( tid0, 0, INFO )
      call flush(iosl) 
      IF (info.NE.0) call pvmfperror('send iready',info)
      write(iosl,*) 'iready2,okok2 = ', iready2,okok2
      call flush(iosl) 
      IF (iready.NE.okok.or.(mode.eq.60.and.iready2.ne.okok2)) THEN
       write(iosl,*) 'okok = ', okok
       if(mode.eq.60)  write(iosl,*) 'okok2 = ', okok2
       WRITE (iosl,*) '>>> calling termcomm: iready=',iready
       if(mode.eq.60) WRITE (iosl,*) '>>> calling termcomm: iready2=',
     *                               iready2
       call flush(iosl)
       call termcomm('islave: not ready')
      ENDIF

c==========================
* receive command
c==========================

 42   call PVMFRECV(tid0,0,ibuf)
      call flush(iosl) 
      call PVMFUNPACK(INTEGER4,iwhat,1,1,info)
      call flush(iosl) 
#endif
      write (iosl,*) '>>> iwhat =',iwhat
      call flush(iosl)
      write (iosl,*) 'islave: ',tempdir(1:lt)

*===========================================================================
* do calculations; the akira_control file must be updated in every iteration 
* (it contains new basis modes) !!! Actually, it is copied twice in the
* first iteration. If you have a good solution to avoid this, change it
* (here and in ssend.r)
*===========================================================================

      if(iwhat.eq.1) then
        ncalc=ncalc+1
        write (iosl,*) 'islave: waiting for <control>, ncalc = ',ncalc
        call flush(iosl)
        okok = 10*okok+1
        info=xeq('rm '//tempdir(1:lt)//'/akira_control')
#ifdef SNF_SCP
        write (iosl,*) 'get akira_control from ',mynode
        call getfile('akira_control',mynode,pnode,iready)
#else
        call gfile('akira_control',tid0,iready)
#endif
        write (iosl,*) 'return form g(et)file, iready=',iready
        write (iosl,*) '                       okok=',okok

        if(iready.eq.okok) then
          write (iosl,*) 'islave: received <akira_control>, ncalc = ',
     *     ncalc
          call flush(iosl)
        else 
          write (iosl,*) 
     *     'islave: no <akira_control>, shutting down node'
          call errex('akira_control file missing')
        end if
        call rmoldscr(tempdir(1:lt),mode)
c--------------------------
c call cslave
c--------------------------
        call cslave(nbm)
      end if
      write(iosl,*) '** islave: iwhat = ',iwhat
      call flush(iosl)
      if(iwhat.ge.0) goto 42

      write (iosl,*) '>>> finishing'
      call flush(iosl)
      write (iosl,*) 'slave tid ',mytid,' stopping.'
      close (UNIT=iosl)
      if(icleanl.eq.1) then
        call slen(filename,lf)
        info=xeq('rm '//filename(1:lf))
      end if
      if(iclean.eq.1) then
        info=xeq('rm -r '//tempdir(1:lt))
      end if

      return
      end
