      subroutine lanhes(grad,dgrad,normm,ic,happ,lam,evec,
     * avib,vvec,olam,iti,lunit,dconv,isel,nsel,csel,oldnorm,rvec,
     * ovvec,vtemp,nbm,onsel,trm,mxres,ibm,oisel,nbmo,ortmod)
*------------------------------------------------------------------
* Lanczos-diagonalization of the Hessian:
* calculate vibrational normal modes and corresponding frequency
* for selected modes according to a Lanczos-like diagonalization
* scheme.
*
* CAUTION: array normm: 
* ---------------------
* on input : mass-weighted basis vectors 
* on output: cartesian basis vectors; not mass-weighted
*
* NOTE: this is a testmodule; it is not optimized for production
*       work, especially note the following restrictions:
*       - the block lanczos method is not tested
*       - there is only a simple single orthogonalization,
*         no strict orthogonalization as in the Davidson module
*       - there is no additional check of the orthonormality
*       - the orthogonalization to modes stored in ortmod is NOT 
*         carried out
*
* written by J. Neugebauer, Uni Erlangen, 2002
*------------------------------------------------------------------
#include "defs.h"
#include "param.h"

      real*8, allocatable :: work(:)
      integer ic(natoms),iti,i,j,disptype,lwork,ilaenv,nsel
      integer info,k,isel(mxnsel),lunit,isel2,nbm,nsel2,ibm(mxnsel)
      integer onbm,onsel,count,iscr,l,isel3,oisel(mxnsel)
      integer nbmo
      real*8 grad(nvar,naus,maxnbm),normm(nvar,maxnbm),cstepq
      real*8 dgrad(nvar,maxnbm),haij,dotprod,qfc2freq,find_mass
      real*8 happ(maxnbm,maxnbm),lam(maxnbm),evec(maxnbm,maxnbm)
      real*8 prec,thres,norm,avib(maxnbm),vvec(nvar,maxnbm)
      real*8 olam(maxnbm),oldnorm(mxnsel),rvec(nvar,maxnbm),
     *       ovvec(nvar,maxnbm),scr,trm(nvar,6),mxres(mxnsel)
      real*8 vtemp(nvar),ovl,ortmod(nvar,nbmo)
      parameter(thres=1.0d-1)
      logical dconv,para,csel(mxnsel)
      logical dcsel
      character*80 text
      external dotprod,qfc2freq,find_mass

*---------------
* initialization
*---------------
      do i=1,mxnsel
        ibm(i)=0
      end do

*----------------------------------------
* no. of basis vectors in last iteration
*----------------------------------------
      onbm=nbm-nsel+1
      disptype=1
      call rkeyint('akira_control','$disptype',disptype)

      write(ftape,*)
      write(ftape,'(2x,a)') '::::::::::::::::::::::::::::::::::::::::'
      write(ftape,'(2x,a,i5)') '    Results for iteration No.:',iti
      write(ftape,'(2x,a,i5)') '    No. of basis vectors     :',nbm
      write(ftape,'(2x,a)') '::::::::::::::::::::::::::::::::::::::::'
      write(ftape,*)

      if(nsel.gt.1) 
     * call serror('lanhes',2,'No block lanczos implemented',ftape)
*================================================================
* 2) calculate vector s_i ("dgrad") for each new basis vector
*================================================================
      do j=onbm,nbm

*----------------------------------------------------------------
* determine step size
*----------------------------------------------------------------
        if(disptype.eq.1) then
          call getcstepq(normm(1,j),cstep,cstepq,ic,natoms)
        else
          stop 'disptype not available'
        end if
        write(ftape,'(a,i4,a,f10.6)')
     *  '  step size for mode no. ',j,' : ',cstepq

*----------------------------------------------------------------
* do numerical differentiation (this yields the vectors s_i=H*b_i)
*----------------------------------------------------------------
        call d_grad(grad,dgrad(1,j),naus,nvar,cstepq,j,maxnbm)

*----------------------------------------------------------------
* transform vectors H*b_i to mass-weighted coordinates
* (note: mass-weighting for differentiation with respect to 
* basis mode is already done)
*----------------------------------------------------------------
        do i=1,nvar
          dgrad(i,j)=dgrad(i,j)/sqrt(find_mass(ic((i+2)/3)))
        end do
        if(printall) then
          write(text, '(a,i5)') 'Vector s_i=H*b_i for i = ',j
          call printdmat(dgrad(1,j),nvar,1,text,'f20.12',ftape)
        end if

*================================================================
* 3) calculate diagonal elements of tridiagonal matrix T (d_i)
*================================================================

*----------------------------------------------------------------
* calculate H_approx_ii for all new possible values i
* H_approx_ii = s_i (dot) b_i
*----------------------------------------------------------------
        happ(j,j)=dotprod(dgrad(1,j),normm(1,j),nvar)
      end do
      if(printall) then
        write(text,'(a,i5)') 'Approximate diagonal of T of dimension ',
     *   nbm
        call printdmat2(happ,nbm,maxnbm,nbm,text,'f20.12',ftape) 
        call flush(ftape)
      end if
  
*================================================================
* 4) calculate vector x_(i+1)=s_i - d_i b_i - t_(i-1) b_(i-1)
*================================================================
      do i=1,nvar
        normm(i,nbm+1)=dgrad(i,nbm)-happ(nbm,nbm)*normm(i,nbm)
        if(nbm.gt.1)
     *   normm(i,nbm+1)=normm(i,nbm+1)-happ(nbm-1,nbm)*normm(i,nbm-1)
      end do
      if(printall) then
        write(text,'(a,i5)') 'Vector x_(i+1) for i = ',nbm
        call printdmat(normm(1,nbm+1),nvar,1,text,'f20.12',ftape) 
      end if

*================================================================
* 5) calculate t_i = happ(i+1,i) = happ(i,i+1) = |x_(i+1)|
*================================================================
      happ(nbm,nbm+1)=sqrt(dotprod(normm(1,nbm+1),normm(1,nbm+1),nvar))
      happ(nbm+1,nbm)=happ(nbm,nbm+1)
C      write(text,'(a,i5)') 'Tridiagonal matrix T of dimension ',nbm+1
C      call printdmat2(happ,(nbm+1),maxnbm,(nbm+1),text,'f20.12',ftape) 

*================================================================
* 6) calculate new basis vector b_(i+1)=x_(i+1)/|x_(i+1)|
*================================================================
      do i=1,nvar
        normm(i,nbm+1)=normm(i,nbm+1)/happ(nbm,nbm+1)
      end do
C      if(nsel.gt.0) then
      if(printall) then
        write(text, '(2(a,i5))') 
     *   'New cartesian basis vectors (not orthogonalized)'
        call printdmat(normm(1,nbm+1),nvar,1,text,'f20.12',ftape) 
      end if
C      end if

*================================================================
* 7A) diagonalize approximate Hessian; store eigenvalues and
*    eigenvectors (not optimized: happ)
*================================================================
      if(nbm.gt.1) then
        do i=1,nbm
          do j=1,nbm
            evec(i,j)=happ(i,j)
          end do
        end do
*----------------------------------------
* determine optimum size of work array
*----------------------------------------
        lwork=(ilaenv(1,'DSYTRD','l',nbm,nbm,nbm,nbm)+2)*nvar
        allocate(work(lwork))
        call dsyev('V','L',nbm,evec,maxnbm,lam,work,lwork,info)
        deallocate(work)
        if(info.ne.0) call serror('lanhes',2,'Could not '//
     *  'calculate eigenvectors##of approximate Hessian',ftape)
      else
        lam(1)=happ(1,1)
        evec(1,1)=1.0d0
      end if

*----------------------------------------------------------------
* print eigenvalues and eigenvectors for this iteration
*----------------------------------------------------------------
      write(text,'(a,i5)') 'Approximate eigenvalues for iteration ',iti
      call printdmat(lam,nbm,1,text,'f20.12',ftape) 
      write(text,'(a,i5)') 'Approximate eigenvectors for iteration ',iti
      call printdmat2(evec,nbm,maxnbm,nbm,text,'f20.12',ftape) 

*----------------------------------------------------------------
* calculate approximate frequencies and print them
*----------------------------------------------------------------
      do i=1,nbm
        avib(i)=qfc2freq(lam(i))
      end do
      write(text, '(a,i5)') 
     * 'Approximate wavenumbers [1/cm] for iteration ',iti
      call printdmat(avib,nbm,1,text,'f20.12',ftape) 
      
*================================================================
* 7B) calculate approximation for normal modes (vvec)
*================================================================
      do i=1,nbm
        do j=1,nvar
          vvec(j,i)=0.0d0
          do k=1,nbm
            vvec(j,i)=vvec(j,i)+evec(k,i)*normm(j,k)
          end do
        end do
      end do
      if(printall) then
        write(text,'(a,i5)') 'Approximate normal modes in iteration ',
     *    iti
        call printdmat(vvec,nvar,nbm,text,'f20.12',ftape) 
      end if

*================================================================
* 7C) calculate residuum vectors for all eigenvectors
*================================================================
      do j=1,nbm
        do i=1,nvar
          rvec(i,j)=0.0d0
          do k=1,nbm
            prec=evec(k,j)*(dgrad(i,k)-lam(j)*normm(i,k))
            rvec(i,j)=rvec(i,j)+prec
          end do
        end do
      end do
      if(printall) then
        write(text, '(a,i5)') 'Residuum vectors for iteration ',iti
        call printdmat(rvec,nvar,nbm,text,'f20.12',ftape) 
      end if

*================================================================
* 7D) select eigenvector to be optimized (root homing)
*================================================================
      do i=1,onsel
        oisel(i)=isel(i)
      end do
      do i=1,onsel
        if(.not.csel(i)) then
          call roothome(isel,iti,i,nbm,lam,olam,vvec,ovvec,normm,
     *      vtemp,rvec,onsel)
        end if
      end do

*----------------------------------------------------------------
* calculate norm of residuum; stop, if convergence is achieved
* (another criterion for convergence might be the absolute value
* of the coefficient of the new basis vector in the approximate
* eigenvector; not yet implemented)
*----------------------------------------------------------------
      nsel2=0
      count=0
      dconv=.true.
*----------------------------------------------------------------
* there might be several roots to be determined
* (block Lanczos method; not yet implemented)
*----------------------------------------------------------------
      do j=1,onsel
        call resicheck(isel,rvec,mxres,j,iti,evec,nbm,dcsel,onsel,
     *  olam,oisel,avib,norm)
        if(.not.csel(j)) then
          count=count+1
          csel(j)=dcsel
          if(csel(j)) then
            nsel2=nsel2+1
          end if
          oldnorm(j)=norm
          write(ftape,*)
          dconv=(dconv.and.csel(j))
        end if
      end do

*----------------------------------------------------------------
* write elementary data to file
*----------------------------------------------------------------
      write(lunit,448) iti,(isel(j),avib(isel(j)),
     * avib(isel(j))-qfc2freq(olam(oisel(j))),mxres(j),oldnorm(j),
     *  evec(nbm-(onsel-j),isel(j)),j=1,onsel)
      call flush(lunit)
448   format(i3,500(i9,f8.2,f10.2,2f12.8,f10.6))
*----------------------------------------------------------------
* store "old" eigenvalues and eigenvectors
*----------------------------------------------------------------
      do i=1,nbm
        olam(i)=lam(i)
        do j=1,nvar
          ovvec(j,i)=vvec(j,i)
        end do
      end do

*----------------------------------------------------------------
* set new no. of selected eigenvectors not yet converged
*----------------------------------------------------------------
      write(idbg,*) 'nsel,nsel2:',nsel,nsel2
      nsel=nsel-nsel2
      if(dconv.or.nsel.eq.0) then
        dconv=.true.
        write(ftape,'(2x,a)') 
        write(ftape,'(2x,a,i5)') 
     *     'Convergence is achieved in iteration ',iti
        write(ftape,'(2x,a)') 
        goto 900
      end if

*================================================================
* 8) explicit orthonormalization of new basis vector
*================================================================

      write(idbg,*) 'nbm+nsel,maxnbm:',nbm+nsel,maxnbm
      if((nbm+nsel).gt.maxnbm) return

      do j=1,nsel
*--------------------------------------------------------------------
* orthonormalization of new basis vector; skip basis vectors which
* are linearly dependent
*--------------------------------------------------------------------
       call normalize(normm(1,nbm+j),nvar)
       do i=1,nbm+j-1
         call gramschmidt(normm(1,i),normm(1,nbm+j),nvar,para,ovl,
     *     othres)
         if(para) then
           write(ftape,'(2x,a)') 
     *      'Linear dependence detected for new basis vector'
           write(ftape,'(2x,a)') 
     *      'I will remove this vector from the basis set'
           write(ftape,*)
         end if
       end do
*--------------------------------------------------------------------
* in principle, we always want our basis vectors to be orthogonal
* to translational and rotational modes
*--------------------------------------------------------------------
C       if(roothmpr.eq.'testmin') then
         do i=1,6
           call gramschmidt(trm(1,i),normm(1,nbm+j),nvar,para,ovl,
     *     othres)
           if(para) then
             write(ftape,'(2x,a)')
     *        'Linear dependence detected for new basis vector'
             write(ftape,'(2x,a)')
     *        'I will remove this vector from the basis set'
             write(ftape,*)
           end if
         end do
C       end if
      end do
      call skipvec(normm,nvar,nbm+nsel,count,1.0d-2)
      nsel=nsel-count

*----------------------------------------------------------------
* basis vector is mass-weighted; this procedure is reversed here
*----------------------------------------------------------------
900   continue
      do i=1,maxnbm
        call nmod_m2c(normm(1,i),natoms,ic)
      end do
      if(nsel.gt.0.and.printall) then
        write(text, '(2(a,i5))') 
     *   'New cartesian basis vectors No. ',nbm+1,' - ',nbm+nsel
        call printdmat(normm(1,nbm+1),nvar,nsel,text,'f20.12',ftape) 
      end if
      return
      end
