#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      SUBROUTINE master

*-----------------------------------------------------------
* this subroutine controls the davidson algorithm and the
* parallel execution of the single-point calculations
*
* by J. Neugebauer, M. Reiher, Uni Erlangen, 2002
*
* migrate to MachineModule (=> spawn_mpi) by
* J. Neugebauer, 2007
*-----------------------------------------------------------

      use CoordModule
      use RestartModule

#include "defs.h"
#include "param.h"

      real*8, allocatable :: trro(:),lambda(:),olambda(:),avib(:)
      real*8, allocatable :: vtemp(:),svec(:),evec(:),vvec(:),ovvec(:)
      real*8, allocatable :: rvec(:),happ(:),normm(:),pol(:),aphs(:)
      real*8, allocatable :: ortmod(:),ggrad(:)
      integer, allocatable :: itstepfl(:)

      integer ize,ipe,iprc,info,i,idum,iti,lunit,itir
      integer system,nbm,nvara,pskp
      integer isel(mxnsel),nsel,onsel,j,ltxt,sunit,iclean,icleanl
      integer nnbm(mxit),no,time1,time2,tt0,tt1
      integer oisel(mxnsel),ibm(mxnsel),nbmo,tnbm
      real*8 oldnorm(mxnsel),xmolsc1,mxres(mxnsel),qfc2freq
      character*8 dash
      character*1 text
      character*13 acontrol
      character*30 txt,b
      logical dconv,csel(mxnsel),laphs
      logical notrro,first
      parameter(lunit=14,no=2)
      external system,qfc2freq

      data dash /' ------ '/
      call secs(tt0)
      first=.true.
      sunit=6
      dconv=.false.
      nsel=1
      do i=1,mxnsel
        isel(i)=i
        csel(i)=.false.
        oldnorm(i)=0.0d0
      end do
      precond='backtra'
      notrro=.false.
      nbmo=0
      nvara=0
      pskp=0

*------------------------------------------------------------------
* create translational and rotational modes
*------------------------------------------------------------------
      allocate(trro(6*nvar))
      call trarot(coord,trro(1:),trro(3*nvar+1:),icharge,natoms)

*------------------------------------------------------------------
* get some input keywords
* (this should be redundant, since these keywords can also be
* read in akiraini)
*------------------------------------------------------------------
      call rkeywd('akira_control','$notrro',notrro)
      if(notrro) write(idbg,*) '** master: found $notrro flag'
      call rkeystr('akira_control','$precond',precond)
      write(idbg,*) '** master: precond = |'//precond(1:7)//'|'
      call rkeywd('akira_control','$appr_hessian',laphs)
      if(laphs) write(idbg,*) '** master: found $appr_hessian flag'
      call rkeyint('akira_control','$maxnbm',maxnbm)
      call rkeyint('akira_control','$northo',nbmo)
      maxnstept=maxnbm*naus

*------------------------------------------------------------------
* never use more modes than 3*natoms = no. of d.o.f.
*------------------------------------------------------------------
      if(nvar.lt.maxnbm) then
        call serror('master',2,'Max. no. of basis vectors is##'//
     *    'larger than no. of degrees of freedom',idbg)
      end if
      if(nvar.lt.nbmo) then
        call serror('master',2,'Orthogonal subspace too large',idbg)
      end if

*------------------------------------------------------------------
* allocate memory for gradient arrays and arrays used for
* davidson algorithm
*
* Note: happrox is the (it x it) matrix, which is updated in each
*       iteration, while aphess is the cartesian hessian guess,
*       which must be supplied, e.g., from a mopac calculation
*------------------------------------------------------------------
      allocate(lambda(maxnbm),olambda(maxnbm),avib(maxnbm),vtemp(nvar))
      allocate(svec(nvar*maxnbm),evec(maxnbm*maxnbm),vvec(nvar*maxnbm))
      allocate(ovvec(nvar*maxnbm),rvec(nvar*maxnbm),happ(maxnbm*maxnbm))
      allocate(normm(nvar*maxnbm),pol(8*naus*maxnbm*mxomeg))
      allocate(grad(nvar,naus,maxnbm),escf(maxnstept+1))
      allocate(istepfl(maxnstept),vmach(maxnstept))

      if(laphs) then
        nvara=nvar
      end if
      allocate(aphs(nvara*nvara))
      allocate(ortmod(nvar*nbmo))
      if(nbmo.gt.0) then
        write(idbg,*) '** master: no. of orthogonal modes: ',nbmo
        call getnctm('akira_control',natoms,ortmod,
     *       '$orthonormal basis modes',nbmo)
        do i=1,nbmo
          call nmod_c2m(ortmod(nvar*(i-1)+1),natoms,icharge)
        end do
        call printdmat(ortmod,nvar,nbmo,'mw orthogonal modes','f22.11',
     *    idbg)
      end if

*----------------------------------------------------------------
* initialization
*----------------------------------------------------------------
      olambda(:) = 0.0d0

*----------------------------------------------------------------
* o.k., we've got an approximate hessian from file
*----------------------------------------------------------------
      if(laphs) then
        call getnctm('akira_control',natoms,aphs,'appr_hessian',nvar)
        call hessianmw(aphs,natoms,icharge)
#ifdef DEBUG
          call printdmat(aphs,nvar,nvar,'mw hessian','f22.11',idbg)
#endif
      end if

      do i=1,naus*maxnbm
        read (dash,'(a8)') vmach(i)
        istepfl(i)=0
      end do

*     --------------------------------------------------------------------
*     in case of MPI, we have to set up all slaves for communication.
*     otherwise, we get into trouble if some of the slaves are not called.
*     --------------------------------------------------------------------
#ifdef AKIRA_MPI
      call spawn_mpi
#endif

*---------------------------------------------------------------
* backup old files and open new ones
*---------------------------------------------------------------
      call mfile('akira_iterations.out')
      call mfile('wavenumbers')
      call mfile('tm_normmodes')
      open(unit=ftape,file='akira_iterations.out')
      call akirahead(ftape)
      open(unit=lunit,file='wavenumbers')
      write(ftape,'(a)')
      write(ftape,'(a)') '  ==========================================='
      if(precond.eq.'lanczos') then
        write(ftape,'(a)') '         Output from module lanczos        '
      else
        write(ftape,'(a)') '         Output from module davidson       '
      end if
      write(ftape,'(a)') '  ==========================================='
      write(ftape,'(a)')

*---------------------------------------------------------------
* initialization ...
*---------------------------------------------------------------
      iti=1
      nbm=1
      nnbm(1)=1
      do i=2,mxit
        nnbm(i)=0
      end do
*------------------------------------------------------------------
* read number of iterations and no. of basis vectors per iteration
*------------------------------------------------------------------
      call rnnbm('akira_control',iti,nbm,nnbm,mxit,itir)
      write(idbg,*) 'initial iti: ',iti
      write(idbg,*) 'initial nbm: ',nbm
      write(idbg,*) 'chosen itir: ',itir
      nsel=nnbm(1)

*     -----------------------------------------------------------------
*     at this point, nstept should be the number of steps present on
*     the restart file => add up all basis modes of all iterations
*     and multiply by number of displacements (naus)
*     -----------------------------------------------------------------
      tnbm=0
      do i=1,iti
        tnbm=tnbm+nnbm(i)
      end do
      nstept=tnbm*naus
      write(*,*) 'nstept: ',nstept

      write(lunit,448) 'it. ',('rnv(',i,')  waven. wvnchange'//
     * '     |r_max|    |r|/nvar exp.coef. ',i=1,nsel)
      call flush(lunit)
448   format(a,500(a,i3,a))
      if(nsel.gt.mxnsel) then
        write(*,*) '     no. of basis vectors to be optimized: ',nnbm(1)
        write(*,*) 'max. no. of basis vectors to be optimized: ',mxnsel
        call serror('master',1,'No. of basis vectors selected for##'//
     *    'optimization is too large',idbg)
        call flush(idbg)
        call serror('master',2,'No. of basis vectors selected for##'//
     *    'optimization is too large',6)
      end if
      onsel=nsel

*------------------------------------------------------------------
* restart ?  => prepare reduced restart file in case itir < iti
*------------------------------------------------------------------
      if(iti.gt.1) then
        write(ftape,*)
        write(ftape,*) '========================================='
        write(ftape,*) '==== AKIRA: ENTERING RESTART SECTION ===='
        write(ftape,*) '========================================='
        write(ftape,*)
        call akiraini(ftape,no)
**------
** read original stepflags
**------
        call rrestart(vmach,grad,istepfl,i,escf,pol,nbm)

**------------------------------------------------------------------
** set up dummy stepflags for former modes
** special preparations necessary for itir = 1
**------------------------------------------------------------------
        tnbm=0
        do i=1,itir
          tnbm=tnbm+nnbm(i)
        end do
        nstept=tnbm*naus
        write(*,*) 'new nstept: ',nstept
        allocate(itstepfl(nstept))

        if(itir == 1) then
          itstepfl(1:nstept) = istepfl(1:nstept)
        else
          itstepfl(:) = 1
        end if
        iti = itir

**---------------------------------------------------------------------------
** write dummy stepflags to restart file; reduce no. of entries to tnbm modes
**---------------------------------------------------------------------------
        call wrestart(vmach,grad,itstepfl,escf,pol,tnbm)
        write(idbg,*) '** master: back from wrestart'
        call flush(idbg)
        if(iti == 1) then
          write(ftape,*)
     *      '** no restart iterations performed since restartit = 1'
          write(ftape,*)
          write(ftape,*) '========================================='
          write(ftape,*) '==== AKIRA: LEAVING RESTART SECTION  ===='
          write(ftape,*) '========================================='
          write(ftape,*)
        end if

**------
** perform davidson iterations for former modes
**------
        nbm=0
        do i=1,iti-1
          nbm=nbm+nnbm(i)
          if(i.ge.maxnbm) goto 900
          call akirainp(grad,itstepfl,vmach,icharge,normm,escf,pol,tnbm)
          if(precond.eq.'lanczos') then
           call secs(time1)
           call lanhes(grad,svec,normm,icharge,happ,lambda,evec,
     *      avib,vvec,olambda,i,lunit,dconv,isel,nsel,csel,oldnorm,
     *      rvec,ovvec,vtemp,nbm,onsel,trro,mxres,ibm,oisel,nbmo,ortmod)
           call secs(time2)
C           write(*,'(1x,a,i10,a)') '** time used in module lanczos: ',
C     *        time2-time1,' seconds'
          else
           call secs(time1)
           call davhes(grad,svec,normm,icharge,happ,lambda,evec,
     *      avib,vvec,olambda,i,lunit,dconv,isel,nsel,csel,oldnorm,rvec,
     *      ovvec,vtemp,nbm,onsel,laphs,aphs,trro,notrro,oisel,mxres,
     *      ibm,nbmo,ortmod,nvara)
           call secs(time2)
           write(*,*)
C           write(*,'(1x,a,i10,a)') '** time used in module davidson: ',
           write(*,*)
C     *        time2-time1,' seconds'
          end if
          if(dconv.or.genstart) then
            iti=i
            goto 900
          else
*------------------------------------------------------------------
* print intermediate results
*------------------------------------------------------------------
            call iterout(vvec,rvec,avib,i,maxnbm,rthres,
     *       natoms,nbm,precond,roothmpr,rabsthres)
          end if
        end do
        if(iti.gt.1) then
          write(ftape,*)
          write(ftape,*) '========================================'
          write(ftape,*) '==== AKIRA: LEAVING RESTART SECTION ===='
          write(ftape,*) '========================================'
          write(ftape,*)
        end if
        nbm=nbm+nnbm(iti)
        deallocate(itstepfl)
        if(nbm.ge.maxnbm) goto 900
**------
** write original stepflags to restart file
**------
        call wrestart(vmach,grad,istepfl,escf,pol,nbm)

**------------------------------------------------------
** check again number of free nodes
** (might have changed while running in restart section)
**------------------------------------------------------
       CALL clean(b)
#ifdef AKIRA_PVM
       CALL cmd_spawn(b)
#endif

      end if
*     -----------------------------------------------------------------
*     ... end of restart section; start subspace iteration
*     -----------------------------------------------------------------


*================================================================
*===== THIS IS THE DAVIDSON SUBSPACE ITERATION ALGORITHM ========
*================================================================

*================================================================
* 1) calculate gradients for next basis vector
*================================================================

*     -----------------------------------------------------------------
*     parallel calculation of single-point steps
*     -----------------------------------------------------------------
      write(idbg,*) 'restart file is: ',restart
100   call rrestart(vmach,grad,istepfl,i,escf,pol,nbm)
      nstept=nbm*naus
      write(idbg,*) 'restart file is: ',restart
      write (idbg,*) ' master| <restart> read'
      write(idbg,*) 'iti    = ',iti
      write(idbg,*) 'nbm    = ',nbm
      write(idbg,*) 'nstept = ',nstept
      write(idbg,*) 'isum   = ',i

*     -----------------------------------------------------------
*     check if all single points in this iteration have been done
*     -----------------------------------------------------------
      if(pskp.eq.2) pskp=0
      if(i.eq.nstept) pskp=2
      write(idbg,*) 'pskp   = ',pskp

      if(pskp.eq.0) then
       write(*,*) '===================================================='
       write(*,'(1x,a,i19)') 'Sending steps for iteration No.: ',iti
       write(*,'(1x,a,i12)') 'No. of basis vectors in this iteration: ',
     *   nbm
       if(iti.gt.1) then
         do i=1,nsel
           write(*,'(1x,a,i2,a,f20.10)') '|r_max(',i,')|    = ',mxres(i)
         end do
         do i=1,nsel
           write(*,'(1x,a,i2,a,f20.10)') 'wavenumber(',i,') = ',
     *       qfc2freq(olambda(isel(i)))
         end do
       end if
       write(*,*) '===================================================='
#ifdef AKIRA_SERIAL
       call serstuff(istepfl,vmach,escf,first,nbm)
#else
       call parstuff(grad,istepfl,vmach,escf,pol,iti,nbm)
#endif
      endif
#ifdef AKIRA_SERIAL
      call howmany_noprc(ize,ipe,istepfl)
#else
      call howmany(ize,ipe,iprc,istepfl)
#endif

*     -----------------------------------------------------------------
*     parallel calculation of steps for this iteration done,
*     go to the evaluation section ...
*     -----------------------------------------------------------------
      if(iti.eq.1) then
        call akiraini(ftape,no)
      end if

*     -----------------------------------------------------------------
*     akirainp reads the latest gradients etc. from restart file ...
*     -----------------------------------------------------------------
      call akirainp(grad,istepfl,vmach,icharge,normm,escf,pol,nbm)

*     -----------------------------------------------------------------
*     this is the heart of the davidson algorithm ...
*     -----------------------------------------------------------------
      if(precond.eq.'lanczos') then
       call secs(time1)
       call lanhes(grad,svec,normm,icharge,happ,lambda,evec,avib,
     *    vvec,olambda,iti,lunit,dconv,isel,nsel,csel,oldnorm,rvec,
     *    ovvec,vtemp,nbm,onsel,trro,mxres,ibm,oisel,nbmo,ortmod)
       call secs(time2)
C       write(*,'(1x,a,i10,a)') '** time used in module lanczos: ',
C     *    time2-time1,' seconds'
      else
       call secs(time1)
       call davhes(grad,svec,normm,icharge,happ,lambda,evec,avib,
     *    vvec,olambda,iti,lunit,dconv,isel,nsel,csel,oldnorm,rvec,
     *    ovvec,vtemp,nbm,onsel,laphs,aphs,trro,notrro,oisel,mxres,ibm,
     *    nbmo,ortmod,nvara)
       write(idbg,*) '** master: back from davhes'
       call flush(idbg)
       call secs(time2)
C       write(*,'(1x,a,i10,a)') '** time used in module davidson: ',
C     *    time2-time1,' seconds'
      end if

*     ---------------------------------------------
*     write g98 fake output for each iteration
*     (approximate normal modes and basis vectors)
*     ---------------------------------------------
      if(g98it) then
*       ----------------------------------------------------
*       normal modes
*       ----------------------------------------------------
        call clean(txt)
        if(iti.lt.10) then
          write(txt,'(A,i1)') 'g98.out.it',iti
        else if(iti.lt.100) then
          write(txt,'(A,i2)') 'g98.out.it',iti
        end if
        do i=1,nbm
          call nmod_m2c(vvec(nvar*(i-1)+1:),natoms,icharge)
        end do
        call llen(txt,ltxt)
        call pgaussian(txt(1:ltxt),vvec,natoms,coord,avib,icharge,nbm)
        do i=1,nbm
          call nmod_c2m(vvec(nvar*(i-1)+1:),natoms,icharge)
        end do
      end if
      if(g98bs) then
*       ----------------------------------------------------
*       basis modes
*       ----------------------------------------------------
        call clean(txt)
        if(iti.lt.10) then
          write(txt,'(A,i1)') 'g98.out.bs',iti
        else if(iti.lt.100) then
          write(txt,'(A,i2)') 'g98.out.bs',iti
        end if
        call llen(txt,ltxt)
        call pgaussian(txt(1:ltxt),normm,natoms,coord,avib,icharge,
     *       nbm+nsel)
      end if

*================================================================
* 9) prepare next iteration: write new control and restart file
*================================================================
      call wrnctm('akira_control',natoms,normm,maxnbm,
     * '$vibrational basis modes')

      if(dconv) then
        goto 900
      else

*       -----------------------------------------------------------------
*       print intermediate results
*       -----------------------------------------------------------------
        call iterout(vvec,rvec,avib,iti,maxnbm,rthres,
     *   natoms,nbm,precond,roothmpr,rabsthres)
      end if
      do i=1,nsel*naus
        istepfl(nbm*naus+i)=0
      end do
c

*     -----------------------------------------------------------------------
*     only generating start vectors for manual selection ???
*     -----------------------------------------------------------------------
      if(genstart.and.iti.gt.1) goto 900

C      write(text,'(i1)') iti
C      info=system('cp control control.'//text)
      iti=iti+1

      nbm=nbm+nsel
      nnbm(iti)=nsel
      write(idbg,*) 'new nbm: ',nbm
      write(idbg,*) 'new iti: ',iti
      call flush(idbg)

      call wrestart(vmach,grad,istepfl,escf,pol,nbm)

*     ---------------------------------------------------------------------
*     write number of new basis vectors for this iteration to akira_control
*     ---------------------------------------------------------------------
      acontrol='akira_control'
      write(idbg,*) 'roothmpr: ', roothmpr
      call wakiractrl(acontrol(1:13),naus,nvar,nnbm,mxit,iti,rthres,
     * laphs,rabsthres,dvthres,ecthres,roothmpr,g98bs,g98it,
     * xmolout,tmout,printall,precond,genstart,rchthres,minnbt,
     * iorth,snfout,natoms,mode,cstep,iclean,icleanl,idbg,ipol,
     *  lastqmatom,irescp,restadf,sleeptime)

*     ----------------------------------------------------------------------
*     check if max. no. of basis modes or max. no. of iterations is exceeded
*     ----------------------------------------------------------------------
      write(idbg,*) 'back from wakiractrl'
      if(nbm.lt.maxnbm.and.nsel.gt.0.and.iti.lt.mxit) then
        goto 100
      else
        write(ftape,*)
        if(iti.ge.mxit) then
          write(ftape,'(2x,a,2i5)')
     *      'Max. no. of iterations reached: ',iti,mxit
        end if
        if(nbm.ge.maxnbm) then
          write(ftape,'(2x,a,2i5)')
     *     'Max. no. of basis vectors reached: ',nbm,maxnbm
        end if
        write(ftape,'(2x,a,i5,a)') 'No convergence achieved with ',
     *    nbm-nsel,' basis vectors.'
        write(ftape,*)
        nbm=nbm-nsel
        iti=iti-1
      end if

*     -----------------------------------------------------------------
*     print final results
*     -----------------------------------------------------------------
900   continue
      call finalout(vvec,rvec,avib,iti,maxnbm,rthres,
     * natoms,nbm,precond,roothmpr,genstart,printall,rabsthres)

*     -----------------------------------------------------------------
*     clean up memory ...
*     -----------------------------------------------------------------
      call wrnctm('akira_control',natoms,normm,maxnbm,
     * '$vibrational basis modes')
      deallocate(trro)
      deallocate(lambda,olambda,vtemp,svec,evec,ovvec,rvec,happ,normm)
      deallocate(pol,aphs)
      deallocate(escf,grad,istepfl,vmach)
      deallocate(ortmod)

      close(lunit)

*     -----------------------------------------------------------------------
*     JN: This normal mode output was somehow messed up (and partly still is)
*     -----------------------------------------------------------------------
      allocate(normm(nvar*nvar))
      normm(:) = 0.0d0
      normm(1:nvar*nbm) = vvec(1:nvar*nbm)

*     -------------------------------------------------------------------
*     xmol output of normal modes and frequencies
*     -------------------------------------------------------------------
      if(xmolout) then
        xmolsc1 = 10.0d0
        call rkeyreal('akira_control','$xmolsc1',xmolsc1)
        call xmol('xmol.XYZ.out',natoms,coord,nbm,normm,avib,xmolsc1,
     *       icharge)
      end if

*     -------------------------------------------------------------------
*     SNF (mass-weighted MOPAC) output of normal modes and frequencies
*     -------------------------------------------------------------------
      if(snfout) then
        call prmopmwnm('snf_normmodes',natoms,nbm,normm,avib,maxnbm)
      end if

*     -------------------------------------------------------------------
*     Turbomole output of normal modes and frequencies
*     -------------------------------------------------------------------
      if(tmout) then
        normm(:) = 0.0d0
        open(lunit,file='tm_normmodes')
        call wrnmodes(natoms,normm,nvar,'$hessian (projected)',lunit)
        do i=1,nbm
          call nmod_m2c(vvec(nvar*(i-1)+1:),natoms,icharge)
        end do
        call wrnmodes(natoms,vvec,nvar,'$vibrational normal modes',
     *   lunit)
        call wfreq(nvar,avib,lunit)
        write(lunit,'(a)') '$end'
        close(lunit)
      end if

*     -------------------------------------------------------------------
*     final memory deallocation
*     -------------------------------------------------------------------
      deallocate(avib,vvec,normm)
      call flush(idbg)

      return
      end
