      subroutine mopacstart(coord,natoms,ic,string)
      implicit none
      integer natoms,nvar,i,j,lu
      integer ic(natoms),ls
      real*8 coord(3,natoms),thres,b2a
      parameter(thres=1.0d-5)
      character*(*) string
      character*2 find_element
      logical flag
      external find_element,b2a
      nvar=3*natoms
      lu=1

*------------------------------------------------
* check correct orientation of molecule
*------------------------------------------------
      flag=.true.
      do i=1,3
        if(abs(coord(i,1)).gt.thres) flag=.false.
      end do
      if((abs(coord(2,2)).gt.thres).or.(abs(coord(3,2)).gt.thres)
     * .or.(abs(coord(3,3)).gt.thres)) flag=.false.
      if(.not.flag) then
        write(*,*) 
        write(*,*) ' Wrong orientation of molecule !'
        write(*,*) ' Orientation needed is as follows: '
        write(*,*) ' Atom 1    0.000000 0.000000 0.000000'
        write(*,*) ' Atom 2      (x)    0.000000 0.000000'
        write(*,*) ' Atom 3      (x)      (y)    0.000000'
        write(*,*) ' Atom 4      (x)      (y)      (z)   '
        write(*,*) 
        write(*,*) ' Please use correct orientation for MOPAC start'
        write(*,*) 
        call serror('mopacstart',3,'Molecule has not the'//
     * ' desired orientation',6)
      end if

*------------------------------------------------
* (re)move old MOPAC output files
*------------------------------------------------
      call mfile('FOR005')
      call mfile('FOR006')
      call rmfile('FOR007')
      call rmfile('FOR011')
      call rmfile('FOR012')
      call rmfile('FORCE1')
      call rmfile('FORCE2')

      call llen(string,ls)
      open(unit=lu,file='FOR005')
      write(lu,'(2A)') string(1:ls),' FORCE XYZ'
      write(lu,*) 
      write(lu,*) 
      do i=1,natoms
        write(lu,100) find_element(ic(i)),(b2a(coord(j,i)),j=1,3)
      end do
      close(lu)
100   format(A2,3(f20.14,' 0 '))
      end

****************************************************************************

      function b2a(val)
* bohr to angstroem conversion
      implicit none
      real*8 b2a,val
#include "fconstants.h"
      b2a=val*bohr2ang
      end

**********************************************************************

      subroutine getncmop(fname,natoms,normm,nbm,vibf,found)
*
*  read normal modes in MOPAC format from fname
*
      implicit none
      integer i,j,natoms,nvar,lu,nbm
      integer ka,kb,kc,min0,nc,la,lb,nr,lc,lf
      real*8 vibf(nbm),normm(natoms*3,nbm),thres
      parameter(thres=1.0d-6)
      character*255 row
      character*(*) fname
      logical found,found2
      nvar=3*natoms
      lu=1
      found=.false.
      call llen(fname,lf)
      open(unit=lu,file=fname(1:lf))
200   read(lu,'(A)',end=999) row
      if((index(row,'root no. ').eq.0).and.
     * (index(row,'ROOT NO. ').eq.0)) goto 200
      found=.true.
      nr=nvar
      nc=nbm
      ka=1
      kc=6
60    kb=min0(kc,nc)
      read(lu,'(A)',end=999) row
      read(lu,'(A)',end=999) row
      read(row,*) (vibf(i),i=ka,kb)
      read(lu,'(A)',end=999) row
      la=1
      lc=40
70    lb=min0(lc,nr)
      do i=la,lb
        read(lu,'(A)',end=999) row
        read(row,130) (normm(i,j),j=ka,kb)
      end do
      if(lb.eq.nr) goto 90
      la=lc+1
      lc=lc+40
      read(lu,'(A)',end=999) row
      go to 70
90    if (kb.eq.nc) goto 999
      ka=kc+1
      kc=kc+6
      if (nr.gt.25) read(lu,'(A)',end=999) row
      do i=1,5
        read(lu,'(A)',end=999) row
      end do
      goto 60
 
999   close(lu)
      if(.not.found) write(*,*) '** getncmop: No normal modes found'
      found2=found
      i=1
110   if(i.gt.3*natoms) goto 120
        j=1
115     if(j.gt.nbm) goto 118
          found2=(found2.or.(normm(i,j).gt.thres))
          if(found2) goto 120
        j=j+1
        goto 115
118   i=i+1  
      goto 110
120   if(.not.found2) write(*,*) '** getncmop: No normal modes found'
      found=found.and.found2
130   format(10x,f10.5,5f12.5)
c
      end
c
c
c*************************************************************
c
      subroutine  prmopmwnm(fname,natoms,nbm,normmode,freq,
     *    maxnbm)
c
      implicit none
c
      character(len=255) fname
      integer natoms,nbm,maxnbm
      real*8 freq(maxnbm),normmode(natoms*3,maxnbm)
      integer ka,kb,kc,la,lb,lc,nr,nc, i,j,out
      out=18
C      nc=ubound(spectrum,1)
C      nr=ubound(spectrum(1)%norm_mode,1)
      nc=nbm
      nr=natoms*3
  
      open(unit=out,file=fname,err=150)
      ka=1
      kc=6
60    kb=min0(kc,nc)
      write (out,100) (i,i=ka,kb)
      if(abs(freq(1)) .ge.1.d-10) 
     *   write (out,110) (freq(i),i=ka,kb)
      write (out,120)
      la=1
      lc=40
70    lb=min0(lc,nr)
      do  i=la,lb
         write (out,130) i,(normmode(i,j),j=ka,kb)
      enddo
      if (lb.eq.nr) go to 90
      la=lc+1
      lc=lc+40
      write (out,140)
      go to 70
90    if (kb.eq.nc) then
         close(out)
         return
      endif
      ka=kc+1
      kc=kc+6
      if (nr.gt.25) write (out,140)
      go to 60
  
100   format (////,3x,9h ROOT NO.,i5,9i12)
110   format (/8x,10f12.5)
120   format (2h  )
130   format (6x,i4,f10.5,10f12.5)
140   format (1h1)
150   write(15,*) 'Could not open file '//fname 
  
      end subroutine prmopmwnm
