#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      SUBROUTINE nodeini

* -------------------------------------------------------------------------
* check virtual machine and see which nodes are there; this subroutine is
* called at the very beginning of the master process, even before the input
* file is read.
*
* comments added by J. Neugebauer, 2007
* -------------------------------------------------------------------------

      use MachineModule

#include "defs.h"
#include "param.h"

#ifdef AKIRA_PVM
#include "fpvm3.h"
#endif

#ifdef AKIRA_MPI
#include "mpif.h"
#endif

      type(type_machine), pointer :: cmach
      integer nnode
      integer newtid,narch,info,newspeed
      integer i,k,l,idum,tid,num_pes
      external num_pes
      character*40 newnode,name
      character*255 newarch
#ifdef AKIRA_MPI
      character*(MPI_MAX_PROCESSOR_NAME) mpiname
#endif

*     --------------------------------------------------------------
*     get number of nodes in virtual machine
*     Note: For PVM, this is what you would get by typing:
*     pvm
*     > conf
*     Since inicomm already starts pvmd in case it was not running
*     before, there should be at least one node already present
*
*     NOTE: masternode will be the first node in the PVM, which
*           is not necessarily the node on which you start the 
*           program if pvmd was already running
*     --------------------------------------------------------------
#ifdef AKIRA_PVM
      call pvmfconfig(nnode,narch,mastertid,masternode,
     *     newarch,newspeed,info)
#endif

#ifdef AKIRA_MPI
      call mpi_comm_size( mpi_comm_world, nnode, info )
      call mpierr('mpi_comm_size','nodeini',info)
      write (idbg,*) '@nodeini@ nnode=',nnode
      call flush(idbg)
      call mpi_get_processor_name(mpiname,l,info)
      call mpierr('mpi_get_processor_name','nodeini',info)
      write (idbg,*) '@nodeini@ running on ',mpiname(1:l)
      call flush(idbg)
      call clean(masternode)
      masternode=mpiname(1:l)
      nnode=nnode-1
#endif
      if (nnode.eq.0) call errex('no nodes found to work on')

      call clean(masterid)
      masterid=masternode
      call llen(masterid,li)

      write (jtape,*) 'master node is ',masternode
      write (idbg,*) 'nodeini| master node is ',masternode
      write (idbg,*) '       | ',nnode,' node(s) in virtual machine'
      call flush(jtape)

*     --------------------------------------------------------
*     loop through nodes that are already present in PVM
*     (note: USE_NODES has not been read at this point)
*     --------------------------------------------------------

      do i=1,nnode
#ifdef AKIRA_PVM
        call pvmfconfig(idum,narch,tid,newnode,newarch,newspeed,info)
#endif
#ifdef AKIRA_MPI
        tid=i
        call clean(newnode)
        call clean(newarch)
        write (newnode,'(I4.4)') i
#endif
        k=1
        do l=1,k
#ifdef AKIRA_PVM
          call add_node(pvmmachlist,newnode,newarch)
#endif
#ifdef AKIRA_MPI
          call add_node(machlist,newnode,newarch)
#endif
          nullify(cmach)
          cmach => current_mach
          cmach%tid = i
          write (idbg,*) 'nodeini| virtual node ',newnode
        end do 
      end do

      write (idbg,*) '       | ###### nodes in virtual machine ######'

#ifdef AKIRA_PVM
      call print_nodes(pvmmachlist,idbg)
#endif
#ifdef AKIRA_MPI
      call print_nodes(machlist,idbg)
#endif
      call flush(idbg)

      return
      end

