      subroutine gredmass(normmod,natoms,ic,redmass)

C calculate reduced mass for a particular normal mode
C written by J. Neugebauer, Uni Erlangen, 2001
 
      implicit none
      integer natoms,i,ic(natoms)
      real*8 normmod(3*natoms),redmass,find_mass
      external find_mass
      redmass=0.0d0
      do i=1,3*natoms
        redmass=redmass+normmod(i)*normmod(i)/find_mass(ic((i+2)/3))
      end do
      redmass=1.d0/redmass
      return
      end 

******************************************************************************

      subroutine nmod_c2m(normmod,natoms,ic)
*
* converts cartesian normal modes to mass-weighted normal modes
*
* J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer natoms,i,ic(natoms)
      real*8 normmod(3*natoms),find_mass,sqmass,sum,zthres
      external find_mass
      parameter(zthres=1.0d-12)
      do i=1,3*natoms
        sqmass=sqrt(find_mass(ic((i+2)/3)))
        normmod(i)=normmod(i)*sqmass
      end do

C renormalize norm modes 
      
      sum=0.0d0
      do i=1,3*natoms
        sum=sum+normmod(i)*normmod(i)
      end do
      sum=sqrt(sum)
      if(sum.le.zthres) sum=1.0d0
      do i=1,3*natoms
        normmod(i)=normmod(i)/sum
      end do
      return
      end 

******************************************************************************
   
      subroutine nmod_m2c(normmod,natoms,ic)
*
* converts mass-weighted normal modes to cartesian normal modes
*
* J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer natoms,i,ic(natoms)
      real*8 normmod(3*natoms),find_mass,sqmass,sum,zthres
      external find_mass
      parameter(zthres=1.0d-12)
      do i=1,3*natoms
        sqmass=sqrt(find_mass(ic((i+2)/3)))
        normmod(i)=normmod(i)/sqmass
      end do

C renormalize norm modes 
      
      sum=0.0d0
      do i=1,3*natoms
        sum=sum+normmod(i)*normmod(i)
      end do
      sum=sqrt(sum)
      if(sum.le.zthres) sum=1.0d0
      do i=1,3*natoms
        normmod(i)=normmod(i)/sum
      end do
      return
      end 

******************************************************************************
   
      subroutine print_nmodes(normmod,natoms)
      implicit none
      integer natoms,i,j,kbuf,i2,i2end,nvar,nd,kend
      real*8 normmod(3*natoms,3*natoms)
      character(len=255) form
      parameter(nd=5)
      form='(i2,i3,5f15.10)'
      nvar=3*natoms
      i2end=nvar/nd
      if(mod(nvar,nd).ne.0) i2end=i2end+1
      do i=1,nvar
        do i2=1,i2end
         kend=i2*nd
         if(i2.eq.i2end) kend=nvar
         kbuf=(i2-1)*nd+1
         write(*,form) i,i2, (normmod(i,j),j=kbuf,kend)
        end do
      end do 
      return
      end


******************************************************************************
   
      integer function ipre(step,naus)

* yields the prefactor for displacement  <step> 

      implicit none
      integer step,natoms,naus,imode,im,idum
      external imode
      if(step.eq.0) then
        ipre=0
      else
        im=imode(step,naus)
        idum=(step-(im-1)*naus)
        ipre=naus/2-(idum-1)
        if(ipre.le.0) ipre=ipre-1
      end if
      end

******************************************************************************
   
      integer function imode(step,naus)

* yields the mode number for displacement <step> 

      implicit none
      integer step,natoms,naus
      imode=(step+naus-1)/naus
      end

*********************************************************************

      subroutine gramschmidt(vec1,vec2,dimv,para,ovl,thres)

*---------------------------------------------------------
* simple Gram-Schmidt orthogonalization
* |vec2_orth> = |vec2> - <vec1|vec2> |vec1>
*---------------------------------------------------------
      implicit none
      integer dimv,i,j
      real*8 vec1(dimv),vec2(dimv),ovl,dotprod,norm
      real*8 thres
      logical para
      external dotprod
      
      para=.false.
      ovl=dotprod(vec1,vec2,dimv)
      do i=1,dimv
        vec2(i)=vec2(i)-ovl*vec1(i)
      end do
      norm=sqrt(dotprod(vec2,vec2,dimv))
      if(norm.gt.thres) then
        call normalize(vec2,dimv)
      else
        do i=1,dimv
          vec2(i)=0.0d0
        end do
        para=.true.
      end if
      end 
 
*********************************************************************

      subroutine skipvec(avec,lvec,nvec,nrem,thres)

*---------------------------------------------------
* skip all vectors with norm < thres
*
* written by J. Neugebauer, Uni Erlangen, 2002
*---------------------------------------------------
      implicit none
      integer ii,j,nvec,nrem,lvec,k,i
      real*8 thres,avec(lvec,nvec),norm,dotprod
      external dotprod

      nrem=0
      i=0
      do ii=1,nvec
        i=i+1
        norm=sqrt(dotprod(avec(1,i),avec(1,i),lvec))
        if(norm.lt.thres) then
          nrem=nrem+1
          do j=i+1,nvec
            do k=1,lvec
              avec(k,j-1)=avec(k,j)
            end do
          end do
          i=i-1
        end if
      end do
      end


**************************************************************************
      subroutine comdist(normm,natoms,ic,dcom)
* check center of mass distortion
      implicit none
      integer i,natoms,ic(natoms),j
      real*8 normm(3,natoms),mtot,cx,cy,cz,find_mass,mi,sqrt,dcom(3)
      intrinsic sqrt
      external find_mass

      mtot=0.0d0
      cx=0.0d0
      cy=0.0d0
      cz=0.0d0
      do i=1,natoms
        mi=find_mass(ic(i))
        mtot=mtot+mi
        cx=cx+normm(1,i)*mi
        cy=cy+normm(2,i)*mi
        cz=cz+normm(3,i)*mi
      end do
*      cx=cx/mtot
*      cy=cy/mtot
*      cz=cz/mtot
      dcom(1) = cx
      dcom(2) = cy
      dcom(3) = cz
      end

**************************************************************************
      double precision function checkcom(normm,natoms,ic)
* check center of mass distortion
      implicit none
      integer i,natoms,ic(natoms),j
      real*8 normm(3,natoms),mtot,cx,cy,cz,find_mass,mi,sqrt
      intrinsic sqrt
      external find_mass

      mtot=0.0d0
      checkcom=0.0d0
      cx=0.0d0
      cy=0.0d0
      cz=0.0d0
      do i=1,natoms
        mi=find_mass(ic(i))
        mtot=mtot+mi
        cx=cx+normm(1,i)*mi
        cy=cy+normm(2,i)*mi
        cz=cz+normm(3,i)*mi
      end do
      cx=cx/mtot
      cy=cy/mtot
      cz=cz/mtot
      checkcom=sqrt(cx*cx+cy*cy+cz*cz)
      end

**************************************************************************

      subroutine calcovl(nvar,normm,normm2,nbm,nbm2,sovl)
*
* calculate normal mode overlap matrix between sets of
* basis vectors normm(nvar,nbm) and normm2(nvar,nbm2)
*
* written by J. Neugebauer, 2004
*
      implicit none
      integer nvar, nbm, nbm2
      real*8 normm(nvar,nbm),normm2(nvar,nbm2),sovl(nbm,nbm2)

      
      call dgemm('T','N',nbm,nbm2,nvar,1.0D0,normm,nvar,
     *    normm2,nvar,0.d0,sovl,nbm)

      end 

**************************************************************************

      logical function checkovl(ovl,nbm,othres)
*
* check normal mode overlap for a particular vector
*
* written by J. Neugebauer, 2004
*
      implicit none
      integer nbm,i
      real*8 ovl(nbm),othres,rovl
  
      checkovl=.true.
      rovl=0.0d0
      do i=1,nbm
        rovl=max(rovl,abs(ovl(i)))
      end do
      if(rovl.gt.othres) checkovl=.false.
      end
