#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      SUBROUTINE parstuff(gra,stepfl,vmach,escf,pol,iti,nbm)

*     ==================================================
*     akira version by J. Neugebauer, Uni Erlangen, 2001
*     modified by J. Neugebauer, 09/2005:
*      - do not shut down nodes if "howmany" results
*        ize = 0 (there might be steps to do in the next
*        iteration)
*
*     migrate to MachineModule by J. Neugebauer, 2007
*     ==================================================

      use MachineModule

#include "defs.h"
#include "param.h"

*
*     g      R8  short gradient buffer (1:nvar)
*     gra    R8  gradient buffer
*     stepfl I4  status flags for each point
*     vmach  R8  machine name for each point
*

      type(type_machine), pointer :: mach
      integer stepfl(*),iti,nbm
      real*8 gra(*),vmach(*)
      real*8 escf(*),pol(*)
      integer i,k,l,idum,mbuf,ize,ipe,iprc,nnode,j

*     ---------------------------------------------------
*     outer loop: check if there are points left to 
*                 calculate (and nodes that could do the
*                 job); sleep for a while in between
*     ---------------------------------------------------
      write(idbg,*) 'parstuff | sleeptime is: ',sleeptime
      call flush(idbg)

      StepLoop_: do

#ifdef DEBUG
        write(idbg,*) 'parstuff| Entering StepLoop'
        call flush(idbg)
#endif

        nullify(mach)
        mach => machlist
      
*       ---------------------------------------------------
*       loop over nodes
*       ---------------------------------------------------
        NodeLoop_: do

#ifdef DEBUG
          write(idbg,*) 'parstuff| Entering NodeLoop'
          call flush(idbg)
#endif

          if(.not.associated(mach)) exit NodeLoop_

          call howmany(ize,ipe,iprc,stepfl)
         
#ifdef DEBUG
          write (idbg,*) '@parstuff ize, ipe, iprc ',ize,ipe,iprc
#endif
          call flush(idbg)
         
*         ---------------------------------------------------
*         spawn process: send input files, directories, etc.
*         ---------------------------------------------------
#ifdef AKIRA_PVM
          if (ize.gt.0) call spawn(mach%numb,jtape)
#endif
          call flush(idbg)
         
*         -----------------------------------------------------------
*         Never shut down nodes in MPI runs: ize is often zero, since
*         stepflags are adjusted in each iteration. If ize is zero,
*         it just means that there are no further points to calculate
*         in this iteration, but we might need the nodes for the
*         next iteration
*         -----------------------------------------------------------
#ifdef AKIRA_MPI
          if (ize.eq.0 .and. mach%nodf == 'rdy') then
            call remnode('shutdown[idle]',0,stepfl,mach%numb)
          end if
#endif
         
#ifdef DEBUG
          write(idbg,*) 'parstuff| Calling fsys()'
          call flush(idbg)
#endif
          call fsys(stepfl)
         
*         ----------------------------------------------
*         look for interactive commands in file DC
*         ----------------------------------------------
#ifdef DEBUG
          write(idbg,*) 'parstuff| Calling readcmd()'
          call flush(idbg)
#endif
          call readcmd
         
*         ----------------------------------------------
*         update of TMPdcstat file
*         ----------------------------------------------
#ifdef DEBUG
          write(idbg,*) 'parstuff| Calling trace_stat()'
          call flush(idbg)
#endif
          call trace_stat
         
*         ----------------------------------------------
*         send "iwhat" flags and steps to slave nodes
*         ----------------------------------------------
#ifdef DEBUG
          write(idbg,*) 'parstuff| Calling ssend()'
          call flush(idbg)
#endif
          call ssend(gra,stepfl,vmach,escf,pol,nbm,mach%numb)
         
*         ----------------------------------------------
*         check for results from this node
*         ----------------------------------------------
#ifdef DEBUG
          write(idbg,*) 'parstuff| Calling getres()'
          call flush(idbg)
#endif
          call getres(gra,stepfl,vmach,escf,pol,nbm,mach%numb)
         
*         ----------------------------------------------
*         check next node
*         ----------------------------------------------
          mach => mach%next
        end do NodeLoop_

#ifdef DEBUG
        write(idbg,*) 'parstuff| Left NodeLoop'
        call flush(idbg)
#endif
      
        call sleep(sleeptime)
      
        call howmany(ize,ipe,iprc,stepfl)
#ifdef DEBUG
          write (idbg,*) '@parstuff_b ize, ipe, iprc ',ize,ipe,iprc
#endif
        call flush(idbg)
      
         
*       ----------------------------------------------
*       exit if no more steps have to be calculated
*       or if no more processors are available
*       ----------------------------------------------
        if (ize+ipe == 0 .or. iprc == 0) exit StepLoop_

      end do StepLoop_

#ifdef DEBUG
      write(idbg,*) 'parstuff| Left StepLoop'
      call flush(idbg)
#endif

      return
      end

