*     -----------------------------------------------
*     migrate to MachineModule by J. Neugebauer, 2007
*     -----------------------------------------------

      subroutine readcmd
*
* read commands to drive the virtual machine from file DC
*
* SHUTDOWN <empty>|<node>|<jobno>   shut down (i.e., run current job
*                                   and deactivate node after it is
*                                   finished
* TERM     <empty>|<node>|<jobno>   terminate node
* REMOVE   <empty>|       <jobno>   remove slot from use
* SPAWN    <empty>|<node>           spawn node or read USE_NODES
* FSYS     <0><1>                   switch off/on fsystem reporting
* CLEAR    <empty>|<node>|<jobno>   clear pending cmds
* UPDATE   <empty>|<opt>            sum of 1 to remove vbsy, 2 to add idle nodes
* TMPDIR   <dirname>                change TMPDIR to <dirname>
* TMPDIR(<node>) <dirname>          change TMPDIR of <node> to <path>
* DCPATH   <path>                   change DCPATH to <dirname>
* DCPATH(<node>) <path>             change DCPATH of <node> to <path>
**

#include "defs.h"
#include "param.h"

      integer lupvm,iostat,irc
      character*255 rec,item(mxitem)
      character*255 filename
      character*24 date
      integer i,k,l,ip,knrec
      logical ex

      call clean(filename)
      filename=mydir(1:lm)//'/DC'

      inquire (file=filename,exist=ex)
      if (.not.ex) return

*     -------------------------------------
*     read file into memory and delete file
*     -------------------------------------
      knrec=0
      lupvm=1
      write(iosl,*) 'readcmd | opening file: ',filename(1:255)
      open (unit=lupvm,file=filename,status='unknown',iostat=irc)
      if (irc.ne.0) return

      do
        knrec=knrec+1
        if (knrec.gt.mxinfi) then
          write (*,*)
     *    '* number of cmd records exceeds maximum of ',mxinfi
          goto 99
        end if
        read (lupvm,'(a)',end=99) incmd(knrec)
      end do

99    knrec=knrec-1
      close (unit=lupvm,status='DELETE')
      do k=1,knrec
        call llen(incmd(k),l)
        call fdate(date)
        write (*,'(3a)') date,' readcmd: ',incmd(k)(1:l)
        call split(incmd(k),item,mxitem)
        call upper(item(1))
        call llen(item(1),l)
        if (item(1)(1:l).eq.'fsys') call cmd_fsys(item)
        if (item(1)(1:l).eq.'shut') call cmd_shut(item)
        if (item(1)(1:l).eq.'shutdown') call cmd_shut(item)
        if (item(1)(1:l).eq.'term') call cmd_kill(item)
        if (item(1)(1:l).eq.'spawn') call cmd_spawn(item(2))
        if (item(1)(1:l).eq.'clear') call cmd_clear(item)
        if (item(1)(1:l).eq.'remove') call cmd_remove(item(2))
        if (item(1)(1:6).eq.'tmpdir') call cmd_addinp(incmd(k))
        if (item(1)(1:6).eq.'dcpath') call cmd_addinp(incmd(k))
        if (item(1)(1:6).eq.'update') call cmd_update(item(2))
      end do

      return
      end

      subroutine cmd_shut(item)
      use MachineModule
      implicit none
      character*255 item(*)
      call cmd_gcmd(item,'shut')
      return
      end

      subroutine cmd_kill(item)
      use MachineModule
      implicit none
      character*255 item(*)
      call cmd_gcmd(item,'term')
      return
      end

      subroutine cmd_clear(item)
      implicit none
      character*255 item(*)
      call cmd_gcmd(item,'----')
      return
      end

      subroutine cmd_spawn(arg)
      use MachineModule
#include "defs.h"
#include "param.h"
      integer info,xeq,lbuf
      character*255 arg,buf
      logical yes

*     ---------------------------------------------------
*     SPAWN without argument: call choose_nodes and rduse
*     ---------------------------------------------------
      if (arg.eq.' ') then
        if(lnopt.le.1) then
          info=xeq('choose_nodes')
        else
          write(buf,'(a)') 'choose_nodes '//cnopt(1:lnopt)
          call llen(buf,lbuf)
          info=xeq(buf(1:lbuf))
        end if
        call rduse(yes)

*     ---------------------------------------------------
*     SPAWN with argument: spawn <arg> directly
*     ---------------------------------------------------
      else
        call add_node(machlist,arg,' ')

      end if
      return
      end


      subroutine cmd_fsys(item)
      implicit none
      character*255 item(*)
#include "param.h"
      integer i,intgr
      i=1
      iprintf=intgr(item(2),i,20)
      return
      end

      subroutine cmd_remove(ans)
      use MachineModule
#include "defs.h"
      type(type_machine), pointer :: mach,cmach
      character*255 ans
#include "param.h"
      integer i,intgr,info
      if (ans.eq.' ') then
        nullify(mach)
        mach => machlist
        do
          if(.not.associated(mach)) return
          select case(mach%nodf)
          case('dead','unf','pvme')
            mach%nodf = 'remo'
          end select
          mach => mach%next
        end do
      end if
      i=1
      info=intgr(ans,i,20)
      nullify(cmach)
      call findnode_by_num(cmach,info)
      if(associated(cmach)) then
        select case(cmach%nodf)
        case('dead','unf','pvme')
          cmach%nodf = 'remo'
        end select
      end if
      return
      end

      subroutine cmd_addinp(rec)
      implicit none
#include "param.h"
      character*255 rec
      integer i,l1,l2,info
      character*255 item(2),jtem(2)

*     ------------------------------------------------------
*     check if same command is already present on input file
*     ------------------------------------------------------
      call split(rec,item,2)
      call llen(item(1),l1)
      do i=1,inrec
       call split(infi(i),jtem,2)
       call llen(jtem(1),l2)
       if (item(1)(1:l1).eq.jtem(1)(1:l2)) then
        write (*,*) 'cmd_addinp: replacing input record'
        call llen(infi(i),l2)
        call llen(rec,l1)
        write (*,*) 'old: ',infi(i)(1:l2)
        write (*,*) 'new: ',rec(1:l1)
        infi(i)=rec
        return
       endif
      enddo
      inrec=inrec+1
      if (inrec.gt.mxinfi) then
       write (*,*)
     *   '* number of input records exceeds maximum of ',mxinfi
       write (*,*) '* increase mxinfi in util.r[readin] and recompile'
       call errex('too many input records')
      endif
      infi(inrec)=rec
      return
      end

      subroutine cmd_gcmd(item,myncmd)
      use MachineModule
#include "defs.h"
      type(type_machine), pointer :: mach,cmach
      character*(*) myncmd
      character*255 item(*)
#include "param.h"
      integer i,l,intgr,thejob
      logical numric

      if (item(2).eq.' ') then
        nullify(mach)
        mach => machlist
        do
          if(.not.associated(mach)) return
          mach%cmd = trim(myncmd)
          mach => mach%next
        end do
      end if
      if (numric(item(2)(1:1))) then
        i=1
        thejob=intgr(item(2),i,20)
        nullify(cmach)
        call findnode_by_num(cmach,thejob)
        if(associated(cmach)) then
          mach%cmd = trim(myncmd)
        else
          write (jtape,*) '-------> job #',thejob,' not found'
        end if
        return
      end if
      call llen(item(2),l)
      nullify(mach)
      mach => machlist
      
3     call findnode_by_name(mach,cmach,item(2))
        if(.not.associated(cmach)) return
        mach%cmd = trim(myncmd)
        mach => mach%next
      goto 3
      end

      subroutine cmd_update(rec)
#include "defs.h"
      character*255 rec
#include "param.h"
      integer i,l,intgr
      logical numric
      if (rec.eq.' ') then
        updopt=3
        call updvm(updopt)
        return
      end if
      updopt=intgr(rec,i,20)
      call updvm(updopt)
      return
      end
