 
      subroutine remod(fname,nbmf,tmpmod,vibf,found,natoms,ic,coord)
 
*-------------------------------------------------------------------
* read normal modes/basis vectors from file
*
* formats supported by remod:
*
*  TURBOMOLE format (either as normal modes or as "basis modes")
*  MOPAC     format (Cartesian normal modes will be read)
*  SNF       format (mass-weighted MOPAC format; SNF always prints
*                    mass-weighted normal modes in <snf.out>)
*  PCM       format (mass-weighted normal modes will be read)
*  ADF       format (Cartesian normal modes will be read)
*  G98       format (Cartesian normal modes)
*  FREE      format (Cartesian normal modes)
*
* written by J. Neugebauer, Uni Erlangen, 2002
* 
* changes:
* ADF extemsion, J. Neugebauer, 10.09.2003
*
*-------------------------------------------------------------------
      implicit none
      integer nbmf,lf,lu,lrow,i,natoms,ic(natoms)
      real*8 tmpmod(3*natoms,nbmf),vibf(3*natoms),coord(3,natoms)
      character*(*)fname
      character*132 row
      logical found

      lu=1
      found=.false.
      call llen(fname,lf)
      call clean(row)
      if(index(fname,'snf.out').ne.0) then
        write(*,*) '** assuming SNF format'
        row(1:3)='snf'
        goto 110
      else if(index(fname,'g98.out').ne.0) then
        write(*,*) '** assuming G98 format'
        row(1:3)='g98'
        goto 110
      else if(index(fname,'freq.adf').ne.0) then
        write(*,*) '** assuming ADF format'
        row(1:3)='adf'
        goto 110
      end if

*-------------------------------------------------------------------
* print possible commands
*-------------------------------------------------------------------
100   write(*,*)
      write(*,*) ' Select format of basis vectors'
      write(*,*) ' ------------------------------'
      write(*,*)
      write(*,*) '   snf      : SNF normal modes (massweighted MOPAC)'
      write(*,*) '   tmn      : TURBOMOLE normal modes'
      write(*,*) '   tmb      : Basis vectors in TURBOMOLE format'
      write(*,*) 
     *          '   tmi      : Internal coordinates in TURBOMOLE format'
      write(*,*) '   g98      : Gaussian98 normal modes'  
      write(*,*) '   mop      : MOPAC normal modes (not massweighted)'
      write(*,*) '   pcm      : PCM normal modes (massweighted)'
      write(*,*) '   adf      : ADF normal modes (not massweighted)'
      write(*,*) '   free     : free format modes (not massweighted)'
      write(*,*) '   <return> : return to main menu'
      write(*,*)
*-------------------------------------------------------------------
* read commands
*-------------------------------------------------------------------
      call clean(row)
      read(*,'(A)') row
110   call slen(row,lrow)
      if(lrow.eq.0) then
        nbmf=0
        return
*-------------------------------------------------------------------
* TURBOMOLE normal modes
*-------------------------------------------------------------------
      else if(row(1:3).eq.'tmn') then
        call getnctm(fname(1:lf),natoms,tmpmod,
     *    '$vibrational normal modes',nbmf)
        call getfrtm(fname(1:lf),natoms,vibf,found)
        return
*-------------------------------------------------------------------
* TURBOMOLE basis modes as printed by RNF/AKIRA
*-------------------------------------------------------------------
      else if(row(1:3).eq.'tmb') then
        call getnctm(fname(1:lf),natoms,tmpmod,
     *    '$vibrational basis modes',nbmf)
        return
*-------------------------------------------------------------------
* free format normal modes
*-------------------------------------------------------------------
      else if(row(1:4).eq.'free') then
        call getfreefnm(fname(1:lf),natoms,tmpmod,nbmf,vibf,found)
        return
*-------------------------------------------------------------------
* Gaussian98 normal modes
*-------------------------------------------------------------------
      else if(row(1:3).eq.'g98') then
C        nbmf=nbmf-6
        call rgaussian(fname(1:lf),tmpmod,natoms,vibf,nbmf)
*-------------------------------------------------------------------
* note: to use Cartesian Hessian as preconditioner, we also need
* translational and rotational modes which are not obtained from
* g98 output
*-------------------------------------------------------------------
C        call trarot(coord,tmpmod(1,nbmf+1),tmpmod(1,nbmf+4),ic,
C     * natoms)
C        do i=nbmf+1,nbmf+6
C          vibf(i)=0.0d0
C          call nmod_m2c(tmpmod(1,i),natoms,ic)
C        end do
C        nbmf=nbmf+6
        found = .true.
        return            
*-------------------------------------------------------------------
* SNF output, i.e., mass-weighted MOPAC format
*-------------------------------------------------------------------
      else if(row(1:3).eq.'snf') then
        nbmf=nbmf-6
        call getncmop(fname(1:lf),natoms,tmpmod,nbmf,vibf,found)
C        call printdmat(tmpmod,3*natoms,nbmf,'getncmop output',
C     *    'f20.12',6)
        do i=1,nbmf
          call nmod_m2c(tmpmod(1,i),natoms,ic)
        end do
*-------------------------------------------------------------------
* note: to use Cartesian Hessian as preconditioner, we also need
* translational and rotational modes which are not obtained from
* SNF output
*-------------------------------------------------------------------
        call trarot(coord,tmpmod(1,nbmf+1),tmpmod(1,nbmf+4),ic,
     * natoms)
        do i=nbmf+1,nbmf+6
          vibf(i)=0.0d0
          call nmod_m2c(tmpmod(1,i),natoms,ic)
        end do 
        nbmf=nbmf+6
        return
*-------------------------------------------------------------------
* ADF output
*-------------------------------------------------------------------
      else if(row(1:3).eq.'adf') then
        nbmf=nbmf-6
        call getncadf(fname(1:lf),tmpmod,natoms,vibf,nbmf,found)
        call printdmat(tmpmod,3*natoms,nbmf,'getncadf output',
     *    'f20.12',6)
*-------------------------------------------------------------------
* note: to use Cartesian Hessian as preconditioner, we also need
* translational and rotational modes which are not obtained from
* ADF output
*-------------------------------------------------------------------
        call trarot(coord,tmpmod(1,nbmf+1),tmpmod(1,nbmf+4),ic,
     * natoms)
        do i=nbmf+1,nbmf+6
          vibf(i)=0.0d0
          call nmod_m2c(tmpmod(1,i),natoms,ic)
        end do 
        nbmf=nbmf+6
        return
*-------------------------------------------------------------------
* MOPAC normal modes (not mass-weighted)
*-------------------------------------------------------------------
      else if(row(1:3).eq.'mop') then
        call getncmop(fname(1:lf),natoms,tmpmod,nbmf,vibf,found)
        return
      else if(row(1:3).eq.'pcm') then
        call getncpcm(fname(1:lf),natoms,tmpmod,nbmf,vibf,found)
        do i=1,nbmf
          call nmod_m2c(tmpmod(1,i),natoms,ic)
        end do
        return
      end if
      goto 100
910   write(*,'(A)') ' Invalid input ...'
      write(*,*)
      goto 100
      open(unit=lu,file=fname(1:lf))
      close(lu)
      end



