      subroutine roothome(isel,iti,ir,nbm,lam,olam,vvec,ovvec,normm,
     *  vtemp,rvec,onsel)
*-------------------------------------------------------------------
* root homing procedure for Jacobi-Davidson/Davidson/Lanczos
* diagonalization
* 
* root homing is done by: - comparison of overlaps of eigenvectors
*                         - least-squaeres deviation of eigenvectors
*                         - comparison of eigenvalues
*
* if the results differ, the overlap criterion will be applied
*                           
* written by J. Neugebauer, 2004
*-------------------------------------------------------------------

#include "defs.h"
#include "param.h"

      integer ir,isel(mxnsel),iti,isel2,isel3,nbm,i,j,k,ipos,onsel
      real*8 lam(maxnbm),olam(maxnbm),vvec(nvar,maxnbm),
     *       normm(nvar,maxnbm),ovvec(nvar,maxnbm),vtemp(nvar),
     *       mxres,rvec(nvar,maxnbm),mxres1

      write(ftape,'(2x,a)')
      write(ftape,'(2x,a)')
     *  '----------------------------------------------'
      write(ftape,'(2x,a,i5)') 'Root homing for selected root No. ',ir
      write(ftape,'(2x,a,i5)') 'in iteration No. ',iti
      write(ftape,'(2x,a)')
     *  '----------------------------------------------'
      write(ftape,'(2x,a)')

*----------------------------------
* special root homing cases
* a) vectors with smallest residuals
*----------------------------------
      if(roothmpr.eq.'minresi') then
        do k=1,nbm
          mxres1=0.0d0
          do i=1,nvar
            mxres1=max(mxres1,abs(rvec(i,k)))
          end do
          ipos=1
          do 100 j=1,nbm
            if(j.eq.k) goto 100
            mxres=0.0d0
            do i=1,nvar
              mxres=max(mxres,abs(rvec(i,j)))
            end do
            if(mxres1.gt.mxres) ipos=ipos+1
100       continue
          if(ipos.eq.ir) then
            isel(ir)=k
            write(ftape,'(2x,a,i5,a)')
     *       'Selected eigenvector No.: ',isel(ir),
     *       ' by looking for smallest residuals'
            return
          end if
        end do
*---------------------------
* b) find minimum eigenvalue
*---------------------------
      else if(roothmpr.eq.'testmin') then
        isel(ir)=ir
        write(ftape,'(2x,a)') 'Selected lowest eigenvectors '
     *   //'because of testmin option'
        return
*---------------------------
* c) choose largest eigenvalues
*---------------------------
      else if(roothmpr.eq.'maxfreq') then
*CH find the eigenvector with (onsel-ik+1)th-largest eigenvalue,
*CH which should be identical to the largest eigenvalue
*CH not yet selected in this iteration.
*CH assuming that normal modes are ordered according
*CH to ascending eigenvalues
        isel(ir)=nbm-(onsel-ir)
        write(ftape,'(2x,a9,i4,a)') 'Selected ',onsel-ir+1,
     *    'th-largest eigenvalue because of maxfreq option'
        return
      end if

*---------------------------------------------------------------------
* set isel(ir)=ir for comparison with initial guess vectors (trackgv);
* in that case, comparison is made to the first basis modes, not to
* to the last eigenvector approximations
* (note: eigenvalues are always compared to last iteration)
*---------------------------------------------------------------------

      isel2=isel(ir)
      if(roothmpr.eq.'trackgv') isel(ir)=ir
      isel3=isel(ir)
      if(iti.gt.1) then

*----------------------------------------------------------------
* ... comparison of eigenvalues ...
*----------------------------------------------------------------

        call selectlam(lam,olam,nbm,isel2)
        write(ftape,*)
        write(ftape,'(2x,a,i5,a)')
     *   'Selected eigenvector No.: ',isel2,
     *   ' by comparison of eigenvalues'

*----------------------------------------------------------------
* ... comparison of overlaps ...
*----------------------------------------------------------------

        if(roothmpr.eq.'trackgv') then
          call selectovl(vvec,normm,nbm,isel(ir),nvar)
          write(ftape,'(2x,a,i5,a)')
     *     'Selected eigenvector No.: ',isel(ir),
     *     ' by comparison of overlaps with initial guess'
        else
          call selectovl(vvec,ovvec,nbm,isel(ir),nvar)
          write(ftape,'(2x,a,i5,a)')
     *     'Selected eigenvector No.: ',isel(ir),
     *     ' by comparison of overlaps with last selection'
        end if

*----------------------------------------------------------------
* ... comparison of eigenvectors ...
*----------------------------------------------------------------
        if(roothmpr.eq.'trackgv') then
          call selectvec(vvec,normm,nbm,isel3,nvar,vtemp)
        else
          call selectvec(vvec,ovvec,nbm,isel3,nvar,vtemp)
        end if
        write(ftape,'(2x,a,i5,a)')
     *   'Selected eigenvector No.: ',isel3,
     *   ' by comparison of eigenvectors'
      end if

*----------------------------------------------------------------
* if selected vectors differ, choose the one selected by
* overlaps
*----------------------------------------------------------------

      if((isel(ir).ne.isel2).or.(isel3.ne.isel2)) then
        call serror('roothome',0,'Modes selected from'
     *  //' eigenvalues, eigenvectors, and overlaps differ.##'
     *  //'I will use those from overlaps',ftape)
      end if
      write(ftape,'(2x,a,i5)')
     *  'Finally selected eigenvector No.: ',isel(ir)
      write(ftape,'(2x,a)')
     *  '----------------------------------------------'
      write(ftape,*)

      end

***********************************************************************

      subroutine selectlam(lam,olam,nbm,isel)
*----------------------------------------------------------
* select mode to be optimized in davidson procedure by
* comparison of eigenvalues (this might cause problems for
* near degeneracies ... better use selectvec or selectovl)
* most primitive form of "root-homing"
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------
      implicit none
      integer nbm,isel,i,j,oisel
      real*8 lam(nbm),olam(nbm),mindiff

      oisel=isel
      isel=1
      mindiff=abs(lam(1)-olam(oisel))
      do i=2,nbm
        if(abs(lam(i)-olam(oisel)).lt.mindiff) then
          mindiff=abs(lam(i)-olam(oisel))
          isel=i
        end if
      end do
      end

***********************************************************************

      subroutine selectvec(vvec,ovvec,nbm,isel,nvar,vtemp)
*----------------------------------------------------------
* select mode to be optimized in davidson procedure by
* comparison of eigenvectors (root-homing)
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------
#include "defs.h"
      integer nbm,isel,i,j,oisel,nvar
      real*8 vvec(nvar,nbm),ovvec(nvar,nbm)
      real*8 dotprod,vtemp(nvar),mindiff,diff,s1,s2
      external dotprod

      write(idbg,*) 'selectvec: ------------------------------'
      write(idbg,*) 'selectvec: nbm = ',nbm
*--------------------------------------------
* determine norm of difference vector
*--------------------------------------------
      oisel=isel
      isel=1
      do j=1,nvar
        vtemp(j)=vvec(j,1)-ovvec(j,oisel)
      end do
      s1=dotprod(vtemp,vtemp,nvar)
C      write(idbg,*) 'selectvec: s1_start = ',s1

*--------------------------------------------
* signs may differ, so check this one ...
*--------------------------------------------
      do j=1,nvar
        vtemp(j)=-vvec(j,1)-ovvec(j,oisel)
      end do
      s2=dotprod(vtemp,vtemp,nvar)
C      write(idbg,*) 'selectvec: s2_start = ',s2
      mindiff=min(s1,s2)
C      write(idbg,*) 'selectvec: mindiff  = ',mindiff

*--------------------------------------------
* now compare with all other possible vectors
*--------------------------------------------
      do i=2,nbm
        do j=1,nvar
          vtemp(j)=vvec(j,i)-ovvec(j,oisel)
        end do
        s1=dotprod(vtemp,vtemp,nvar)
C        write(idbg,*) 'selectvec: s1 = ',s1,' for i =',i
        do j=1,nvar
          vtemp(j)=-vvec(j,i)-ovvec(j,oisel)
        end do
        s2=dotprod(vtemp,vtemp,nvar)
C        write(idbg,*) 'selectvec: s2 = ',s2,' for i =',i
        diff=min(s1,s2)
C        write(idbg,*) 'selectvec: diff = ',diff,' for i =',i
        if(diff.lt.mindiff) then
          mindiff=diff
C          write(idbg,*) 'selectvec: new mindiff  = ',mindiff
          isel=i
        end if
      end do
      end

******************************************************************

      subroutine  selectovl(vvec,ovvec,nbm,isel,nvar)
*----------------------------------------------------------
* select mode to be optimized in davidson procedure by
* comparison of overlaps with old selected eigenvector
* (root-homing)
* (c.f., e.g., C. W. Murray, S. C. Racine, E. Davidson,
* J. Comp. Phys. 103, 382 (1992); W. Butscher,
* W. J. E. Kammer, J. Comput. Phys. 20, 13 (1976))
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------
#include "defs.h"
      integer nbm,isel,i,oisel,nvar
      real*8 vvec(nvar,nbm),ovvec(nvar,nbm)
      real*8 dotprod,s1,s2
      external dotprod

      oisel=isel
C      write(idbg,*) 'selectovl: ------------------------------'
C      write(idbg,*) 'selectovl: nbm = ',nbm

*--------------------------------------------
* calculate overlap for first vector
*--------------------------------------------
      isel=1
      s1=abs(dotprod(ovvec(1,oisel),vvec(1,1),nvar))
C      write(idbg,*) 'selectovl: s1_start = ',s1

*--------------------------------------------
* now compare with all other possible vectors
*--------------------------------------------
      do i=2,nbm
        s2=abs(dotprod(ovvec(1,oisel),vvec(1,i),nvar))
C        write(idbg,*) 'selectovl: overlap = ',s1,' for i =',i
        if(s2.gt.s1) then
          s1=s2
C          write(idbg,*) 'selectovl: new max overlap = ',s1
          isel=i
        end if
      end do
      end

