#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      SUBROUTINE rslave(co,ilab,normmodes,jstepa,iclean)

**************************************************
* akira version by J. Neugebauer, Uni Erlangen, 2001
* slightly adapted for excited state gradients by
* C. Herrmann, Uni Jena 04/2005
**************************************************

#include "defs.h"
#include "param.h"

#ifdef AKIRA_PVM
#include "fpvm3.h"
#endif

#ifdef AKIRA_MPI
#include "mpif.h"
#endif

      real*8, allocatable :: dco(:)

      integer ibuf,info,nn,idir,l,mypid,mbuf,iclean,icleanl,im
      integer i,imode,ipre,j
      integer*4 itst,jstepa
      real*8 co(3,natoms),zthres, norm
      real*8 normmodes(nvar*maxnbm)
      INTEGER ilab(natoms),disptype
      logical valdtype
      external imode,ipre,valdtype,norm
      parameter (zthres=0.001d0)
      character(len=10),allocatable :: elinfo(:)

#ifdef AKIRA_MPI
      INTEGER w,st(MPI_STATUS_SIZE)
      DATA w /MPI_COMM_WORLD/
#endif

      ipid=0
      masstype=1
      disptype=1

*-----------------------------------------------------
c receive step range
*-----------------------------------------------------
#ifdef AKIRA_MPI
      CALL MPI_RECV(itst,1,MPI_INTEGER,tid0,410,w,st,info)
      CALL mpierr('MPI_RECV itst','rslave',info)
      CALL MPI_RECV(jstepa,1,MPI_INTEGER,tid0,411,w,st,info)
      CALL mpierr('MPI_RECV jstepa','rslave',info)
#endif
#ifdef AKIRA_SERIAL
      jstepa=isstep
#endif
#ifdef AKIRA_PVM
      CALL PVMFRECV(tid0,0,ibuf)
      CALL PVMFUNPACK(INTEGER4,itst  ,1,1,info)
      CALL PVMFUNPACK(INTEGER4,jstepa,1,1,info)
#endif

      WRITE (iosl,*) '>>> slave calc on tid ',mytid
      WRITE (iosl,*) 'TID0=',tid0,' STEP ',jstepa

*-----------------------------------------------------
* read akira_control (and control, if necessary)
*-----------------------------------------------------

       call rakiractrl(tempdir(1:lt)//'/akira_control',rthres,rabsthres,
     *   dvthres,ecthres,
     *  roothmpr,g98it,g98bs,xmolout,tmout,printall,precond,genstart,
     *  rchthres,minnbt,iorth,snfout,natoms,mode,cstep,naus,iclean,
     *  icleanl,idbg,ipol,lastqmatom,irescp,restadf,sleeptime)

      if(mode.lt.20.or.(mode.ge.60.and.mode.lt.70)) then
        write(iosl,*) ' * rslave: calling rcontrol'
        call flush(iosl)
        if(mode.lt.20) then
          CALL rcontrol(tempdir(1:lt)//'/control',rpast,smult,natoms,
     *    jscfc,isuhf,riflag)
        else
          CALL rcontrol(tempdir(1:lt)//'/control',rpast,smult,
     *    lastqmatom,jscfc,isuhf,riflag)
        end if
        write(iosl,*) ' * rslave: back from rcontrol'
      end if
      write(iosl,*) ' * rslave: lastqmatom = ', lastqmatom
      call flush(iosl)
      call rkeyint(tempdir(1:lt)//'/akira_control','$masstype',masstype)
      write(iosl,*) ' * rslave: back from rkeyint'
      call flush(iosl)

*-----------------------------------------------------
* read normal coordinates (not mass-weighted)
*-----------------------------------------------------

      write(iosl,*) 'rslave: starting getnctm'
      call flush(iosl)
      call getnctm(tempdir(1:lt)//'/akira_control',natoms,normmodes,
     * '$vibrational basis modes',maxnbm)
      write(iosl,*) 'rslave: back from getnctm'
      call flush(iosl)

*-----------------------------------------------------
* extract coords
*-----------------------------------------------------

      write(iosl,*) 'rslave: start reading equilibrium coordinates'
      call flush(iosl)
      if(mode.lt.60) then
        call rcoord(tempdir(1:lt)//'/coord.ori',nn,co,ilab)
        if(nn.ne.natoms) then
           write(iosl,*) 'natoms different on coord and control'
#ifdef AKIRA_SERIAL
           call errex('natoms different on coord and control')
#endif
#ifdef AKIRA_PVM
           CALL pvmfexit(info)
#endif
           stop
        end if
      else
         call readcomqum(tempdir(1:lt)//'/prmcrd3.ori',tempdir(1:lt)//
     *   '/prmtop3',natoms,co,ilab,maxatoms)
!CH         call rcoord(tempdir(1:lt)//'/coord.ori',lastqmatom,co,ilab)
      endif
      write(iosl,*) 'rslave: back from reading equilibrium coordinates'
      call flush(iosl)

*-----------------------------------------------------
* determine mode no., and prefactor for displacement
* along this mode
*-----------------------------------------------------

* mode number:
      im=imode(jstepa,naus)

* prefactor for displacement:
      idir=ipre(jstepa,naus)

      write(iosl,*) 'step       =        ',jstepa
      write(iosl,*) 'mode no.   =        ',im
      write(iosl,*) 'prefactor  =        ',idir

      write(iosl,*) 'mode = ', mode

      i=im

*---------------------------------------------------------
* normalize normal coordinates (in principle, there should
* be normalized normal coordinates in "control")
*---------------------------------------------------------

         write(iosl,*) 'rslave: calling normalize, norm is:'
         write(iosl,*) norm(normmodes(3*natoms*(i-1)+1),3*natoms)
         call flush(iosl)
         if(norm(normmodes(3*natoms*(i-1)+1),3*natoms).lt.zthres) then
           call errex('norm of normal modes too small')
         end if
         call normalize(normmodes(3*natoms*(i-1)+1),3*natoms)
         write(iosl,*) 'rslave: back from normalize'

c--------------------------
* restore MO vector
c--------------------------
         if(mode.lt.30.or.mode.eq.60) then
           IF (isuhf.EQ.0) THEN
           write(iosl,*)'rslave: calling getmos for mos'
            CALL getmos('mos',jstepa,mypid)
           write(iosl,*)'rslave: back from getmos for mos'
           ELSE
           write(iosl,*)'rslave: calling getmos for alpha'
            CALL getmos('alpha',jstepa,mypid)
           write(iosl,*)'rslave: calling getmos for beta'
            CALL getmos('beta',jstepa,mypid)
           write(iosl,*)'rslave: back from getmos for beta'
           ENDIF
         end if

*-------------------------------------------------------------------
* change coordinate
*-------------------------------------------------------------------

      call rkeyint(tempdir(1:lt)//'/akira_control','$disptype',disptype)
      write(iosl,*) 'disptype: ',disptype
      if(.not.valdtype(disptype)) call errex('disptype not valid')

      allocate(dco(3*natoms))
      if(disptype.eq.1) then
        call displacecoord(normmodes,natoms,co,ilab,im,dco,cstep,idir,
     &   maxnbm)
      end if

c---------------------------------------
c write displaced coordinates
c---------------------------------------
      if(mode.lt.60) then
        call wcoord(tempdir(1:lt)//'/coord',natoms,dco,ilab)
      else
        call writecomqum(tempdir(1:lt)//'/prmcrd3',natoms,dco)
        write(iosl,*) 'rslave; calling atomorder_comqum_mm2qm ...'
        call flush(iosl)
        allocate(elinfo(lastqmatom))
        call atomorder_comqum_mm2qm(tempdir(1:lt)//'/comqum.dat',
     *       tempdir(1:lt)//'/coord.ori',natoms,dco,elinfo,lastqmatom)
        write(iosl,*) 'rslave; back from atomorder_comqum_mm2qm ...'
        call flush(iosl)
        call writecomqum(tempdir(1:lt)//'/prmcrd1',lastqmatom,dco)
        call wcoordcq(tempdir(1:lt)//'/coord',lastqmatom,dco,elinfo)
        deallocate(elinfo)
      end if
      deallocate(dco)

      end

c*******************************************************************

      subroutine rmoldscr(tempdir,mode)
****
* some old TURBOMOLE scratch files may disturb a new calculation in
* the same directory. however, we do not want to create a new directory
* for every step. Therefore, all old scratch files are deleted.
*
* written by J. Neugebauer, Uni Erlangen, 2002
****

#include "defs.h"

      integer maxn
      parameter(maxn=30)
      integer mode,lf,nfiles(maxn),xeq,lt,info,i
      character*(*) tempdir
      character*(255) tempfile
      character*10 scrf(10,3)
      logical ex
      external xeq

      data (scrf(i,1),i=1,3) / 'blockpar  ','hessian   ','ddip      ' /
      data (scrf(i,3),i=1,4) / 'dipl_a    ','nconvx    ','nrestx    ',
     *                         'ndipol    ' /
      data nfiles /3,0,4,0,0,0,0,0,0,0,
     *             0,0,0,0,0,0,0,0,0,0,
     *             0,0,0,0,0,0,0,0,0,0/
      if(mode.gt.maxn) return
      do i=1,nfiles(mode)
        call clean(tempfile)
        call slen(tempdir,lt)
        write(tempfile,'(3a)') tempdir(1:lt),'/',scrf(i,mode)
        call slen(tempfile,lf)
        inquire(file=tempfile(1:lf),exist=ex)
        if(ex) then
          write(iosl,*) 'rmoldscr | deleting old scratch file:'
          write(iosl,*) '         | '//tempfile(1:lf)
          info=xeq('rm '//tempfile(1:lf))
        else
          write(iosl,*) 'rmoldscr | no such scratch file:'
          write(iosl,*) '         | '//tempfile(1:lf)
        end if
      end do
      end

      INTEGER FUNCTION xprog(p,f)

#include "defs.h"
#include "param.h"

      CHARACTER*(*) p,f
      CHARACTER*255 cmdout
      INTEGER info,tmpxeq
      CHARACTER*24 date
      CALL fdate(date)
      CALL concat(p,' >',cmdout)
      CALL appba(cmdout,f)
#ifdef DEBUG
      WRITE (iosl,*) date,cmdout
#endif
      info=tmpxeq(cmdout)
#ifdef DEBUG
      WRITE (iosl,*) date,' RC=',info
#endif
      IF (info.NE.0) THEN
       WRITE (iosl,*) p,' failed; process shut down. mytid=',mytid
       call flush(iosl)
       CALL slshutdown(99,tid0,mytid)
      ENDIF
      call flush(iosl)
      xprog=info
      return
      end
