      subroutine wadf(tdr,natoms,ic,co,iosl)
*
*-------------------------------------------------------------------
*  
*  generates ADF-input file 'adfslave.in' on slaves
*  'adfslave.in' differs from 'adf.in' (adf input file provided
*  by the user) only by 
*  a) the coordinates (which are replaced by an elongated 
*     structure) and
*  b) leaving out the 'Unit' block, if present in 'adf.in'
*
*  written by C.Herrmann, University of Bonn, 01/2005
*  modified by J. Neugebauer, 08/2005
*  
*-----------------------------------------------------------------------
*      
#include "fconstants.h"
c      
      integer gunit,aunit,natoms,ic(*),ltd,i,j,ibefore
      integer iend,k,maxrow,mxitem,iafter,info,lsi,system
      parameter(gunit=10,aunit=11,maxrow=100000,mxitem=10)
      real*8 co(3,*)
      character*(*) tdr
      character*(132) row(maxrow),item(mxitem),scratch,uitem
      character*2 find_element
      external find_element
      character*255 fname, aname
      logical lsym
      integer lrow(maxrow)
c            
      call llen(tdr,ltd)
      call clean(aname)
      call clean(fname)
      aname=tdr(1:ltd)//'/adf.in'
      fname=tdr(1:ltd)//'/adfslave.in'
      write(iosl,*) 'wadf: NATOMS = ', natoms
      call flush(iosl)

c
      lsym=.false.
      i = 1
      open(aunit,file=aname,status='unknown')
10    read(aunit,'(a)',end = 35) row(i)
      write(iosl,*)  row(i) 
      call flush(iosl)
      call llen(row(i),lrow(i))
      call split(row(i),item,mxitem)
      call llen(item(1),lsi)
      uitem(1:lsi)=item(1)(1:lsi)
      call upper(uitem)
c read over equilibrium atom coordinates
      if (uitem(1:lsi).eq.'ATOMS') then
c mark beginning of coordinate section
        ibefore=i
        i = i+1
20      read(aunit,'(a)') row(i)
        call split(row(i),item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
c mark end of coordinate section
          iafter = i+1
          goto 30
        end if
        i=i+1
        goto 20
c if there is a unit block specifying other than standard units (Angstrom),
c leave it out
      else if(uitem(1:lsi).eq.'UNITS') then
CCH        i = i-1
        do k = 1,maxrow
25      read(aunit,'(a)') scratch
        call split(scratch,item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
          i=i-1
          goto 30
        else
          goto 25
        end if
        end do
      else if(uitem(1:lsi).eq.'SYMMETRY') then
c JN: note: even if SYMMETRY key is not found, SYMMETRY NOSYM should be included !!
        call clean(row(i))
        row(i)='SYMMETRY NOSYM'
        call llen(row(i),lrow(i))
        lsym=.true.
      end if
30    continue
      i=i+1
      goto 10
35    continue
      close(aunit)
      iend=i-1
c
      open(unit=gunit,file=fname,status='unknown')
CCH writing rows in adf-input file with blanks at the end 
CCH makes ADF crash for some reason ... 
C      write(gunit,'(a)') (row(i)(1:lrow(i)),i=1,ibefore)
      do i=1,ibefore
        write(gunit,'(a)') row(i)(1:lrow(i))
      end do
      do i=1,natoms
        write(gunit,40) find_element(ic(i)), (co(j,i)*bohr2ang,j=1,3)
      end do
40    format(a4,2x,3F18.12)
      write(gunit,'(a)') 'End'
      if(.not.lsym) then
        write(gunit,'(a)') 
        write(gunit,'(a)') 'SYMMETRY NOSYM'
        write(gunit,'(a)') 
      end if
      do i=iafter,iend
        write(gunit,'(a)') row(i)(1:lrow(i))
      end do
C      write(gunit,'(a)') (row(i)(1:lrow(i)),i=iafter,iend)
      close(gunit)
c 
      info=system('chmod u+x '//fname)
c
      end

************************************************************************
      subroutine radf(fname,natoms,qmmmatomtype)

*-------------------------------------------------------------------
*
*  reads ADF input file fname
*
*  written by C. Herrmann, Universitaet Jena 04/2005
*
*-------------------------------------------------------------------
c
      implicit none
c
      integer iunit,natoms,lfname,mxitem,i,ii
      parameter(mxitem=10)
      character*255 row,item(mxitem)
      character*(*) fname,qmmmatomtype(*)

      natoms = 0
      iunit=10
      call llen(fname,lfname)
      open(unit=iunit,file=fname(1:lfname))
10    read(iunit,'(a132)',end=30) row
      call upper(row)
      call split(row,item,mxitem)
c if Atom-block found, determine number of atoms 
      if (item(1).eq.'ATOMS') then
20      read(iunit,'(a132)') row
        call split(row,item,mxitem)
        if(item(1).eq.'END')then
          goto 10
        else
          natoms = natoms+1
          i=i+1
          goto 20
        end if
      else if(item(1).eq.'QMMM') then
40      read(iunit,'(a)') row
        call upper(row)
        call split(row,item,mxitem)
        if(item(1).eq.'MM_CONNECTION_TABLE') then
          ii=1
45        read(iunit,'(a)') row
          call upper(row)
          call split(row,item,mxitem)
          if(item(1).eq.'SUBEND') then
             goto 10
          else
             qmmmatomtype(ii)=item(3)
             ii=ii+1
          end if
          goto 45
        end if
        goto 40  
        if(item(1).eq.'END') 
     *  then
          write(*,*)
          write(*,*) '*****************************************'
          write(*,*) ' WARNING: Found QMMM keyword in adf.in,  '
          write(*,*) ' but no MM_CONNECTION_TABLE specified!   '
          write(*,*) '*****************************************'
          write(*,*)
          goto 10
        end if
      end if
      goto 10
30    continue
      close(iunit)
      end


*************************************************************************

      subroutine gadflastqm(lastqmatom,qmmmatomtype)

*-------------------------------------------------------------------
*
*  written by C. Herrmann, Universitaet Jena, 07/2005
*
*-------------------------------------------------------------------
c
      implicit none
c
      integer gunit,mxitem,lrow,nempty,natoms,i,j
      integer ist,end,k,maxrow,ii,lastqmatom,nlink,nqmatom
      parameter(mxitem=10,maxrow=1000)
      character*(132) row,item(mxitem)
      character*(2) el(maxrow),qmmmatomtype(*)
      real*8 co(3,maxrow),string2real
#include "fconstants.h"
c
      gunit = 11
      natoms=0
      nqmatom=0
      nlink=0
      lastqmatom=0
      i=1
*
      open(gunit,file='adf.in',status='unknown')
10    read(gunit,'(a132)',end=30) row
      call split(row,item,mxitem)
c if Atom-block found, read in equilibrium atom coordinates
      if(item(1).eq.'QMMM'.or.item(1).eq.'qmmm'.or.
     *item(1).eq.'Qmmm') then
40      read(gunit,'(a)') row
        call upper(row)
        call split(row,item,mxitem)
        if(item(1).eq.'MM_CONNECTION_TABLE') then
          ii=1
45        read(gunit,'(a)') row
          call upper(row)
          call split(row,item,mxitem)
          if(item(1).eq.'SUBEND') then
             goto 10
          else
             qmmmatomtype(ii)=item(3)
             if(item(3).eq.'QM'.or.(item(3).eq.'LI')) then
               lastqmatom=lastqmatom+1
             end if
             ii=ii+1
          end if
          goto 45
        end if
        goto 40  
        if(item(1).eq.'END') 
     *  then
          write(*,*)
          write(*,*) '*****************************************'
          write(*,*) ' WARNING: Found QMMM keyword in adf.in,  '
          write(*,*) ' but no MM_CONNECTION_TABLE specified!   '
          write(*,*) '*****************************************'
          write(*,*)
          goto 10
        end if
      end if
      goto 10
30    continue
      close(gunit)
*
      end

*********************************************************************
c
      subroutine radfcoord(fname,natoms,co,ic,iosl)
c
c  written by C. Herrmann, Uni Jena 04/2005
c
      implicit none
c
      character*(*) fname
      integer gunit,mxitem,lrow,nempty,natoms,i,j,coordunit
      integer lf,iosl,find_nucl_charge,ic(natoms)
      integer ist,end,k,maxrow
      parameter(mxitem=10,maxrow=100)
      character*(132) row,item(mxitem)
      character*(2) el(natoms)
      real*8 co(3,natoms),string2real
      logical valbohr
#include "fconstants.h"
c
      gunit=10
      valbohr=.false.
      i=1
c
      call llen(fname,lf)
      open(gunit,file=fname(1:lf),status='unknown')
10    read(gunit,'(a132)',end=30) row
      call upper(row)
      call split(row,item,mxitem)
      if (item(1).eq.'Atoms'.or.item(1).eq.'ATOMS'.or.item(1).eq.'atoms'
     *    ) then
20      read(gunit,'(a132)') row
        call split(row,item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
          goto 10
        else
          read(row,*) el(i), co(1,i), co(2,i), co(3,i)
          ic(i) = find_nucl_charge(el(i))
          i=i+1
          goto 20
        end if
c----------------------------------------------------------------------
c if there is a unit block specifying other than standard units (Angstrom),
c set logical variable valbohr = .true.
c---------------------------------------------------------------------
      else if(item(2).eq.'BOHR') then
          valbohr=.true.
      end if
      goto 10
30    continue
      close(gunit)
c----------------------------------------------------------------------
c if coordinates are given in Angstroem (ADF default), convert them to Bohr
c (Akira internal standard)
c----------------------------------------------------------------
      if(.not.valbohr) then
        do i = 1,natoms
        do j=1,3
CJN         write(*,*) 'i,j ', i,j
           co(j,i) =  co(j,i)*(1/bohr2ang)
        end do
        end do
      end if

      end
c
c
c***********************************************************************
c
      subroutine wadfdip(tdr,fname,iosl)
*
*-------------------------------------------------------------------
*
*  generates ADF single-point input file name on slaves
*
*  written by C.Herrmann, University of Jena, 04/2005
*
*-------------------------------------------------------------------
*
      implicit none
c
      integer gunit,aunit,ltd,i,j,ibefore,lf
      integer iend,k,maxrow,mxitem,iafter,info
      parameter(gunit=10,aunit=11,maxrow=100,mxitem=10)
      character*(*) tdr
      character*(132) row(maxrow),item(mxitem),scratch
      character*2 find_element
      external find_element
      character*255 aname,fnamefull 
      character*(*) fname
      integer lrow(maxrow),iosl,system
#include "fconstants.h"
c
      call llen(tdr,ltd)
      call llen(fname,lf)
      fnamefull=tdr(1:ltd)//'/'//fname(1:lf)
      aname=tdr(1:ltd)//'/adfslave.in'
      call llen(fnamefull,lf)
c
      i = 1
      open(aunit,file=aname,status='unknown')
10    read(aunit,'(a)',end = 35) row(i)
      call llen(row(i),lrow(i))
      call split(row(i),item,mxitem)
      if (item(1).eq.'Geometry'.or.item(1).eq.'GEOMETRY'.or.item(1).eq.
     *'geometry') then
c mark beginning of geometry section
        ibefore=i-1
        i = i+1
20      read(aunit,'(a)') row(i)
        call split(row(i),item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
c mark end of geometry section
          iafter = i+1
          goto 30
        end if
        i=i+1
        goto 20
c if there is a unit block specifying other than standard units (Angstrom),
c leave it out
      end if
30    continue
      i=i+1
      goto 10
35    continue
      close(aunit)
      iend=i-1
c
      open(unit=gunit,file=fnamefull(1:lf),
     *status='unknown')
CCH writing rows in adf-input file with blanks at the end
CCH makes ADF crash for some reason ...
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=1,ibefore)
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=iafter,iend)
      call flush(iosl)
      close(gunit)
c
      info=system('chmod u+x '//fnamefull(1:lf))
c
      end 


c***********************************************************************
c
      subroutine wadfpol(tdr,fname,iosl)
*
*-------------------------------------------------------------------
*
*  generates ADF single-point input file name on slaves
*  for calculation of polarizabilities
*
*  written by C.Herrmann, University of Jena, 04/2005
*
*-------------------------------------------------------------------
*
      implicit none
c
#include "param.h"
      integer gunit,aunit,ltd,i,j,ibefore,lf
      integer iend,k,maxrow,iafter,info,system
      parameter(gunit=10,aunit=11,maxrow=100)
      character*(*) tdr
      character*(132) row(maxrow),item(mxitem),scratch
      character*2 find_element
      external find_element
      character*255 aname,fnamefull 
      character*(*) fname
      integer lrow(maxrow),iosl
#include "fconstants.h"
c
      call llen(tdr,ltd)
      call llen(fname,lf)
      fnamefull=tdr(1:ltd)//'/'//fname(1:lf)
      aname=tdr(1:ltd)//'/adfslave.in'
      call llen(fnamefull,lf)
c
      i = 1
      open(aunit,file=aname,status='unknown')
10    read(aunit,'(a)',end = 35) row(i)
      call llen(row(i),lrow(i))
      call split(row(i),item,mxitem)
      if (item(1).eq.'Geometry'.or.item(1).eq.'GEOMETRY'.or.item(1).eq.
     *'geometry') then
c mark beginning of geometry section
        ibefore=i-1
        i = i+1
20      read(aunit,'(a)') row(i)
        call split(row(i),item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
c mark end of geometry section
          iafter = i+1
          goto 30
        end if
        i=i+1
        goto 20
c if there is a unit block specifying other than standard units (Angstrom),
c leave it out
      end if
30    continue
      i=i+1
      goto 10
35    continue
      close(aunit)
      iend=i-1
c
      open(unit=gunit,file=fnamefull(1:lf),
     *status='unknown')
CCH writing rows in adf-input file with blanks at the end
CCH makes ADF crash for some reason ...
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=1,ibefore)
      write(gunit,'(a)') 'Response'
      write(gunit,'(a)') '  allcomponents'
CJN -----------------------------------------------
CJN Akira does not support Raman calculations (yet)
CJN -----------------------------------------------
      if(ipol > 0) stop "No Akira Raman Calculations!"
CJN      if(omeg(1).ne.0.0d0) then
CJN        write(gunit,'(a7,i3)') ' nfreq ',ipol 
CJN        write(gunit,'(a8,f16.8)') ' frqbeg ',omeg(1) 
CJN        write(gunit,'(a8,f16.8)') ' frqend ',omeg(ipol)
CJN        write(gunit,'(a)') ' hartree'  
CJN      end if
      write(gunit,'(a)') 'End' 
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=iafter,iend)
      call flush(iosl)
      close(gunit)
c
      info=system('chmod u+x '//fnamefull(1:lf))
c
      end 

c*************************************************

      subroutine wadfcontrol(co,natoms,irescp,lastqmatom,el)

*-------------------------------------------------------------------
*
*  Generates fake coord file 
*  in Turbomole format 
*  
*  written by C. Herrmann, Universitaet Bonn, 10/2004
*  modified by C. Herrmann, ETH Zurich, 05/30/07
*
*-------------------------------------------------------------------
c
      implicit none
c
      integer cunit,gunit,mxitem,lrow,nempty,natoms,i,j,coordunit
      integer ist,end,k,maxrow,ii,lastqmatom,irescp,litem,idum
      parameter(mxitem=10,maxrow=100000)
      character*255 restadf,bla(10)
      character(len=132) row,item(mxitem)
      character(len=2) el(*)
      real*8 co(3,*),string2real
      logical valbohr,ex
#include "fconstants.h"
c
      cunit = 10
      gunit = 11
      coordunit=12
      natoms=0
      valbohr=.false.
      irescp=0
      i=1
*
*-------------------------------------------------------------------
*  read in coordinates from 'adf.in'
*-------------------------------------------------------------------
      open(gunit,file='adf.in',status='unknown')
10    read(gunit,'(a132)',end=30) row
      call split(row,item,mxitem)
c if Atom-block found, read in equilibrium atom coordinates
      if (item(1).eq.'Atoms'.or.item(1).eq.'ATOMS'.or.item(1).eq.'atoms'
     *    ) then
20      read(gunit,'(a132)') row
        read(row,*,end=21) (bla(j),j=1,6)
21      continue
        call split(row,item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
          goto 10
        else
CCH the following if-construction makes adfdefine compatible
CCH with both numbered and unnumbered atomlists in adf.in.
          if(j.eq.5) then
          read(row,*) el(i), co(1,i), co(2,i), co(3,i)
          natoms = natoms+1
          i=i+1
          goto 20
          else 
          read(row,*) idum, el(i), co(1,i), co(2,i), co(3,i)
          natoms = natoms+1
          i=i+1
          goto 20
          end if
        end if
c if there is a unit block specifying other than standard units (Bohr),
c set logical variable valbohr = .true.
      else if(item(2).eq.'Bohr'.or.item(2).eq.'BOHR'.or.item(2).eq.
     *    'bohr') then
          valbohr=.true.
      else if(item(1).eq.'RESTART'.or.item(1).eq.'restart'.or.
     *    item(1).eq.'Restart') then
          irescp=1
          call llen(item(2),litem)
          inquire(exist=ex,file=item(2)(1:litem))
          if(.not.ex) then
            write(*,*)
            write(*,*) '********************************************'
            write(*,*) ' WARNING: Found RESTART keyword in adf.in,  '
            write(*,*) ' but file '//item(2)(1:litem)//
     *                 ' does not exist!     '
            write(*,*) ' Please either remove the RESTART keyword   '
            write(*,*) ' or provide the file '//item(2)(1:litem)//'!'
            write(*,*) '********************************************'
            write(*,*)
            stop
          end if
          restadf = item(2)
      end if
      goto 10
30    continue
      close(gunit)
c if coordinates are given in Angstroem (ADF default), convert them to Bohr
c (Akira internal standard)
      if(.not.valbohr) then
        do i = 1,natoms
        do j=1,3
           co(j,i) =  co(j,i)*(1/bohr2ang)
        end do
        end do
      end if
*
*
*-------------------------------------------------------------------
*  write akira_control file
*-------------------------------------------------------------------
      open(cunit,file='akira_control',status='unknown')
      write(cunit,'(a11)') '$anfmode 40'
      write(cunit,'(a8,2x,i5)') '$irescp ',irescp
      if(irescp.gt.0) then
        write(cunit,'(a11,2x,a)') '$restadf = ',restadf
      end if
      write(cunit,'(a12,2x,i5)') '$lastqmatom ',lastqmatom
      write(cunit,'(a)') '$rundimensions '
      write(cunit,'(a11,2x,i5)')  '  natoms= ',natoms
      write(cunit,'(a)') '$end'
      close(cunit)

*
*-------------------------------------------------------------------
*  write fake coord file
*-------------------------------------------------------------------
      open(coordunit,file='coord')
      write(coordunit,'(a)') '$coord'
      do i = 1,natoms
      write(coordunit,'(3f18.12,2x,a3)') (co(j,i),j=1,3),el(i)
      end do
      write(coordunit,'(a)') '$end'
      close (coordunit)
*
      end subroutine wadfcontrol

