
      subroutine qmmmgrad2grad(qmmmgrad,grad,lastqmatom,fname)
!===================================================
! write QMMM gradient components for QM system at
! appropriate positions to gradient vector for the 
! whole QMMM system.
!
! written by C. Herrmann, Northwestern University 01/2008
!===================================================

      implicit none

      integer lf,lastqmatom,i
      integer,allocatable :: pos(:)
      character(len=*) fname
      real*8 grad(*),qmmmgrad(*)

      allocate(pos(lastqmatom))    
 
      call llen(fname,lf)
      call getmmpos(fname(1:lf),pos,lastqmatom) 

      do i=1,lastqmatom
        grad((pos(i)-1)*3+1)=qmmmgrad((i-1)*3+1)
        grad((pos(i)-1)*3+2)=qmmmgrad((i-1)*3+2)
        grad((pos(i)-1)*3+3)=qmmmgrad((i-1)*3+3)
      end do

      deallocate(pos)

      end subroutine

!*******************************************************

      subroutine getmmpos(fname1,pos,lastqmatom)
!==================================================
! read positions of QM atoms in Comqum's MM atom list
!
! written by C. Herrmann, Northwestern University 01/2008
!==================================================

      implicit none

      
      integer iunit,lastqmatom,pos(*),lf,jstart,jend,i,j,nrow
      parameter(iunit=16)
      character(len=*) fname1
      character(len=255) row

      call llen(fname1,lf)
      open(unit=iunit,file=fname1(1:lf),err=900)

10    read(iunit,'(a)',end=100) row
      if(index(row,'$end').ne.0) goto 100
      if(index(row,'$correspondance_list').ne.0) then
         read(iunit,*) 
         nrow=ceiling(real(lastqmatom)/12.0d0)
         jstart=1
         jend=0
         do i=1,nrow
           jend=jend+12
           if(jend.gt.lastqmatom) jend=lastqmatom
           read(iunit,'(12i6)',end=20) (pos(j),j=jstart,jend)
20         continue
           jstart=jstart+12
         end do
      end if
      goto 10
       
100   close(iunit)      

      goto 1000
900   write(*,*) 'atomorder_comqum_qm2mm: could not open file '
     *            //fname1(1:lf)
1000  continue

      end subroutine


!**********************************************************
      subroutine atomorder_comqum_mm2qm(fname1,fname2,natoms,co,elinfo,
     *                                   lastqmatom)
c============================================================
c * extract QM atom coordinates from list of (displaced) QM/MM
c   coordinates, where QM atoms may be distributed 
c   anywhere among the MM atoms, and reorder them 
c   such that the QM atoms are the first in 
c   list of coordinates. The MM atoms are not reordered accordingly,
c   because in the calling routine, the MM coordinates will not be 
c   needed anymore (might be changed one day).
c * The element symbols and additional specifications
c   are read from the coord.ori file
c * The order of the QM/MM atoms used in AKIRA corresponds to the
c   one used in Comqum's prmcrd3 file.
c * The positions of the QM atoms in the list of QM and MM coordinates 
c   as stored in prmcrd3 are given after the keyword $correspondance_list
c   in the comqum.dat file
c
c written by C. Herrmann, Northwestern University, 12/2007
c============================================================

      implicit none

      integer lf,natoms,iunit,lastqmatom,i,j,mxitem,ii
      character(len=*) fname1,fname2,elinfo(*)
      parameter(iunit=15,mxitem=10)
      character(len=255) row,item(mxitem)
      real*8 co(3,*)
      integer,allocatable :: pos(:)

      call llen(fname1,lf)

!===============================
! read new order
!===============================

      allocate(pos(lastqmatom))
      call llen(fname1,lf)
     
      call getmmpos(fname1(1:lf),pos,lastqmatom)

!===================================
! extract QM atoms
!===================================

      do i=1,lastqmatom

        co(1,i)=co(1,pos(i))
        co(2,i)=co(2,pos(i))
        co(3,i)=co(3,pos(i))

      end do

      deallocate(pos)

!================================================================
! read element symbols and additional information from coord.ori
!================================================================

      call llen(fname2,lf)
      open(unit=iunit,file=fname2(1:lf),err=901)
110   read(iunit,'(a)',end=120) row
      if(index(row,'$end').ne.0) goto 120
      if(index(row,'$coord').ne.0) then
        do i=1,lastqmatom
          read(iunit,'(a)',end=120) row
          call split(row,item,mxitem)
          write(elinfo(i),'(a2,x,a7)') item(4),item(5)
        end do
      end if
      goto 110 
120   close(iunit)

      goto 1000
901   write(*,*) 'atomorder_comqum_qm2mm: could not open file '
     *            //fname2(1:lf)
1000  continue

      end subroutine

c************************************************************

      subroutine writecomqum(fname1,lastqmatom,co)
c==================================================
c write molecular coordinates to file of prmcrd type
c
c written by C. Herrmann, ETHZ, 07.11.2007
c==================================================

      implicit none

#include "fconstants.h"

      character(len=*) fname1
      integer iunit,lf,i,nrow,j,lastqmatom
      real*8 co(3,*)
      parameter(iunit=25)

      nrow=ceiling(real(lastqmatom)/2.0d0)

      call llen(fname1,lf)
      open(unit=iunit,file=fname1(1:lf))
      write(iunit,*)
      write(iunit,'(i6)') lastqmatom
      do i=1,nrow-1
        write(iunit,'(6f12.7)') (co(j,i*2-1)*bohr2ang,j=1,3),
     *                           (co(j,i*2)*bohr2ang,j=1,3) 
      end do      
      if(mod(lastqmatom,2).ne.0) then
        write(iunit,'(6f12.7)') (co(j,lastqmatom)*bohr2ang,j=1,3)
      else
        write(iunit,'(6f12.7)') (co(j,lastqmatom-1)*bohr2ang,j=1,3),
     *                          (co(j,lastqmatom)*bohr2ang,j=1,3)
      end if

      close(iunit)
       

      end subroutine 

c*****************************************************************

      subroutine readcomqum(fname1,fname2,natoms,co,ic,maxatoms)
c========================================================
c - read coordinates of QM + MM system from fname1 
c - read element symbols of QM + MM system from fname2 (both in Amber format)
c - convert coordinates from Angstroem to Bohr
c
c written by C. Herrmann, ETHZ, 07.11.2007
c========================================================

      implicit none

#include "fconstants.h"

      character(len=*) fname1,fname2
      integer lf,natoms,iunit,i,j,ic(*),nrow,find_ic_amber
      integer jstart,ii,jj,find_nucl_charge,maxatoms
      real*8 co(3,*),mass(maxatoms)
      parameter(iunit=23)
      character(len=255) row,fstring,el(maxatoms)

      call llen(fname1,lf)
 
      open(unit=iunit,file=fname1(1:lf))
      read(iunit,*)
      read(iunit,*) natoms
      nrow=ceiling(real(natoms)/2.0d0)
      do i=1,nrow
        read(iunit,*,end=5) (co(j,i*2-1),j=1,3),(co(j,i*2),j=1,3)
      end do      
5     close(iunit)

      do i=1,natoms
        do j=1,3
          co(j,i)=co(j,i)*(1.0d0/bohr2ang)
        end do
      end do

      call llen(fname2,lf)

      open(unit=iunit,file=fname2(1:lf))
10    read(iunit,'(a)',end=20) row
      if(index(row,'FLAG MASS').ne.0) then
        read(iunit,'(a)') row
CCH trying to use format string from fname2 does not work for
CCH some reason... 
!        ii=index(row,'(')
!        jj=index(row,')')
!        read(row(ii+1:jj-1),'(a)') fstring
!        call llen(fstring,lf)
!        fstring="'("//fstring(1:lf)//")'"
!        call llen(fstring,lf)
!        write(*,*) '|'//fstring(1:Lf)//'|'
        nrow=ceiling(real(natoms)/5.0d0)
        jstart=0
        do i=1,nrow
!          read(iunit,fstring(1:lf),end=20) (ic(jstart+j),j=1,5)
          read(iunit,'(5E16.8)',end=20) (mass(jstart+j),j=1,5)
          jstart=jstart+5
        end do
        goto 20
      end if
      goto 10  
20    close(iunit)

      do i=1,natoms
        ic(i)=find_ic_amber(mass(i))
      end do
       
      end subroutine 

c****************************************************************

      subroutine gcomqumnatoms(fname1,natoms)

      implicit none

      character(len=*) fname1
      integer natoms,lf,iunit
      parameter(iunit=24)

      call llen(fname1,lf)

      open(unit=iunit,file=fname1(1:lf))
      read(iunit,*)
      read(iunit,*) natoms
      close(iunit)

      end subroutine 

c**************************************************

      subroutine getnatomstm(fname,qmnatoms)

      implicit none

      character(len=*) fname
      integer lf,iunit,qmnatoms
      parameter(iunit=25)
      character(len=20) row

      call llen(fname,lf)

      qmnatoms=0
      open(unit=iunit,file=fname(1:lf))
10    read(iunit,'(a)',end=20) row
      if(index(row,'$coord').ne.0) then
15      read(iunit,'(a)',end=20) row
        if(index(row,'$end').ne.0) goto 20
        qmnatoms=qmnatoms+1
        goto 15   
      end if
      goto 10
20    close(iunit)

      end subroutine

c******************************************************

      integer function find_ic_amber(amass)

      implicit none

      integer dim
      parameter(dim=7)
      real*8 mass(dim),amass
      integer ic(dim),i

!      if(amass.eq.1.008d0) find_ic_amber=1

      data mass / 1.008d0, 12.01d0, 14.01d0, 16.00d0, 
     *           32.06d0, 55.85d0, 15.9994d0 /
      data ic / 1,6,7,8,16,26,8 /

      i=1
10     if(amass.eq.mass(i)) then 
         find_ic_amber=ic(i)
         goto 20
       end if
       i=i+1
       if(i.gt.dim+1) then
         write(*,*) ''
         write(*,*) ' function find_ic_amber: unknown atomic mass:',
     *              amass
         write(*,*) ' STOP.'
         write(*,*) ''
         stop
       end if 
       goto 10
20     continue

      end function
