      subroutine selectstp(normm,nbm,naus,natoms,istepfl,ic,coord,
     * mcharge,maxnbm,apphess,ortmod,nbmo)

*-------------------------------------------------------------------------
* interactive program for preparation/selection of intitial basis vectors
* (and preconditioner ...)
*
* written by J. Neugebauer, Uni Erlangen, 2002
* modified by C. Herrmann, Uni Bonn, 2004
*-------------------------------------------------------------------------

*------------------------------------------------------------
* types of basis modes:
* 0 - no mode selected
* 1 - vibrational stretch between two atoms
* 2 - cartesian nuclear x-coordinate for an atom
* 3 - cartesian nuclear y-coordinate for an atom
* 4 - cartesian nuclear z-coordinate for an atom
* 5 - valence bending involving three atoms
* 6 - basis vector read from file or calculated by MOPAC
* 7 - breathing mode - take coord file and use as normal mode
* 8 - torsion involving four atoms
* 9 - out-of-plane mode involving four atoms
*------------------------------------------------------------

#include "defs.h"

      real*8, allocatable :: tmod(:),tmod2(:),submod(:),trro(:,:)
      integer mxatb
      parameter(mxatb=100)
      integer naus,natoms,lrow,i,nmodc,ntmod,j,nvar,at1,at2,mcharge
      integer istepfl(naus*3*natoms),ic(natoms),nbm,atb(mxatb),maxnbm
      integer atb2(mxatb),btyp(mxatb),count,atb3(mxatb),at3,system,lf
      integer nbmf,info,nasub,atb4(mxatb),at4,bbtyp,nbmo
      real*8 normm(3*natoms,maxnbm+1),coord(3,natoms),thres,xmolsc1
      real*8 ovl,ortmod(3*natoms,maxnbm)
      parameter(thres=1.0d-6)
      character*132 row
      character*255 filename
      logical valid,para,checkat,ex,nmpres,found,apphess,notrro
      logical pint,pcart,pmop,pfile,psym

*-------------------------------------------------------------------------
* print flags for items in initial basis vector menu
*-------------------------------------------------------------------------
      pint=.false.
      pcart=.false.
      pmop=.false.
      pfile=.false.
      psym=.false.

*-------------------------------------------------------------------------
* initialize variables
*-------------------------------------------------------------------------
      apphess=.false.
      notrro=.false.
      nbm=0
      nbmf=0
      nbmo=0
      nmpres=.false.
      nvar=3*natoms
      bbtyp=0
      do i=1,nvar
        do j=1,maxnbm
          normm(i,j)=0.0d0
        end do
        do j=1,naus
          istepfl((i-1)*naus+j)=1
        end do
      end do
      do i=1,mxatb
        btyp(i)=0
      end do
      xmolsc1 = 10.0d0
      call rkeyreal('akira_control','$xmolsc1',xmolsc1)

*-------------------------------------------------------------------------
* selection menu
*-------------------------------------------------------------------------
      valid=.false.
100   if(valid) goto 200
CCH      call pcoord(natoms,coord,ic,mcharge,6,mode)
      write(*,*)
      write(*,*) ' Selection of initial basis vectors'
      write(*,*) ' ----------------------------------'
*-------------------------------------------------------------------------
* basis vectors in buffer ?
*-------------------------------------------------------------------------
      if(nbmf.gt.0) then
        if(bbtyp.eq.0) then
          write(*,*) ' No. of basis modes in buffer (read from file): ',
     *     nbmf
        else if(bbtyp.eq.1) then
          write(*,*) ' No. of symmetry coordinates in buffer : ',nbmf
        end if
        write(*,*)
      end if
*-------------------------------------------------------------------------
* basis vectors in set of orthonormal vectors ?
*-------------------------------------------------------------------------
      if(nbmo.gt.0) then
        write(*,*) ' No. of vectors selected for orthogonal space : ',
     *     nbmo
        write(*,*)
      end if
*-------------------------------------------------------------------------
* basis vectors in use ? if there are some in use, then print type of b.v.
*-------------------------------------------------------------------------
      write(*,*) ' No. of basis modes selected: ',nbm
      if(nbm.eq.maxnbm) then
        write(*,*) 'Maximum number of basis vectors in use !'
        write(*,*)
      end if
      if(nbm.gt.maxnbm) then
        call serror('selectstp',2,'No. of basis vectors to large!',6)
      else if(nbm.gt.0) then
        do i=1,nbm
          if(btyp(i).eq.1) then
            write(*,'(2x,a,2i4)')
     *        ' stretching mode for atoms : ',atb(i),atb2(i)
          else if(btyp(i).eq.5) then
            write(*,'(2x,a,3i4)')
     *        ' bending modes for atoms   : ',atb(i),atb2(i),atb3(i)
          else if(btyp(i).eq.8) then
            write(*,'(2x,a,4i4)')
     *    ' torsion for atoms         : ',atb(i),atb2(i),atb3(i),atb4(i)
          else if(btyp(i).eq.9) then
            write(*,'(2x,a,4i4)')
     *    ' out-of-plane mode involv. : ',atb(i),atb4(i),atb2(i),atb3(i)
          else if(btyp(i).eq.2) then
             write(*,'(2x,a,i4)')
     *        ' x coordinate for atom     : ',atb(i)
          else if(btyp(i).eq.3) then
             write(*,'(2x,a,i4)')
     *        ' y coordinate for atom     : ',atb(i)
          else if(btyp(i).eq.4) then
             write(*,'(2x,a,i4)')
     *        ' z coordinate for atom     : ',atb(i)
          else if(btyp(i).eq.6) then
             write(*,'(2x,a,i4)')
     *        ' buffer vector no.         : ',atb(i)
          else if(btyp(i).eq.7) then
             write(*,'(2x,a)') ' totally symmetric breathing mode'
          else if(btyp(i).eq.10) then
             write(*,'(2x,a)') ' totally symmetric (mw) breathing mode'
          end if
        end do
        write(*,*)
      end if
*-------------------------------------------------------------------------
* suggest commands
*-------------------------------------------------------------------------
      write(*,*) ' choose one of the following commands:'
      write(*,*)
*--------------------------------------------------------------------------
* selection of new basis modes only if the maximum number is not yet in use
*--------------------------------------------------------------------------
      if(nbm.le.maxnbm) then
         write(*,*)
     * '  -------------------------------------------------------------'
       if(pint) then
         write(*,*)
     * '  str <i j>  :  select stretch mode between atoms i and j'
         write(*,*)
     * '  bd <i j k> :  select bending mode between atoms i, j, and k'
         write(*,*)
     *'  tor <ijkl> :  select torsion mode between atoms i, j, k, and l'
         write(*,*)
     *'  oop <ijkl> :  select out-of-plane mode (the angle between bond'
         write(*,*)
     *'                i-j and plane (jkl) will be varied in this mode)'
         write(*,*)
     * '  br         :  select breathing mode (totally symmetric)'
         write(*,*)
     * '  brm        :  select mass-weighted breathing mode'
         write(*,*)
     * '  pint       :  hide internal coordinate options'
       else
         write(*,*)
     * '  pint       :  show internal coordinate options'
       end if
         write(*,*)
     * '  -------------------------------------------------------------'
       if(psym) then
         write(*,*)
     * '  sym        :  generate symmetry coordinates as basis vectors'
         write(*,*)
     * '  spesym     :  generate normal modes for special symmetries'
         write(*,*)
     * '  psym       :  hide symmetry coordinate options'
       else
         write(*,*)
     * '  psym       :  show symmetry coordinate options'
       end if
         write(*,*)
     * '  -------------------------------------------------------------'
       if(pcart) then
         write(*,*)
     * '  at <i>     :  select all cartesian basis vectors for atom i'
         write(*,*)
     * '  atx <i>    :  select cartesian x-basis vector for atom i'
         write(*,*)
     * '  aty <i>    :  select cartesian y-basis vector for atom i'
         write(*,*)
     * '  atz <i>    :  select cartesian z-basis vector for atom i'
         write(*,*)
     * '  aatz       :  select all cartesian z-basis vectors'
         write(*,*)
     * '  cfull      :  select all cartesian basis vectors'
         write(*,*)
     * '                (full frequency analysis)'
         write(*,*)
     * '  pcart      :  hide cartesian coordinate options'
       else
         write(*,*)
     * '  pcart      :  show cartesian coordinate options'
       end if
         write(*,*)
     * '  -------------------------------------------------------------'
       if(pfile) then
         write(*,*)
     * '  re  <i>    :  read <i> basis vectors from file'
         write(*,*)
     * '  resub <i>  :  read <i> basis vectors for subsystem from file'
         write(*,*)
     * '  pfile      :  hide basis-vector-from-file options'
       else
         write(*,*)
     * '  pfile      :  show basis-vector-from-file options'
       end if
         write(*,*)
     * '  -------------------------------------------------------------'
       if(pmop) then
         write(*,*)
     * '  moppm3     :  do MOPAC PM3 calculation; or'
         write(*,*)
     * '  mopam1     :  do MOPAC AM1 calculation; or'
         write(*,*)
     * '  mopmindo   :  do MOPAC MINDO/3 calculation;'
         write(*,*)
     * '                select basis vectors from MOPAC output'
         write(*,*)
     * '  pmop       :  hide basis-vectors-from-MOPAC options'
       else
         write(*,*)
     * '  pmop       :  show basis-vectors-from-MOPAC options'
       end if
         write(*,*)
     * '  -------------------------------------------------------------'
       write(*,*)
     * '  trarot     :  create translational and rotational modes'
      end if
*--------------------------------------------------------------------------
* ... operations involving the vectors in use can only be carried out
* if there are vectors in use, can't they ?
*--------------------------------------------------------------------------
      if(nbm.gt.0) then
       write(*,*)
     *'  disp       :  display basis vectors using MOLDEN'
       write(*,*)
     *'  ort        :  orthonormalization to trans/rot modes'
       write(*,*)
     *'  c          :  clear all selected basis vectors'
       write(*,*)
     *'  p          :  print basis vectors'
       write(*,*)
     *'  pm         :  print mass-weighted basis vectors'
       write(*,*)
     *'  g98        :  write g98 fake output to file <g98.out.it0>'
       write(*,*)
     *'  xmol       :  write xmol XYZ output to file <xmol.XYZ.it0>'
      end if
*--------------------------------------------------------------------------
* ... operations involving the vectors in buffer can only be carried out
* if there are vectors in buffer, can't they ?
*--------------------------------------------------------------------------
      if(nbmf.gt.0) then
       write(*,*)
     *'  cbv <i>    :  copy buffer vector <i> to set of basis vectors'
       write(*,*)
     *'  acbv       :  copy all buffer vectors to set of basis vectors'
       write(*,*)
     *'  cov <i>    :  use buffer vector <i> as orthogonal vector'
       write(*,*)
     *'  acov       :  use all buffer vectors as orthogonal vectors'
       write(*,*)
     *'  bdisp      :  display buffer vectors using MOLDEN'
       write(*,*)
     *'  bc         :  clear all buffer vectors'
       write(*,*)
     *'  bp         :  print buffer vectors'
       write(*,*)
     *'  bpm        :  print mass-weighted buffer vectors'
       write(*,*)
     *'  bg98       :  write g98 fake output of buffer vectors'
       write(*,*)
     *'                to file <g98.out.buf>'
       write(*,*)
     *'  bxmol      :  write xmol XYZ output of buffer vectors'
       write(*,*)
     *'                to file <xmol.XYZ.buf>'
      end if
      write(*,*)
     *'  submenu    :  enter subsystem menu'
      write(*,*)
     *'  <return>   :  quit this menu'
      write(*,*)
*--------------------------------------------------------------------------
* read command
*--------------------------------------------------------------------------
      call clean(row)
      read(*,'(A)') row
      call slen(row,lrow)
      if(lrow.eq.0) then
        if(nbm.gt.0) then
          valid=.true.
        else
          call serror('selectstp',4,'No basis vectors selected!',6)
        end if
      else if(row(1:4).eq.'psym') then
        psym = .not.psym
      else if(row(1:4).eq.'pint') then
        pint = .not.pint 
      else if(row(1:5).eq.'pcart') then
        pcart = .not.pcart
      else if(row(1:4).eq.'pmop') then
        pmop = .not.pmop
      else if(row(1:5).eq.'pfile') then
        pfile = .not.pfile

      else if(row(1:6).eq.'spesym') then
*--------------------------------------------------------------------------
* special normal coordinates (completely defined by symmetry)
* at the moment only implemented for molecules of type:
*  - ML4 (tetrahedral)
*--------------------------------------------------------------------------
        if(allocated(tmod)) deallocate(tmod)
        nbmf=3*natoms
        allocate(tmod2(6*natoms*nbmf))
        call specialnm(natoms,coord,ic,tmod2,tmod2(3*natoms*nbmf+1:),
     *       at1)
        if(at1.eq.0) then
          nbmf=0
        else
          allocate(tmod(3*natoms*(at1+1)))
          do i=1,at1*3*natoms
            tmod(i)=tmod2(i)
          end do
          nbmf=at1
          bbtyp=1
        end if
        deallocate(tmod2)

      else if(row(1:3).eq.'sym') then
*--------------------------------------------------------------------------
* symmetry coordinates
*--------------------------------------------------------------------------
        if(allocated(tmod)) deallocate(tmod)
        nbmf=3*natoms
        allocate(tmod2(3*natoms*(nbmf+1)))
        call symmc(natoms,coord,ic,tmod2,tmod2(3*natoms*nbmf+1:),
     *   nbmf,at1)
        if(at1.eq.0) then
          nbmf=0
        else
          allocate(tmod(3*natoms*(at1+1)))
          do i=1,at1*3*natoms
            tmod(i)=tmod2(i)
          end do
          nbmf=at1
          bbtyp=1
        end if
        deallocate(tmod2)

      else if(row(1:3).eq.'mop') then
*--------------------------------------------------------------------------
* o.k., we want a semi-empirical guess ...
*--------------------------------------------------------------------------
        if(allocated(tmod)) deallocate(tmod)
        nbmf=nvar
        write(*,'(2x,A)')
        write(*,'(1x,A)') '** preparing mopac input ...'
*--------------------------------------------------------------------------
* which model do we wanna use ?
*--------------------------------------------------------------------------
        if(row(4:6).eq.'pm3') then
          call mopacstart(coord,natoms,ic,'PM3')
        else if(row(4:6).eq.'am1') then
          call mopacstart(coord,natoms,ic,'AM1')
        else if(row(4:8).eq.'mindo') then
          call mopacstart(coord,natoms,ic,'MINDO/3')
        else
          nbmf=0
          goto 100
        end if
*--------------------------------------------------------------------------
* do the MOPAC calculation
*--------------------------------------------------------------------------
        write(*,'(1x,A)') '** please wait while running mopac ...'
        info=system('mopac')
        write(*,'(1x,A)') '** ... mopac calculation finished'
        if(info.ne.0) then
          call serror('selectstp',3,'MOPAC calculation '//
     *    'failed!',6)
          nbmf=0
          goto 100
        end if
*--------------------------------------------------------------------------
* try to read MOPAC output ...
*--------------------------------------------------------------------------
        write(*,'(1x,A)') '** reading mopac output ...'
        allocate(tmod(3*natoms*(nbmf+1)))
        call getncmop('FOR006',natoms,tmod,nbmf,
     *    tmod(3*natoms*nbmf+1:),found)
        nmpres=found
        if(.not.found) then
          nbmf=0
        end if
      else if(row(1:6).eq.'trarot') then
*--------------------------------------------------------------------------
* create translational and rotational modes
* (is of no practial use ...)
*--------------------------------------------------------------------------
        if(allocated(tmod)) deallocate(tmod)
        nbmf=6
        write(*,'(2x,A)')
        write(*,'(1x,A)')
     *   '** creating translational and rotational modes ...'
        allocate(tmod(3*natoms*(nbmf+1)))
        call trarot(coord,tmod,tmod(3*natoms*3+1:),ic,natoms)
        do i=1,nbmf
          tmod(3*natoms*nbmf+i)=0.0d0
          call nmod_m2c(tmod((i-1)*nvar+1:),natoms,ic)
        end do
      else if(row(1:3).eq.'str') then
*--------------------------------------------------------------------------
* vibrational stretch selected ...
*--------------------------------------------------------------------------
        read(row(4:),*,err=900,end=900) at1,at2
        if(at1.lt.1.or.at2.lt.1.or.at1.gt.natoms.or.at2.gt.natoms.or.
     *  at1.eq.at2) goto 910
        if(checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,1)) goto 920
        nbm=nbm+1
        btyp(nbm)=1
        call getstrbm(coord,natoms,normm,ic,at1,at2,nbm)
        atb(nbm)=at1
        atb2(nbm)=at2

*--------------------------------------------------------------------------
* nuclear Cartesian displacements for x direction, ...
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'atx') then
        read(row(4:),*,err=900,end=900) at1
        if(at1.lt.1.or.at1.gt.natoms) goto 910
        if(checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,2)) goto 920
        nbm=nbm+1
        atb(nbm)=at1
        btyp(nbm)=2
        normm((at1-1)*3+1,nbm)=1.0d0
*--------------------------------------------------------------------------
* ... y direction, ...
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'aty') then
        read(row(4:),*,err=900,end=900) at1
        if(at1.lt.1.or.at1.gt.natoms) goto 910
        if(checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,3)) goto 920
        nbm=nbm+1
        atb(nbm)=at1
        btyp(nbm)=3
        normm((at1-1)*3+2,nbm)=1.0d0
*--------------------------------------------------------------------------
* ... z direction, ...
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'atz') then
        read(row(4:),*,err=900,end=900) at1
        if(at1.lt.1.or.at1.gt.natoms) goto 910
        if(checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,4)) goto 920
        nbm=nbm+1
        atb(nbm)=at1
        btyp(nbm)=4
        normm((at1-1)*3+3,nbm)=1.0d0
*--------------------------------------------------------------------------
* ... and all of them, ...
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'at') then
        read(row(3:),*,err=900,end=900) at1
        if(at1.lt.1.or.at1.gt.natoms) goto 910
        count=0
        do i=1,3
          if(.not.checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,i+1)) then
            count=count+1
            atb(nbm+count)=at1
            btyp(nbm+count)=i+1
            normm((at1-1)*3+i,nbm+count)=1.0d0
          end if
        end do
        nbm=nbm+count
*--------------------------------------------------------------------------
* ... and all of all of them ...
*--------------------------------------------------------------------------
      else if(row(1:5).eq.'cfull') then
        if(3*natoms.gt.maxnbm) then
          write(*,*) '** maxnbm too small! set maxnbm to 3*natoms!'
          goto 910
        end if
        nbm=0
        count=0
        do at1=1,natoms
          write(*,*) 'current at1: ',at1
          do i=1,3
            count=count+1
            atb(nbm+count)=at1
            btyp(nbm+count)=i+1
            normm((at1-1)*3+i,nbm+count)=1.0d0
          end do
        end do
        nbm=3*natoms
*--------------------------------------------------------------------------
* ... choose all basis vectors in z-direction (useful for planar molecules)
*--------------------------------------------------------------------------
      else if(row(1:4).eq.'aatz') then
        if(natoms.gt.maxnbm) then
          write(*,*) '** maxnbm too small! set maxnbm to larger value!'
          goto 910
        end if
        nbm=0
        count=0
        do at1=1,natoms
          count=count+1
          atb(nbm+count)=at1
          btyp(nbm+count)=4
          normm(at1*3,nbm+count)=1.0d0
        end do
        nbm=natoms
*--------------------------------------------------------------------------
* print mass-weighted vectors in use
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'pm') then
        do i=1,nbm
          call nmod_c2m(normm(1,i),natoms,ic)
        end do
        call printdmat(normm,nvar,nbm,'massweighted basis vectors',
     *   'f20.12',6)
        do i=1,nbm
          call nmod_m2c(normm(1,i),natoms,ic)
        end do
*--------------------------------------------------------------------------
* print vectors in use
*--------------------------------------------------------------------------
      else if(row(1:1).eq.'p') then
        call printdmat(normm,nvar,nbm,'non-massweighted basis vectors',
     *   'f20.12',6)
*--------------------------------------------------------------------------
* xmol XYZ output
*--------------------------------------------------------------------------
      else if(row(1:4).eq.'xmol') then
        do i=1,nbm
          call nmod_c2m(normm(1,i),natoms,ic)
        end do
        call xmol('xmol.XYZ.it0',natoms,coord,nbm,normm,
     *    normm(1,maxnbm+1),xmolsc1,ic)
        do i=1,nbm
          call nmod_m2c(normm(1,i),natoms,ic)
        end do
*--------------------------------------------------------------------------
* g98 fake output
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'g98') then
        do i=1,nvar
          if(abs(normm(i,maxnbm+1)).lt.1.0d-4)
     *    normm(i,maxnbm+1)=10000.0d0
        end do
        call pgaussian('g98.out.it0',normm,natoms,coord,
     *    normm(1,maxnbm+1),ic,nbm)
*--------------------------------------------------------------------------
* intermediate orthogonalization
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'ort') then
        goto 400
*--------------------------------------------------------------------------
* copy vector from buffer to set of vectors in use
* copy also wavenumber, if available
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'cbv'.and.nbmf.gt.0) then
        read(row(4:),*,err=900,end=900) at1
        if(at1.le.0.or.at1.gt.nbmf) goto 910
        nbm=nbm+1
        btyp(nbm)=6
        atb(nbm)=at1
        do i=1,nvar
          normm(i,nbm)=tmod((at1-1)*nvar+i)
        end do
*--------------------------------------------------------------------------
* copy also wavenumber, if available
*--------------------------------------------------------------------------
        normm(nbm,maxnbm+1)=tmod(3*natoms*nbmf+at1)
*--------------------------------------------------------------------------
* copy vector from buffer to set of orthogonal vectors
* (akira keeps solutions orthogonal to these vectors
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'cov'.and.nbmf.gt.0) then
        read(row(4:),*,err=900,end=900) at1
        if(at1.le.0.or.at1.gt.nbmf) goto 910
        nbmo=nbmo+1
        do i=1,nvar
          ortmod(i,nbmo)=tmod((at1-1)*nvar+i)
        end do
*--------------------------------------------------------------------------
* copy all vectors from buffer to set of orthogonal vectors
* (akira keeps solutions orthogonal to these vectors
*--------------------------------------------------------------------------
      else if(row(1:4).eq.'acov'.and.nbmf.gt.0) then
        do j=1,nbmf
          nbmo=nbmo+1
          do i=1,nvar
            ortmod(i,nbmo)=tmod((j-1)*nvar+i)
          end do
        end do
*--------------------------------------------------------------------------
* copy all vectors from buffer to set of vectors in use
*--------------------------------------------------------------------------
      else if(row(1:4).eq.'acbv'.and.nbmf.gt.0) then
        if((nbmf+nbm).gt.maxnbm) then
          write(*,*)
          write(*,*) ' Too many basis vectors in buffer !'
          write(*,*)
          goto 100
        end if
        do j=1,nbmf
          atb(j+nbm)=j
          btyp(j+nbm)=6
          do i=1,nvar
            normm(i,j+nbm)=tmod((j-1)*nvar+i)
          end do
*--------------------------------------------------------------------------
* copy also wavenumber, if available
*--------------------------------------------------------------------------
         normm(j,maxnbm+1)=tmod(3*natoms*nbmf+j)
        end do
        nbm=nbm+nbmf
*--------------------------------------------------------------------------
* enter subsystem menu
*--------------------------------------------------------------------------
      else if(row(1:7).eq.'submenu') then
        if(allocated(tmod)) deallocate(tmod)
        nbmf=0
        allocate(tmod(nvar*(nvar+1)))
        call subsystem(nbmf,natoms,ic,coord,tmod,tmod(nvar*nvar+1:),
     &      apphess)
        if(nbmf.eq.0) then
          deallocate(tmod)
        else
          bbtyp=0
        end if
*--------------------------------------------------------------------------
* read vectors from file
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'re') then
        if(allocated(tmod)) deallocate(tmod)
        nbmf=0
        if(row(3:5).eq.'sub') then
          read(row(6:),*,err=900,end=900) nbmf
        else
          read(row(3:),*,err=900,end=900) nbmf
        end if
        if(nbmf.le.0) then
          nbmf=0
          goto 910
        end if
        write(*,'(2x,A)')
        write(*,'(2x,A)') 'Enter filename:'
        read(*,'(A)',end=910,err=910) filename
        call llen(filename,lf)
        inquire(file=filename(1:lf),exist=ex)
        nasub=natoms
        if(row(3:5).eq.'sub') then
          write(*,'(2x,A)')
          write(*,'(2x,A)') 'Enter No. of atoms in subsystem: '
          write(*,'(2x,A)') '(the first <No.> atoms will be taken !!!)'
          read(*,*,end=910,err=910) nasub
        end if
        if(ex) then
          allocate(tmod(3*natoms*(nbmf+1)))
          at1=nbmf
          if(nasub.lt.natoms.and.nasub.gt.0) then
            allocate(submod(3*nasub*(at1+1)))
            call remod(filename(1:lf),nbmf,submod,
     *       submod(3*nasub*nbmf+1:),nmpres,nasub,ic,coord)
            nmpres=.false.
            do i=0,nbmf-1
              tmod(3*natoms*nbmf+i+1)=submod(3*nasub*nbmf+i+1)
              do j=1,3*natoms
                if(j.le.3*nasub) then
                  tmod(j+i*3*natoms)=submod(j+i*3*nasub)
                else
                  tmod(j+i*3*natoms)=0.0d0
                end if
              end do
            end do
            deallocate(submod)
          else
            call remod(filename(1:lf),nbmf,tmod,tmod(3*natoms*nbmf+1:),
     *           nmpres,nasub,ic,coord)
          end if
          if(nbmf.eq.0) deallocate(tmod)
          bbtyp=0
        else
          nbmf=0
          goto 910
        end if
*--------------------------------------------------------------------------
* display vectors in use by calling MOLDEN
*--------------------------------------------------------------------------
      else if(row(1:4).eq.'disp') then
        if(nbm.eq.0) then
          write(*,*) '** no modes selected !'
          if(nbmf.gt.0)
     &     write(*,*) '** use <bdisp> option to display buffer vectors!'
          goto 910
        end if
        do i=1,nvar
          if(abs(normm(i,maxnbm+1)).lt.1.0d-4)
     *    normm(i,maxnbm+1)=10000.0d0
        end do
        call pgaussian('g98.out.it0',normm,natoms,coord,
     *    normm(1,maxnbm+1),ic,nbm)
        count=system('molden g98.out.it0 1>/dev/null &')
        write(*,*) '---------------------------------------------'
        write(*,*) 'Press <return> to get back to basis selection'
        write(*,*) '---------------------------------------------'
        read(*,'(a)')
*--------------------------------------------------------------------------
* clear all vectors in use
*--------------------------------------------------------------------------
      else if(row(1:1).eq.'c') then
        nbm=0
        do i=1,nvar
          do j=1,maxnbm
            normm(i,j)=0.0d0
          end do
          do j=1,naus
            istepfl((i-1)*naus+j)=1
          end do
        end do
*--------------------------------------------------------------------------
* print mass-weighted vectors in buffer
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'bpm'.and.nbmf.gt.0) then
        do i=1,nbmf
          call nmod_c2m(tmod((i-1)*nvar+1:),natoms,ic)
        end do
        call printdmat(tmod,nvar,nbmf,'massweighted buffer vectors',
     *       'f20.12',6)
        do i=1,nbmf
          call nmod_m2c(tmod((i-1)*nvar+1:),natoms,ic)
        end do
*--------------------------------------------------------------------------
* print vectors in buffer
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'bp'.and.nbmf.gt.0) then
        call printdmat(tmod,nvar,nbmf,'non-massweighted buffer vectors',
     *       'f20.12',6)
*--------------------------------------------------------------------------
* write g98 fake output of vectors in buffer
*--------------------------------------------------------------------------
      else if(row(1:4).eq.'bg98'.and.nbmf.gt.0) then
        do i=1,nvar
          if(abs(normm(i,maxnbm+1)).lt.1.0d-4)
     *    normm(i,maxnbm+1)=10000.0d0
        end do
        call pgaussian('g98.out.buf',tmod,natoms,coord,
     *    normm(1,maxnbm+1),ic,nbmf)
*--------------------------------------------------------------------------
* xmol XYZ output of buffer vectors
*--------------------------------------------------------------------------
      else if(row(1:4).eq.'bxmol'.and.nbmf.gt.0) then
        do i=1,nbm
          call nmod_c2m(tmod((i-1)*nvar),natoms,ic)
        end do
        call xmol('xmol.XYZ.buf',natoms,coord,nbmf,tmod,
     *     normm(1,maxnbm+1),xmolsc1,ic)
        do i=1,nbm
          call nmod_m2c(tmod((i-1)*nvar),natoms,ic)
        end do
*--------------------------------------------------------------------------
* display vectors in buffer by calling MOLDEN
*--------------------------------------------------------------------------
      else if(row(1:5).eq.'bdisp'.and.nbmf.gt.0) then
        do i=1,nvar
          if(abs(tmod(i+3*natoms*nbmf)).lt.1.0d-4)
     *    tmod(i+3*natoms*nbmf)=10000.0d0
        end do
        call pgaussian('g98.out.buf',tmod,natoms,coord,
     *    tmod(3*natoms*nbmf+1:),ic,nbmf)
        count=system('molden g98.out.buf 1>/dev/null &')
        write(*,*) '---------------------------------------------'
        write(*,*) 'Press <return> to get back to basis selection'
        write(*,*) '---------------------------------------------'
        read(*,'(a)')
*--------------------------------------------------------------------------
* mass-weighted breathing mode selected
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'brm') then
        do i=1,nbm
          if(btyp(i).eq.10) goto 920
        end do
        nbm=nbm+1
        btyp(nbm)=10
        call breathm(coord,ic,normm(1,nbm),natoms,.true.)
*--------------------------------------------------------------------------
* breathing mode selected
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'br') then
        do i=1,nbm
          if(btyp(i).eq.7) goto 920
        end do
        nbm=nbm+1
        btyp(nbm)=7
        call breathm(coord,ic,normm(1,nbm),natoms,.false.)
*--------------------------------------------------------------------------
* bending mode selected
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'bd') then
        read(row(3:),*,err=900,end=900) at1,at2,at3
        if(at1.lt.1.or.at2.lt.1.or.at1.gt.natoms.or.at2.gt.natoms.or.
     *  at1.eq.at2) goto 910
        if(at3.lt.1.or.at3.gt.natoms.or.at1.eq.at3.or.at3.eq.at2)
     *  goto 910
        if(checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,5)) goto 920
        nbm=nbm+1
        btyp(nbm)=5
        call getbenbm(coord,natoms,normm,ic,at1,at2,at3,nbm)
        atb(nbm)=at1
        atb2(nbm)=at2
        atb3(nbm)=at3
*--------------------------------------------------------------------------
* torsion selected
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'tor') then
        read(row(4:),*,err=900,end=900) at1,at2,at3,at4
        if(at1.lt.1.or.at2.lt.1.or.at1.gt.natoms.or.at2.gt.natoms.or.
     *  at1.eq.at2) goto 910
        if(at3.lt.1.or.at3.gt.natoms.or.at1.eq.at3.or.at3.eq.at2)
     *  goto 910
        if(at4.lt.1.or.at4.gt.natoms.or.at1.eq.at4.or.at2.eq.at4
     *    .or.at3.eq.at4) goto 910
CCC Not yet        if(checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,5)) goto 920
        nbm=nbm+1
        btyp(nbm)=8
        call gettorsbm(coord,natoms,normm,ic,at1,at2,at3,at4,nbm)
        atb(nbm)=at1
        atb2(nbm)=at2
        atb3(nbm)=at3
        atb4(nbm)=at4
*--------------------------------------------------------------------------
* out-of-plane mode selected
*--------------------------------------------------------------------------
      else if(row(1:3).eq.'oop') then
        read(row(4:),*,err=900,end=900) at1,at4,at2,at3
        if(at1.lt.1.or.at2.lt.1.or.at1.gt.natoms.or.at2.gt.natoms.or.
     *  at1.eq.at2) goto 910
        if(at3.lt.1.or.at3.gt.natoms.or.at1.eq.at3.or.at3.eq.at2)
     *  goto 910
        if(at4.lt.1.or.at4.gt.natoms.or.at1.eq.at4.or.at2.eq.at4
     *    .or.at3.eq.at4) goto 910
CCC Not yet        if(checkat(at1,at2,atb,atb2,at3,atb3,nbm,btyp,5)) goto 920
        nbm=nbm+1
        btyp(nbm)=9
        call getoopbm(coord,natoms,normm,ic,at1,at2,at3,at4,nbm)
        atb(nbm)=at1
        atb2(nbm)=at2
        atb3(nbm)=at3
        atb4(nbm)=at4
*--------------------------------------------------------------------------
* clear set of orthogonal modes
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'oc'.and.nbmo.gt.0) then
        do i=1,nvar
          do j=1,nbmo
            ortmod(i,j)=0.0d0
          end do
        end do
        nbmo=0
*--------------------------------------------------------------------------
* clear buffer
*--------------------------------------------------------------------------
      else if(row(1:2).eq.'bc'.and.nbmf.gt.0) then
        bbtyp=0
        nmpres=.false.
        if(allocated(tmod)) deallocate(tmod)
        nbmf=0
      else
        write(*,*) 'Unknown command'
      end if
      goto 100
*--------------------------------------------------------------------------
* error messages
*--------------------------------------------------------------------------
900   write(*,'(A)') ' Unknown command / Syntax error'
      write(*,*)
      goto 100
910   write(*,'(A)') ' Invalid input ...'
      write(*,*)
      goto 100
920   write(*,'(A)') ' Vector already in basis set ...'
      write(*,*)
      goto 100

*--------------------------------------------------------
* set stepflags
*--------------------------------------------------------

200   do i=1,nbm
        do j=1,naus
          istepfl((i-1)*naus+j)=0
        end do
      end do

300   continue

*--------------------------------------------------------
* orthonormalization of mass-weighted basis modes
*--------------------------------------------------------
      write(*,'(A)')
      write(*,'(A)') ' Do you want to orthogonalize your guess vectors'
      write(*,'(A)') ' to translational and rotational modes ?'
      write(*,'(A)') ' (y/n, default = yes)'
      write(*,'(A)')
      call clean(row)
      read(*,'(A)') row
      call slen(row,lrow)
      if(lrow.eq.0) then
        notrro=.false.
      else if(lrow.eq.1) then
        if(row(1:1).eq.'y'.or.row(1:1).eq.'Y') then
          notrro=.false.
        else if(row(1:1).eq.'n'.or.row(1:1).eq.'N') then
          notrro=.true.
        else
          write(*,'(A)') '** unknown command'
          goto 300
        end if
      else
        write(*,'(A)') '** unknown command'
        goto 300
      end if

400   write(*,'(A)')
      write(*,'(A)')
     *  ' -----------------------------------------------------'
      write(*,'(2x,A)')
     *  'Orthonormalization of mass-weighted basis vectors ...'
      do i=1,nbm
        call nmod_c2m(normm(1,i),natoms,ic)
      end do
      do i=1,nbmo
        call nmod_c2m(ortmod(1,i),natoms,ic)
      end do
*--------------------------------------------------------
* basis vectors should be orthonormal to transrot modes
* also ortmod vectors should be orthonormal to them
*--------------------------------------------------------
      if(.not.notrro) then
       write(*,'(2x,A)')
       write(*,'(1x,A)')
     *  '** creating translational and rotational modes ...'
       allocate(trro(3*natoms,6))
       call trarot(coord,trro(:,1),trro(:,4),ic,natoms)
       do j=1,nbm
        do i=1,6
         call gramschmidt(trro(:,i),normm(1,j),nvar,para,ovl,thres)
        end do
       end do
       do j=1,nbmo
        do i=1,6
         call gramschmidt(trro(:,i),ortmod(1,j),nvar,para,ovl,thres)
        end do
       end do
      else
       write(*,'(1x,A)')
     *  '** skipping orthonormalization to translational '//
     *  'and rotational modes ...'
      end if

*--------------------------------------------------------
* ortmod vectors should be orthonormal to each other
*--------------------------------------------------------
      do j=1,nbmo
       do i=1,j-1
        call gramschmidt(ortmod(1,i),ortmod(1,j),nvar,para,ovl,thres)
       end do
      end do
      if(nbmo.gt.0) then
        call skipvec(ortmod,nvar,nbmo,count,1.0d-2)
        write(*,'(2x,A,i5)')
     &  'No. of vectors removed in orthogonal space: ',count
        nbmo=nbmo-count
      end if
*---------------------------------------------------------------
* shall we keep the solution vectors within the 'orthogonal' set
* or orthogonal to this subspace ?
*---------------------------------------------------------------
      if(nbmo.gt.0) then
        write(*,*)
450     write(*,*) ' You have selected an additional set of vectors'
        write(*,*) ' to restrict the search space for the soluation'
        write(*,*) ' vectors. Do you want to ...'
        write(*,*) '  sub - keep the solution vectors within this set'
        write(*,*) '        (default)'
        write(*,*) '  ort - keep the solution orthogonal to this set'
        write(*,*) '  q   - quit without using this set of vectors'
        write(*,*)
        call clean(row)
        read(*,'(A)') row
        call slen(row,lrow)
        if(lrow.eq.0.or.row(1:3).eq.'sub') then
*         ---------------------------------------------------------
*         create subspace orthogonal to current 'keepspace'
*         (projection onto 'keepspace' is done as orthogonalization
*         to the space orthogonal to 'keepspace')
*         ---------------------------------------------------------
          if(allocated(tmod2)) deallocate(tmod2)
          allocate(tmod2(nvar*nvar))
          call getortspace(ortmod,nbmo,nvar,tmod2,trro)
          deallocate(tmod2)
        else if(row(1:3).eq.'ort') then
          continue
        else if(row(1:1).eq.'q') then
          nbmo = 0
        else
          write(*,'(A)') '** unknown command'
          goto 450
        end if
*--------------------------------------------------------
* orthogonalize basis vectors to set of vectors in ortmod
*--------------------------------------------------------
        do i=1,nbmo
         do j=1,nbm
           call gramschmidt(ortmod(1,i),normm(1,j),nvar,para,ovl,thres)
         end do
        end do
        call skipvec(normm,nvar,nbm,count,1.0d-2)
        nbm=nbm-count
      end if
*--------------------------------------------------------
* basis vectors should be orthonormal to each other
*--------------------------------------------------------
      do j=1,nbm
       do i=1,j-1
        call gramschmidt(normm(1,i),normm(1,j),nvar,para,ovl,thres)
        if(para) then
          write(*,'(2x,a)')
     *     'Linear dependence detected for new basis vector'
          write(*,*)
           write(*,'(2x,a)')
     *       'I will remove this vector from the basis set'
        end if
       end do
      end do
*--------------------------------------------------------------------------
* found linear dependencies ? then skip basis vectors
*--------------------------------------------------------------------------
      call skipvec(normm,nvar,nbm,count,1.0d-2)
      write(*,'(2x,A,i5)') 'No. of vectors removed: ',count
      write(*,'(2x,A)') '... done'
      nbm=nbm-count
      do i=nbm+1,nbm+count
        do j=1,naus
          istepfl((i-1)*naus+j)=1
        end do
      end do

      deallocate(trro)
      write(*,'(2x,A)')
     *  'Recalculating non-massweighted coordinates ...'
      do i=1,nbm
        call nmod_m2c(normm(1,i),natoms,ic)
      end do
      do i=1,nbmo
        call nmod_m2c(ortmod(1,i),natoms,ic)
      end do
      write(*,'(2x,A)')
     *  '... done'
      write(*,'(A)')
     *  ' -----------------------------------------------------'
500   if(.not.valid) goto 100

*--------------------------------------------------------------------------
* fine, mode selection completed;
*--------------------------------------------------------------------------

      if(nmpres.and.nbmf.eq.nvar.and.(.not.apphess)) then

*--------------------------------------------------------------------------
* do we want an advanced preconditioner ?
*--------------------------------------------------------------------------

        write(*,'(A)')
        write(*,'(A)') ' Do you want to use approximate Hessian'
        write(*,'(A)') ' for preconditioning (y/n, default = y) ? '
        write(*,'(A)')
        read(*,'(A)',err=999,end=999) row
        apphess=.false.
        call slen(row,lrow)
        if(lrow.eq.0) row(1:1)='y'
        if(row(1:1).ne.'n') then
          apphess=.true.
          write(*,'(A)')
     *      ' ** writing Hessian guess to file akira_control ...'
          call aphprec(tmod,natoms,tmod(3*natoms*nbmf+1:),ic)
          write(*,'(A)') ' ** ... done'
        end if
      end if
999   return
      end

