#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      subroutine sendres(nit,cpu,wall,escf,gcpu,gwall,grad,
     *   jstepa,nummodes)

c     ==============================================================
c     send final results using SCP, PVM, MPI, or pass them to serres
c
c     written by J. Neugebauer, ETHZ, 2007,
c     using former rslave routines as templates
c     ==============================================================

#include "defs.h"
#include "param.h"

#ifdef AKIRA_PVM
#include "fpvm3.h"
#endif

#ifdef AKIRA_MPI
#include "mpif.h"
#endif

      integer ipoldum,info,j,kstep,mbuf,jstepa,nummodes,intonly
      integer*4 nit
      real*8 escf,cpu,wall,gcpu,gwall,gsum
      real*8 dipol(3),grad(nvar),polar(8,mxomeg)
      real*8 gtenlao(9,mxomeg),gtenao(9,mxomeg),aten(27,mxomeg)
      logical vroaflag

#ifdef AKIRA_MPI
      integer w,st(MPI_STATUS_SIZE)
      data w /MPI_COMM_WORLD/
#endif

      kstep = 1
c     --------------------------------------------------------
c     determine vroaflag and ipoldum
c     --------------------------------------------------------
      vroaflag=.false.
      ipoldum=ipol
      if ((ipol-1000).ge.0) then
        vroaflag=.true.
        ipoldum=ipol-1000
        if (ipoldum.eq.0) ipoldum=1
      end if

c     --------------------------------------------------------
c     determine gsum and write it
c     --------------------------------------------------------
      gsum=0.0d0
      do j=1,nvar
         gsum=gsum+grad(j)**2
      enddo
      write (iosl,*) 'grad norm',gsum
      call flush(iosl)

c     --------------------------------------------------------
c     write no. of SCF iterations, CPU and wall clock time
c     --------------------------------------------------------
      write (iosl,*) 'scf energy: ',escf
      write (iosl,*) 'dipole    : ',dipol(1),dipol(2),dipol(3)
      write (iosl,*) 'grad(1)   : ',grad(1)
      write (iosl,*) 'iterations: ',nit
      write (iosl,*) 'cpu       : ',cpu
      write (iosl,*) 'wall      : ',wall
      write (iosl,*) 'gwall     : ',gwall
      call flush(iosl)

#ifdef AKIRA_MPI
#ifdef AKIRA_MPI_SSEND
c     --------------------------------------------------------
c     MPI block using MPI_SSEND
c     --------------------------------------------------------

        call MPI_SSEND(nit,1,MPI_INTEGER,tid0,fsys_restag,w,info)
        call mpierr('MPI_SSEND nit','rslave',info)
        call MPI_SSEND(cpu,1,MPI_DOUBLE_PRECISION,tid0,801,w,info)
        call mpierr('MPI_SSEND cpu','rslave',info)
        call MPI_SSEND(wall,1,MPI_DOUBLE_PRECISION,tid0,802,w,info)
        call mpierr('MPI_SSEND wall','rslave',info)
        call MPI_SSEND(gcpu,1,MPI_DOUBLE_PRECISION,tid0,803,w,info)
        call mpierr('MPI_SSEND gcpu','rslave',info)
        call MPI_SSEND(gwall,1,MPI_DOUBLE_PRECISION,tid0,804,w,info)
        call mpierr('MPI_SSEND gwall','rslave',info)
        call MPI_SSEND(escf,1,MPI_DOUBLE_PRECISION,tid0,805,w,info)
        call mpierr('MPI_SSEND escf','rslave',info)
        call MPI_SSEND(grad,nvar,MPI_DOUBLE_PRECISION,tid0,807,w,info)
        call mpierr('MPI_SSEND grad','rslave',info)
#else
c     --------------------------------------------------------
c     MPI block using MPI_SEND
c     --------------------------------------------------------

      call MPI_SEND(nit,1,MPI_INTEGER,tid0,fsys_restag,w,info)
      call mpierr('mpi_send nit','rslave',info)
      call MPI_SEND(cpu,1,MPI_DOUBLE_PRECISION,tid0,801,w,info)
      call mpierr('mpi_send cpu','rslave',info)
      call MPI_SEND(wall,1,MPI_DOUBLE_PRECISION,tid0,802,w,info)
      call mpierr('mpi_send wall','rslave',info)
      call MPI_SEND(gcpu,1,MPI_DOUBLE_PRECISION,tid0,803,w,info)
      call mpierr('mpi_send gcpu','rslave',info)
      call MPI_SEND(gwall,1,MPI_DOUBLE_PRECISION,tid0,804,w,info)
      call mpierr('mpi_send gwall','rslave',info)
      call MPI_SEND(escf,1,MPI_DOUBLE_PRECISION,tid0,805,w,info)
      call mpierr('mpi_send escf','rslave',info)
      call MPI_SEND(grad,nvar,MPI_DOUBLE_PRECISION,tid0,807,w,info)
      call mpierr('mpi_send grad','rslave',info)
#endif
#endif

#ifdef AKIRA_PVM
c     --------------------------------------------------------
c     PVM block
c     --------------------------------------------------------
      call PVMFINITSEND(PVMDATADEFAULT,mbuf)
      call PVMFPACK(INTEGER4,nit,kstep,1,info)
      call PVMFPACK(REAL8,cpu,kstep,1,info)
      call PVMFPACK(REAL8,wall,kstep,1,info)
      call PVMFPACK(REAL8,gcpu,kstep,1,info)
      call PVMFPACK(REAL8,gwall,kstep,1,info)
      call PVMFPACK(REAL8,escf,1,1,info)
      call PVMFPACK(REAL8,grad,nvar,1,info)
      call PVMFSEND( TID0, 0, INFO )
#endif

#ifdef AKIRA_SERIAL
c     --------------------------------------------------------
c     SERIAL block
c     --------------------------------------------------------

      call serres(nit,cpu,wall,escf,gcpu,gwall,grad,jstepa,nummodes)
#endif

c     --------------------------------------------------------
c     notify master process
c     --------------------------------------------------------
      call mnotify(5,mytid,9,'results sent',12)

      return
      end
