      subroutine specialnm(natoms,coord,ic,normm,tmp,count)

*-------------------------------------------------------------------
* create special sets of normal coordinates which can be deduced 
* from symmetry arguments
*
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------

      implicit none
      integer natoms,count,ic(natoms),mc,i,itl(4),atc,k,j
      real*8 coord(3,natoms),normm(3,natoms,3*natoms),
     &       tmp(3,natoms,3*natoms),dcom(3)
      real*8 thres,ovl,checkcom,find_mass
      parameter(thres=1.d-6)
      logical lpg,para
      character*3 pg
      external checkcom,find_mass

      count = 0
      do k = 1,3*natoms
        do i = 1,natoms
          do j = 1,3
            tmp(j,i,k) = 0.0d0
          end do
        end do
      end do

*----------------------------------------------
* check tetraeder group (only ML4 works here !)
*----------------------------------------------

      pg = 'td '
      call checkpg('td ',coord,ic,natoms,lpg,thres)
      write(*,*) '** checking for td symmetry ...',lpg
      if(natoms .ne. 5) then
        write(*,*) '** spesym for td works only for ML4 systems'
        lpg = .false.
      end if

*-----------------------
* determine central atom
*-----------------------

      atc = 0
      do i = 1, natoms
        if(abs(coord(1,i)).lt.thres) then
          if(abs(coord(2,i)).lt.thres .and. abs(coord(3,i)).lt.thres)
     &     mc = i
        else
          atc = atc + 1
          itl(atc) = i
        end if
      end do

      if(lpg) then

*       ---------------------------------------------
*       create normal modes for tetraeder point group
*       1. totally symmetric (breathing) mode
*       --------------------------------------------- 
        count = count + 1
        call breathm(coord,ic,normm(1,1,count),natoms,.false.)

*       ---------------------------------------------------------
*       the elements of the breathing mode are identical to
*       the atom-centered z-coordinates (see Bersurker, p. 16/17)
*       ---------------------------------------------------------

        do i = 1,natoms
          do j = 1,3
            tmp(j,i,3*i) = normm(j,i,count)
          end do
          if(i.eq.mc) tmp(3,i,3*i) = 1.0d0
        end do

*       ------------------------------------------------
*       construct x coordinates in atom-centered systems
*       ------------------------------------------------

        do j = 1,3
          tmp(j,itl(1),3*itl(1)-2) = coord(j,itl(4))-coord(j,itl(1))
          tmp(j,itl(4),3*itl(4)-2) = coord(j,itl(1))-coord(j,itl(4))
          tmp(j,itl(2),3*itl(2)-2) = coord(j,itl(2))-coord(j,itl(3))
          tmp(j,itl(3),3*itl(3)-2) = coord(j,itl(3))-coord(j,itl(2))
        end do
        do i = 1,4
          call normalize(tmp(1,1,3*itl(i)),3*natoms)
          call normalize(tmp(1,1,3*itl(i)-2),3*natoms)
          call gramschmidt(tmp(1,1,3*itl(i)),tmp(1,1,3*itl(i)-2),
     &     3*natoms,para,ovl,thres)
          if(para) stop 'ALARM !!! detected linear dependency'
        end do

*       ------------------------------------------------
*       construct y coordinates in atom-centered systems
*       ------------------------------------------------

        do i = 1,4
          call vecprod(tmp(1,itl(i),3*itl(i)-2),tmp(1,itl(i),3*itl(i)),
     &     tmp(1,itl(i),3*itl(i)-1))
           call normalize(tmp(1,1,3*itl(i)-1),3*natoms)
        end do
!        do i = 1, natoms
!          do j = 1, 3
!            tmp(j,i,3*itl(2)-1) = -tmp(j,i,3*itl(2)-1)
!            tmp(j,i,3*itl(3)-1) = -tmp(j,i,3*itl(3)-1)
!          end do
!        end do
      end if

!      atc = 0
!      do k = 1,3*natoms
!        if((k+2)/3.ne.mc) then
!        atc = atc +1
!          do i = 1,natoms
!            do j = 1,3
!              normm(j,i,atc) = tmp(j,i,k) 
!            end do
!          end do
!        end if
!      end do
!      count = 3*natoms-3

      do k = 1,3*natoms
        do i = 1,natoms
          do j = 1,3
            normm(j,i,k) = 0.0d0
          end do
        end do
      end do

      do i = 1,natoms
        do j = 1,3

*         -------------------------
*         construct mode in irrep a
*         -------------------------

          normm(j,i,1) = 0.5d0*(tmp(j,i,3*itl(1))+
     &     tmp(j,i,3*itl(2))+tmp(j,i,3*itl(3))+tmp(j,i,3*itl(4)))

*         -------------------------
*         construct mode in irrep e
*         -------------------------

          normm(j,i,2) = 0.5d0*(tmp(j,i,3*itl(1)-2)-
     &     tmp(j,i,3*itl(2)-2)-tmp(j,i,3*itl(3)-2)+tmp(j,i,3*itl(4)-2))
          normm(j,i,3) = 0.5d0*(tmp(j,i,3*itl(1)-1)-
     &     tmp(j,i,3*itl(2)-1)-tmp(j,i,3*itl(3)-1)+tmp(j,i,3*itl(4)-1))

*         -----------------------------
*         construct mode in irrep t2(')
*         -----------------------------

          normm(j,i,4) = 0.5d0*(tmp(j,i,3*itl(1))-
     &     tmp(j,i,3*itl(2))+tmp(j,i,3*itl(3))-tmp(j,i,3*itl(4)))
          normm(j,i,5) = 0.5d0*(tmp(j,i,3*itl(1))+
     &     tmp(j,i,3*itl(2))-tmp(j,i,3*itl(3))-tmp(j,i,3*itl(4)))
          normm(j,i,6) = 0.5d0*(tmp(j,i,3*itl(1))-
     &     tmp(j,i,3*itl(2))-tmp(j,i,3*itl(3))+tmp(j,i,3*itl(4)))

*         -----------------------------
*         construct mode in irrep t2(")
*         -----------------------------

          normm(j,i,7) = 0.25d0*(-tmp(j,i,3*itl(1)-2)+
     &     tmp(j,i,3*itl(2)-2)-tmp(j,i,3*itl(3)-2)+tmp(j,i,3*itl(4)-2))+
     &     sqrt(3.0d0)/4.0d0*(-tmp(j,i,3*itl(1)-1)+
     &     tmp(j,i,3*itl(2)-1)-tmp(j,i,3*itl(3)-1)+tmp(j,i,3*itl(4)-1))
          normm(j,i,8) = 0.25d0*(-tmp(j,i,3*itl(1)-2)-
     &     tmp(j,i,3*itl(2)-2)+tmp(j,i,3*itl(3)-2)+tmp(j,i,3*itl(4)-2))+
     &     sqrt(3.0d0)/4.0d0*(+tmp(j,i,3*itl(1)-1)+
     &     tmp(j,i,3*itl(2)-1)-tmp(j,i,3*itl(3)-1)-tmp(j,i,3*itl(4)-1))
          normm(j,i,9) = 0.5d0*(tmp(j,i,3*itl(1)-2)+
     &     tmp(j,i,3*itl(2)-2)+tmp(j,i,3*itl(3)-2)+tmp(j,i,3*itl(4)-2))
        end do
      end do
      count = 9

      do i = 1, count
        call normalize(normm(1,1,i),3*natoms)
        call comdist(normm(1,1,i),natoms,ic,dcom)
        write(*,*) 'Center of mass distortion for mode ',i,':',
     &    checkcom(normm(1,1,i),natoms,ic)
        do j = 1,3
          normm(j,mc,i) = normm(j,mc,i)-dcom(j)/find_mass(ic(mc))
        end do
        call normalize(normm(1,1,i),3*natoms)
        write(*,*) 'Center of mass distortion after correction:',
     &    checkcom(normm(1,1,i),natoms,ic)
      end do
      
      end

