#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

      subroutine ssend(gra,stepfl,vmach,escf,pol,nbm,machnumb)

*     ==================================================
*     akira version by J. Neugebauer, Uni Erlangen, 2001
*     update of MPI/sfile version by J. Neugebauer, 2005
*
*     migrate to MachineModule by J. Neugebauer, 2007
*     ==================================================

      use MachineModule

#include "defs.h"
#include "param.h"

#ifdef AKIRA_PVM
#include "fpvm3.h"
#endif

#ifdef AKIRA_MPI
#include "mpif.h"
#endif

*     -----------------------------------------------
*     send steps to be treated to node no. <machnumb>
*     -----------------------------------------------
      type(type_machine), pointer :: mach,qmach
      integer stepfl(*),nbm,machnumb
      real*8 gra(*),escf(*),pol(*)
      integer i
      integer mbuf,info,j,nsec,nsed,nex,now,k,ize,ipe,iprc,cmd,nodf
      integer*4 iwhat,i4,jstep,tid
      real*8 vmach(*)
      character*24 date
      character*40 myname,name

      call findnode_by_num(mach,machnumb)
      if(.not.associated(mach)) call errex('stale pointer in ssend')
      
      if(mach%cmd == 'term') then
        call remnode('term cmd',mach_term,stepfl,mach%numb)
      end if

      if(mach%nodf /= 'rdy') return
      if(mach%cmd == 'vbsy') return

      i=mach%numb
      tid=mach%tid
      call getmach_name(mach,myname)

*     -----------------------
*     check for pending steps
*     -----------------------
      do j=1,nstept
        if (stepfl(j).eq.0) then
          call secs(now)
          call howmany(ize,ipe,iprc,stepfl)
          if (ize.lt.ipe) then
         
*           ----------------------------------------------------
*           check if a running node would finish the present job 
*           earlier than the currently ready node
*           use the current node if no information about its 
*           performance is available
*           ----------------------------------------------------
            nex=mach%last
            if (nex.eq.0) goto 31
            nex=nex+now
           
            nullify(qmach)
            qmach => machlist
            NodeLoop_: do
              if(.not.associated(qmach)) goto 31
           
*             -------------------------
*             OK, there is a ready node
*             -------------------------
           
              if (qmach%nodf == 'rdy') then
           
*               --------------------------------------------------
*               no performance information; bailout at this point, 
*               using it when ssend is called for that node
*               --------------------------------------------------
           
                nsed=qmach%last
                if (nsed.eq.0) exit NodeLoop_
           
*               -----------------------------------------------------------------------
*               check if this one will probably finish earlier than the current one ...
*               -----------------------------------------------------------------------
           
                nsed=qmach%last+now
                if (nsed.lt.nex) exit NodeLoop_
              end if
           
*             ---------------------------------------------------------------------
*             it migh be even advantageous to wait for a currently running node ...
*             (JN: check: does this if construct make sense?)
*             ---------------------------------------------------------------------
           
              if (qmach%nodf == 'rdy') then
                nsed=qmach%last*2+qmach%time
                if (nsed.lt.nex) exit NodeLoop_
              end if
              qmach => qmach%next
            end do NodeLoop_
           
*           --------------------------------------------------------
*           it seems to be advantageous not to use the present node.
*           --------------------------------------------------------
           
            write (idbg,*) '  ssend| ',qmach%name(1:8),
     *       ' expected to finish at ',nsed
            write (idbg,*) '       | ',myname(1:8),
     *       ' expected to finish at ',nex
            write (idbg,*) '       | so wait for ',mach%name(1:8)
***         return
31          continue
          end if
         
          call flush(idbg)
          mach%time = now
          jstep=j
          mach%stpa = jstep
          stepfl(j)=-1
          read (myname(1:8),'(a8)') vmach(j)
          call delp(vmach(j))
          goto 77
        end if
      end do
      return

77    call fdate(date)
      write (jtape,'(a,a,i10,1x,a,a,i4,a,i4)')
     . date,'-> ',tid,myname(1:8),' sending step',jstep
      call flush(jtape)

      iwhat=1
      i4=i
#ifdef AKIRA_MPI
      k=mpi_comm_world
#ifdef AKIRA_MPI_SSEND
      call mpi_ssend(iwhat,1,mpi_integer,tid,409,k,info)
      call mpierr('mpi_ssend iwhat','ssend',info)
#else
      call mpi_send(iwhat,1,mpi_integer,tid,409,k,info)
      call mpierr('mpi_send iwhat','ssend',info)
#endif
#endif

#ifdef AKIRA_PVM
      call pvmfinitsend(pvmdatadefault,mbuf)
      call pvmfpack( integer4, iwhat, 1, 1, info )
      call pvmfsend( tid, 0, info )
#endif

*     ----------------------------------------------------------------
*     NOTE: akira_control file has to be sent again in every iteration
*     (or, to be more precise: in every step). It contains the
*     new basis modes. Received in: islave.r
*     ----------------------------------------------------------------
#ifndef SNF_SCP
      call sfile('akira_control',tid,info)
#endif

#ifdef AKIRA_MPI
#ifdef AKIRA_MPI_SSEND
      call mpi_ssend(i4,1,mpi_integer,tid,410,k,info)
      call mpierr('mpi_ssend i4','ssend',info)
      call mpi_ssend(jstep,1,mpi_integer,tid,411,k,info)
      call mpierr('mpi_ssend jstep','ssend',info)
#else
      call mpi_send(i4,1,mpi_integer,tid,410,k,info)
      call mpierr('mpi_send i4','ssend',info)
      call mpi_send(jstep,1,mpi_integer,tid,411,k,info)
      call mpierr('mpi_send jstep','ssend',info)
#endif
#endif

#ifdef AKIRA_PVM
      call pvmfinitsend(pvmdatadefault,mbuf)
      call pvmfpack( integer4, i4       , 1, 1, info )
      call pvmfpack( integer4, jstep    , 1, 1, info )
      call pvmfsend( tid, 0, info )
#endif
      mach%nodf = 'run'

*     ---------------------------------------------------------------------
*     write restart file
*     ---------------------------------------------------------------------
      call wrestart(vmach,gra,stepfl,escf,pol,nbm)

      return
      end


      subroutine delp(t)
      implicit none
      character*8 t
      integer i,j
      do i=1,8
        if (t(i:i).eq.'.') goto 1
      enddo
      return
1     do j=i,8
        t(j:j)=' '
      enddo
      return
      end
