      subroutine subsystem(nbmf,natoms,ic,coord,normm,vibf,apphess)
*
* construction of approximate Hessian matrix for full system
* from separate Hessians for subsystems
*
*  nsub           : no. of subsystems defined so far
*  flsub(1/2,:)   : first and last atom of each subsystem
*  nnmsub(:)      : no. of modes read for subsystem
*  hssub(:)       : .true. if Hessian has been created for subsystem
*  hcsub(:)       : .true. if Hessian has been copied to full system
*  ncsub(:)       : .true. if modes have been copied to full system
*  
* written by J. Neugebauer, 2003, 2004
*
      use AkiraTypes

#include "defs.h"

      real*8, allocatable :: fullhess(:),worka(:,:),workb(:)

      integer nbmf,natoms,ic(natoms),lrow,maxsub,i,j,idum,info
      parameter(maxsub=9)
      type(type_subhes)   :: subh(maxsub)
      integer afirst,alast,nsub,lsub,nvar,lf,subnvar
      integer flsub(2,maxsub)
      integer nnmsub(maxsub),idum2
      real*8 coord(3,natoms),normm(3*natoms,3*natoms),fqfc
      real*8 vibf(3*natoms),scr,rints
      logical valid,nmpres,ex,apphess,showprint
      logical ncsub(maxsub),hcsub(maxsub),hssub(maxsub)
      character*132 row,filename
      character*9 yesno9
      external fqfc,yesno9

      nbmf=0
      nsub=0
      nvar=3*natoms
      valid=.false.
      showprint=.false.
      allocate(fullhess(nvar*nvar))
      fullhess(:) = 0.0d0
      do i=1,nvar
        vibf(i)=0.0d0
        do j=1,nvar
          normm(i,j)=0.0d0
        end do
      end do
      do i=1,maxsub
        hssub(i)=.false.
        hcsub(i)=.false.
        ncsub(i)=.false.
        nnmsub(i)=0
        flsub(1,i)=0
        flsub(2,i)=0
      end do

10    if(valid) then
        deallocate(fullhess)
        return
      end if
      write(*,*)      
      write(*,*) '===================================================='
      write(*,'(1x,a,i3)') ' No. of subsystems specified so far: ',nsub
      if(nsub.gt.0) then
        write(*,*)
        write(*,'(2x,7a)') 
     &    ' no.','  from','    to','   #modes','  copied?',
     &    '  hessian','  copied?'
      end if
      do i=1,nsub
        write(*,'(3i6,i9,3a9)') i,flsub(1,i),flsub(2,i),nnmsub(i),
     &    yesno9(ncsub(i)),yesno9(hssub(i)),yesno9(hcsub(i))
      end do
      write(*,*) '===================================================='
      write(*,*)      
      write(*,*) ' Select one of the following commands:'
      write(*,*)
      write(*,*) '  ns <i> <j>   : create new subsystem, ranging from'
      write(*,*) '                 atom <i> to <j>'
      write(*,*) '  rnm <i>      : read normal modes for subsystem <i>'
      write(*,*) '  chs <i>      : create Hessian for subsystem <i>'
      write(*,*) 
     *'  trarot <i>   : create trarot-modes for subsystem <i>'
      write(*,*)
     *'  n2fs <i>     : copy modes for subsystem <i> to full system'
      if(showprint) then
        write(*,*) 
     *'  subphs <i>   : print sub-system hessian for subsystem <i>'
        write(*,*) 
     *'  subpnm <i>   : print sub-system normal modes for subsystem <i>'
        write(*,*) 
     *'  subpwn <i>   : print sub-system wavenumbers for subsystem <i>'
        write(*,*)
     *'  phs          : print (appr.) hessian for full system'
        write(*,*)
     *  '  pnm          : print (appr.) full system normal modes'
        write(*,*)
     *  '  pwn          : print (appr.) full system wavenumbers'
        write(*,*) 
     *  '  hidepr       : hide print options'
      else
        write(*,*) 
     *  '  showpr       : show print options'
      end if
      write(*,*)
     *'  c2fh <i>     : copy Hessian for subsystem <i> to full Hessian'
      write(*,*)
     *'  ints <i j r> : set interaction strength between blocks <i> and'
     * //' <j> to <r>'
      write(*,*)
     *'  inta <r>     : set interaction strength for all blocks to <r>'
      write(*,*)
     *'  whs          : write (appr.) hessian to akira_control'
      write(*,*)'                 for preconditioning purposes'
      write(*,*) 
     *'  cnm          : create normal modes by diagonalization'
      write(*,*) 
     * '                 of current full system (appr.) Hessian'
      write(*,*) '  cl           : clear all subsystem entries'
      write(*,*) '  *,q          : return to basis vector menu'
      call clean(row)
      read(*,'(A)') row
      call slen(row,lrow)
      if(lrow.eq.0) then
        continue
      else if(row(1:1).eq.'*'.or.row(1:1).eq.'q') then
*==========================================
* leave this menu
*==========================================
        valid=.true.
      else if(row(1:6).eq.'showpr') then
*==========================================
* show print options
*==========================================
        showprint=.true.
      else if(row(1:6).eq.'hidepr') then
*==========================================
* hide print options
*==========================================
        showprint=.false.
      else if(row(1:2).eq.'ns') then
*==========================================
* specify new subsystem
*==========================================
        read(row(3:),*,end=900,err=900) afirst,alast
        if(afirst.lt.1.or.afirst.gt.natoms) goto 910
        if(alast.lt.1.or.alast.gt.natoms) goto 910
        nsub=nsub+1
        if(afirst.ge.alast) then
          flsub(1,nsub)=alast
          flsub(2,nsub)=afirst
        else 
          flsub(1,nsub)=afirst
          flsub(2,nsub)=alast
        end if
        lsub=flsub(2,nsub)-flsub(1,nsub)+1
        write(idbg,'(1x,2(a,i3))') 
     *   'creating subsystem no. ',nsub,' natoms = ',lsub
        allocate(subh(nsub)%hess(lsub*3,lsub*3))
        allocate(subh(nsub)%normm(lsub*3,lsub*3))
        allocate(subh(nsub)%vibf(lsub*3))
      else if(row(1:3).eq.'rnm') then
*==========================================
* read normal mode input
*==========================================
        read(row(4:),*,end=900,err=900) idum
        if(idum.le.0.or.idum.gt.nsub) goto 910
        lsub=flsub(2,idum)-flsub(1,idum)+1
        write(*,'(2x,A)') 'Enter filename for normal mode input:'
        read(*,'(A)',end=920,err=920) filename
        call llen(filename,lf)
        inquire(file=filename(1:lf),exist=ex)
        write(*,'(2x,A)') 'Enter no. of modes on file:'
        write(*,'(2x,A)') '(<return> = all 3N modes)'
        read(*,'(A)',err=910,end=910) row
        call slen(row,lrow)
        if(lrow.eq.0) then
          subnvar=3*lsub
        else
          read(row,*,end=900,err=900) subnvar
        end if
        if(subnvar.le.0.or.subnvar.gt.3*lsub) goto 910
        if(ex) then
          if(index(filename(1:lf),'forceapprox').eq.0) then
            call remod(filename(1:lf),subnvar,subh(idum)%normm,
     *         subh(idum)%vibf,nmpres,lsub,ic(flsub(1,idum)),
     *         coord(1,flsub(1,idum)))
          else 
            call getforceapprox(filename(1:lf),subnvar,lsub,
     *       subh(idum)%hess)
            hssub(idum)=.true.
          end if
        else
          goto 920
        end if
        if(.not.nmpres) then
          goto 920
        else
          nnmsub(idum)=subnvar
        end if
      else if(row(1:3).eq.'chs') then
*==========================================
* create Hessian for this subsystem
*==========================================
        read(row(4:),*,end=900,err=900) idum
        if(idum.le.0.or.idum.gt.nsub) goto 910
        if(index(filename(1:lf),'forceapprox').eq.0) then
          call nmf2hess(subh(idum)%hess,subh(idum)%normm,
     *      subh(idum)%vibf,lsub,ic(flsub(1,idum)))
          hssub(idum)=.true.
        else
          call hess2nmf(subh(idum)%hess,subh(idum)%normm,
     *     subh(idum)%vibf,lsub,ic(flsub(1,idum)),info)
          if(info.ne.0) goto 920
        end if
      else if(row(1:6).eq.'subphs') then
*==========================================
* print hessian for subsystem
*==========================================
        read(row(7:),*,err=910,end=910) idum
        if(idum.lt.1.or.idum.gt.nsub) goto 910
        lsub=flsub(2,idum)-flsub(1,idum)+1
        write(*,*) 'Print Hessian for subsystem No. ',idum
        call printdmat(subh(idum)%hess,3*lsub,3*lsub,
     *   'Subsystem Hessian','f20.12',6)
      else if(row(1:6).eq.'subpnm') then
*==========================================
* print normal modes for subsystem
*==========================================
        read(row(7:),*,err=910,end=910) idum
        if(idum.lt.1.or.idum.gt.nsub) goto 910
        lsub=flsub(2,idum)-flsub(1,idum)+1
        write(*,*) 'Print mass-weighted normal modes for subsystem No. '
     *    ,idum
        do i=1,3*lsub
          call nmod_c2m(subh(idum)%normm(:,i),lsub,ic(flsub(1,idum)))
        end do
        call printdmat(subh(idum)%normm,3*lsub,3*lsub,
     *   'Subsystem normal modes','f20.12',6)
        do i=1,3*lsub
          call nmod_m2c(subh(idum)%normm(:,i),lsub,ic(flsub(1,idum)))
        end do
      else if(row(1:6).eq.'subpwn') then
*==========================================
* print wavenumbers for subsystem
*==========================================
        read(row(7:),*,err=910,end=910) idum
        if(idum.lt.1.or.idum.gt.nsub) goto 910
        lsub=flsub(2,idum)-flsub(1,idum)+1
        write(*,*) 'Print wavenumbers for subsystem No. ',idum
        call printdmat(subh(idum)%vibf,3*lsub,1,
     *   'Subsystem wavenumbers (1/cm)','f20.12',6)
      else if(row(1:6).eq.'trarot') then
*====================================================
* create translational/rotational modes for subsystem
*====================================================
        read(row(7:),*,err=910,end=910) idum
        if(idum.lt.1.or.idum.gt.nsub) goto 910
        lsub=flsub(2,idum)-flsub(1,idum)+1
        call trarot(coord(1,flsub(1,idum)),subh(idum)%normm,
     &    subh(idum)%normm(:,3),ic(flsub(1,idum)),lsub)
        write(*,*) '** created trarot modes for subsystem ',idum
        nnmsub(idum)=6
        do i=1,6
          subh(idum)%vibf(i)=10000.0d0
          call nmod_m2c(subh(idum)%normm(:,i),lsub,ic(flsub(1,idum)))
        end do
      else if(row(1:4).eq.'n2fs') then
*==========================================
* copy subsystem modes to full system
*==========================================
        read(row(5:),*,err=910,end=910) idum
        if(idum.lt.1.or.idum.gt.nsub) goto 910
        if(nnmsub(idum).eq.0) goto 910
        lsub=flsub(2,idum)-flsub(1,idum)+1
        do i=1,nnmsub(idum)
          call copy2fnorm(normm(1,nbmf+1),nvar,flsub,nsub,
     *     subh(idum)%normm(:,i),lsub,idum,nbmf)
          subh(idum)%vibf(i)=vibf(nbmf+1)
        end do
        write(*,*) 'current nbmf: ',nbmf
        ncsub(idum)=.true.
      else if(row(1:4).eq.'c2fh') then
*==========================================
* copy subsystem hessian to full system
*==========================================
        read(row(5:),*,err=910,end=910) idum
        if(idum.lt.1.or.idum.gt.nsub) goto 910
        if(.not.hssub(idum)) goto 910
        lsub=flsub(2,idum)-flsub(1,idum)+1
        call copy2fhess(fullhess,nvar,flsub,nsub,subh(idum)%hess,lsub,
     *   idum)
        hcsub(idum)=.true.
      else if(row(1:4).eq.'ints') then
*===================================================
* set interaction strength for particular subsystems
*===================================================
        read(row(5:),*,err=910,end=910) idum,idum2,rints
        call intsub(fullhess,nvar,flsub,nsub,rints,idum,idum2)
      else if(row(1:4).eq.'inta') then
*============================================
* set interaction strength for all subsystems
*============================================
        read(row(5:),*,err=910,end=910) rints
        if(nsub.gt.0) then
          do i=1,nsub
           do j=i+1,nsub
            call intsub(fullhess,nvar,flsub,nsub,rints,i,j)
           end do
          end do
        else if(nsub.eq.0) then
          fullhess(:) = rints
        end if
      else if(row(1:3).eq.'phs') then
*==========================================
* print hessian for full system 
*==========================================
        write(*,*) 'Print Hessian for full system'
        call printdmat(fullhess,nvar,nvar,
     *   'Full Hessian','f20.12',6)
      else if(row(1:3).eq.'pnm') then
*==========================================
* print normal modes for full system
*==========================================
        write(*,*) 'Print normal modes for full system'
        do i=1,nvar
          call nmod_c2m(normm(1,i),natoms,ic)
        end do
        call printdmat(normm,nvar,nvar,
     *   'Full system normal modes (mw)','f20.12',6)
        do i=1,nvar
          call nmod_m2c(normm(1,i),natoms,ic)
        end do
      else if(row(1:3).eq.'pwn') then
*==============================================
* print vibrational wavenumbers for full system
*==============================================
        write(*,*) 'Print vibrational wavenumbers for full system'
        call printdmat(vibf,nvar,1,
     *   'Full system wavenumbers','f20.12',6)
      else if(row(1:3).eq.'cnm') then
*==========================================
* diagonalize Hessian to get normal modes
*==========================================
        call hess2nmf(fullhess,normm,vibf,natoms,ic,info)
        nbmf=nvar
        if(info.ne.0) then
          nbmf=0
        else
*-------------------------------------------------------
* reorder normal modes (for displaying them with molden)
*-------------------------------------------------------
          allocate(worka(nvar,nvar),workb(nvar))
          do j=1,nvar
            workb(j) = vibf(j)
            do i=1,nvar
              worka(i,j) = normm(i,j)
            end do
          end do
          do j=1,6
            vibf(nvar-6+j)=workb(j)
            do i=1,nvar
              normm(i,nvar-6+j)=worka(i,j)
            end do
          end do
          do j=7,nvar
            vibf(j-6)=workb(j)
            do i=1,nvar
              normm(i,j-6)=worka(i,j)
            end do
          end do
          deallocate(worka,workb)
          write(*,*) 'successfully created normal modes ...'
        end if 
      else if(row(1:3).eq.'whs') then
        apphess=.true.
        call wraphess(fullhess,natoms)
      else if(row(1:2).eq.'cl') then
*==========================================
* clear all subsystems
*==========================================
        apphess=.false.
        fullhess(:) = 0.0d0
        do i=1,nvar
          vibf(i)=0.0d0
          do j=1,nvar
            normm(i,j)=0.0d0
          end do
        end do
        do i=1,nsub
          deallocate(subh(i)%hess,subh(i)%normm,subh(i)%vibf)
        end do
        nsub=0
      else
        goto 900
      end if
      goto 10

900   write(*,'(A)') ' Unknown command / Syntax error'
      write(*,*)
      goto 10
910   write(*,'(A)') ' Invalid input ...'
      write(*,*)
      goto 10
920   write(*,'(A)') ' Invalid input ...'
      write(*,*)
      deallocate(subh(nsub)%hess,subh(nsub)%normm,subh(nsub)%vibf)
      nsub=nsub-1
      goto 10
      end

*====================================================================

      subroutine nmf2hess(hess,normm,vibf,natoms,ic)
*--------------------------------------------------------------------
* build hessian matrix from normal modes and vibrational frequencies
*
* written by J. Neugebauer, Uni Erlangen, 2003
*--------------------------------------------------------------------
      implicit none
      integer nvar,natoms,ic(natoms),i,j
      real*8 hess(3*natoms,3*natoms),normm(3*natoms,3*natoms)
      real*8 vibf(3*natoms),fqfc
      external fqfc

      nvar=3*natoms
*--------------------------------------------------------------------
* normal modes must be mass-weighted here
*--------------------------------------------------------------------
      do i=1,nvar
        call nmod_c2m(normm(1,i),natoms,ic)
      end do

*--------------------------------------------------------------------
* calculate force constants
*--------------------------------------------------------------------
      do i=1,nvar
        do j=1,nvar
          hess(i,j)=0.0d0
        end do
        hess(i,i)=fqfc(vibf(i))
      end do

*--------------------------------------------------------------------
* calculate approximate cartesian hessian
*--------------------------------------------------------------------
      call hessian_nm2m(hess,natoms,normm)
      call hessianumw(hess,natoms,ic)

*--------------------------------------------------------------------
* normal modes must be mass-unweighted now
*--------------------------------------------------------------------
      do i=1,nvar
        call nmod_m2c(normm(1,i),natoms,ic)
      end do
      end


*====================================================================

      subroutine hess2nmf(hess,normm,vibf,natoms,ic,info)
*--------------------------------------------------------------------
* diagonalize Cartesian Hessian matrix to obtain normal modes and 
* vibrational frequencies
*
* written by J. Neugebauer, Uni Erlangen, 2003
*--------------------------------------------------------------------
      implicit none
      integer nvar,natoms,ic(natoms),i,j,info
      real*8 hess(3*natoms,3*natoms),normm(3*natoms,3*natoms)
      real*8 vibf(3*natoms),qfc2freq
      external qfc2freq

      nvar=3*natoms

*--------------------------------------------------------------------
* mass-weight and diagonalize Hessian
*--------------------------------------------------------------------
      call hessianmw(hess,natoms,ic)
      call hessian_diag(hess,natoms,normm,vibf,info)
      call hessianumw(hess,natoms,ic)

*--------------------------------------------------------------------
* calculate vibrational frequencies from force constants
*--------------------------------------------------------------------
      do i=1,nvar
        vibf(i)=qfc2freq(vibf(i))
      end do

*--------------------------------------------------------------------
* normal modes must be mass-unweighted
*--------------------------------------------------------------------
      do i=1,nvar
        call nmod_m2c(normm(1,i),natoms,ic)
      end do
      end

************************************************************************

      subroutine wraphess(hess,natoms)
*---------------------------------------------
* write (appr.) hessian to akira_control file
*
* written by J. Neugebauer, Uni Erlangen, 2003
*---------------------------------------------
      implicit none
      integer i,j,natoms,nvar,lu,mxrow,nrow
      parameter(mxrow=40000)
      character*132 buf,cbuf(mxrow)
      real*8 hess(3*natoms,3*natoms)
      logical ex

      nvar=3*natoms
      lu=1
      nrow=0
      inquire(file='akira_control',exist=ex)
      open(unit=lu,file='akira_control',err=999)
      if(.not.ex) goto 800
100   read (lu,'(A)',err=998,end=800) buf
101   if((index(buf,'$appr_hessian').ne.0)) then
110     read(lu,'(A)',err=998,end=800) buf
        if(index(buf,'$').eq.0) then
          goto 110
        else
          goto 101
        end if
      else if(index(buf,'$end').ne.0) then
        goto 800
      else
        nrow=nrow+1
        if(nrow.gt.mxrow) then
          write(*,*) '** aphprec: too many rows in akira_control'
          stop
        end if
        cbuf(nrow)(1:132)=buf(1:132)
      end if
      goto 100
800   if(ex) rewind(lu)
      do i=1,nrow
        write(lu,'(A)') cbuf(i)
      end do
      call wrnmodes(natoms,hess,nvar,'$appr_hessian',lu)
      write(lu,'(A)') '$end'
      goto 1000
 
998   write(*,*) '** aphprec: Error while reading akira_control'
      goto 1000
999   write(*,*) '** aphprec: could not open akira_control'
1000  close(lu)
      end

      subroutine copy2fnorm(normm,nvar,flsub,nsub,nmsub,lsub,subno,
     *     nbmf)
*------------------------------------------------------------------
* copies subsystem normal modes to modes for full system
*
* written by J. Neugebauer, Uni Erlangen, July 2003
*------------------------------------------------------------------
      implicit none
      integer i,nvar,nsub,flsub(2,nsub),lsub,subno,i1,i2,nbmf
      real*8 normm(nvar),nmsub(3*lsub)
      
      if(nbmf.ge.nvar) then
        write(*,*) '** copy2fnorm: no. of basis modes: ',nbmf
        write(*,*) '**             cannot add additional modes'
        return
      else
        nbmf=nbmf+1
      end if
      i1=(flsub(1,subno)-1)*3+1
      i2=(flsub(2,subno))*3
      do i=1,nvar
        normm(i)=0.0d0
      end do
      do i=i1,i2
        normm(i)=nmsub(i-i1+1)
      end do
      end

      subroutine copy2fhess(hess,nvar,flsub,nsub,hsub,lsub,subno)
*------------------------------------------------------------------
* copies subsystem Hessian to Hessian for full system
*
* written by J. Neugebauer, Uni Erlangen, July 2003
*------------------------------------------------------------------
      implicit none
      integer i,nvar,nsub,flsub(2,nsub),lsub,j,subno,i1,i2
      real*8 hess(nvar,nvar),hsub(3*lsub,3*lsub)
      
      i1=(flsub(1,subno)-1)*3+1
      i2=(flsub(2,subno))*3
      do i=i1,i2
        do j=i1,i2
          hess(i,j)=hsub(i-i1+1,j-i1+1)
        end do
      end do
      end

      subroutine intsub(hess,nvar,flsub,nsub,rints,subno,subno2)
*------------------------------------------------------------------
* switches on a interaction of strength <rints> (a.u.) between
* different subsystem blocks in the Hessian.
*
* written by J. Neugebauer, Uni Erlangen, July 2003
*------------------------------------------------------------------
      implicit none
      integer i,nvar,nsub,flsub(2,nsub),j,subno,i1,i2,subno2,j1,j2
      real*8 hess(nvar,nvar),rints
      
      i1=(flsub(1,subno)-1)*3+1
      i2=(flsub(2,subno))*3
      j1=(flsub(1,subno2)-1)*3+1
      j2=(flsub(2,subno2))*3
      do i=i1,i2
        do j=j1,j2
          hess(i,j)=rints
          hess(j,i)=rints
        end do
      end do
      end

      subroutine getforceapprox(fname,nvar,natoms,hess)
*---------------------------------------------------------------
* read approximate hessian from file "forceapprox"
*
* Note: this subroutine assumes that a non-massweighted, 
*       Cartesian Hessian is present in forceapprox !
*
* written by J. Neugebauer, Uni Erlangen, July 2003
*---------------------------------------------------------------
      character*(*) fname
      integer nvar,natoms,lf,lu,mxitem,lform,i,j
      integer istart,iend,irow,nent,k
      parameter(mxitem=12)
      real*8 hess(nvar,nvar)
      character*132 row,item(mxitem),form

      i=0
      lu=1
      nent=8
      call slen(fname,lf)
      open(lu,file=fname(1:lf))

10    read(lu,'(A)') row
      if(row(1:1).eq.'#') goto 10
      if(row(1:12).eq.'$forceapprox') then
        call split(row,item,mxitem)
        read(item(2),'(a)') form
        write(*,*) 'found format: ',form
        if(form.ne.'(8f10.5)') call serror('getforceapprox',2,
     *     'format changed in forceapprox',6)
        goto 20
      end if
      goto 10

20    read(lu,'(A)') row
      if(row(1:1).eq.'#') then
        write(*,*) row
        goto 20
      end if
      if(row(1:4).eq.'$end') goto 999
      i=i+1
      if(i.gt.nvar) goto 999
      write(*,*) 'reading coordinate ',i
      irow=(i+nent-1)/nent
      do k=1,irow
        istart=(k-1)*nent+1
        iend=min(i,istart-1+nent)
        write(*,*) '  reading entries ',istart,' to ',iend
        read(row,'(8f10.5)') (hess(j,i),j=istart,iend)
        if(k.lt.irow) read(lu,'(A)') row
      end do
      write(*,*) '...  done          ',i
      goto 20

999   close(lu)
      do i=1,nvar
        do j=i+1,nvar
          hess(j,i)=hess(i,j)
        end do
      end do
      end 

      function yesno9(lin)
      implicit none
      logical lin
      character*9 yesno9
      if(lin) then
        yesno9='      yes'
      else
        yesno9='       no'
      end if
      end
