      subroutine wgauss(tdr,natoms,ic,co,iosl)
*
*-------------------------------------------------------------------
*  
*  generates Gaussian-input file 'force.com' on slaves
*
*  link 0 commands, route section, title section, 
*  and molecule specification are read from 'akira.com' 
*  as provided by the user.
*
*  written by C.Herrmann, Universitaet Bonn, 10/2004
*  
*-------------------------------------------------------------------
*      
      implicit none
c
#include "fconstants.h"
      integer gunit,aunit,jstepa,natoms,nempty,iosl
      integer ic(*),ltd,i,j,ivor,inach,iend,mxrow
      parameter(mxrow=1000)
      real*8 co(3,*)
      character*(*) tdr
      character*80 row(mxrow)
      character*2 find_element
      external find_element
      character*255 fname, aname
      character*255,allocatable :: element(:)
      parameter(gunit=13,aunit=14)
      allocate(element(natoms))
c
      call llen(tdr,ltd)
      aname=tdr(1:ltd)//'/akira.com'
      fname=tdr(1:ltd)//'/force.com'
      nempty=0
c
      open(aunit,file=aname,status='unknown')
      i=1
1     read(aunit,'(a)') row(i)
      if(row(i)(1:10).eq.'          ') nempty = nempty +1
      if(nempty.eq.2) then
        i=i+1
        read(aunit,'(a)') row(i)
        ivor=i
        inach=i+1
        do j=1,natoms
          read(aunit,'(a)')
        end do
        i=i+1
        goto 2
      end if
      i=i+1
      goto 1
2     read(aunit,'(a)',end=3) row(i)
      i=i+1
      goto 2
3     close(aunit)
      iend=i-1
c
      open(unit=gunit,file=fname,status='unknown')
      write(gunit,'(a)') (row(i),i=1,ivor)
      do i=1,natoms
        write(gunit,10) find_element(ic(i)), (co(j,i)*bohr2ang,j=1,3)
      end do
      write(gunit,'(a)') (row(i),i=inach,iend)
10    format(a4,2x,3F18.12)
      write(gunit,*) ''
      close(gunit)

      deallocate(element)
c
      end subroutine wgauss

*************************************************************************
      subroutine wgausscontrol(co,natoms,maxatoms)

*-------------------------------------------------------------------
*
*  Generates fake coord file 
*  in Turbomole format
*  
*  written by C. Herrmann, Universitaet Bonn, 10/2004
*  modified by C. Herrmann, ETH Zurich, 05/30/07
*
*-------------------------------------------------------------------

      implicit none

      integer cunit,gunit,mxitem,lrow,nempty,natoms,i,j,coordunit
      integer ist,end,maxatoms
      parameter(mxitem=10)
#include "fconstants.h"
      character(len=132) row,item(mxitem)
      character(len=2) el(maxatoms)
      real*8 co(3,*),string2real
*
      cunit = 10
      gunit = 11
      coordunit=12
      nempty = 0
*
*-------------------------------------------------------------------
*  read in coordinates from 'akira.com'
*-------------------------------------------------------------------
      open(gunit,file='akira.com',status='unknown')
1     read(gunit,'(a132)',end=2) row
      call llen(row,lrow)
      if(row(1:10).eq.'          ') nempty = nempty +1 
      if(nempty.eq.2) then
      i=1
      read(gunit,'(a132)',end=2) row
3     read(gunit,'(a132)',end=2) row
      if(row(1:10).ne.'          ') then
        read(row,*) el(i), co(1,i), co(2,i), co(3,i)
        do j=1,3
           co(j,i) =  co(j,i)*(1/bohr2ang)
        end do
        i=i+1
        goto 3
      end if
      end if
      goto 1 
2     continue
      close(gunit)
      natoms = i-1
*

*-------------------------------------------------------------------
*  write akira_control file
*-------------------------------------------------------------------
      open(cunit,file='akira_control',status='unknown')
      write(cunit,'(a11)') '$anfmode 30'
      write(cunit,'(a)') '$rundimensions '
      write(cunit,'(a11,2x,i3)')  '  natoms= ',natoms
      write(cunit,'(a)') '$end'
      close(cunit)

*-------------------------------------------------------------------
*  write fake coord file
*-------------------------------------------------------------------
      open(coordunit,file='coord')
      write(coordunit,'(a)') '$coord'
      do i = 1,natoms
      write(coordunit,'(3f18.12,2x,a3)') (co(j,i),j=1,3),el(i)
      end do
      write(coordunit,'(a)') '$end'
      close (coordunit)
*
      end subroutine wgausscontrol
c
c
c***********************************************************************
c
      subroutine chkgnosymm
c
c check whether Keyword 'nosymm' is present in Gaussian input file -
c if not, insert it.
c
c written by C. Herrmann, University of Jena, 2005
c
      implicit none
c
      integer :: gunit,mxrow,i,iend,mxitem,litem,j
      parameter(mxrow = 500,gunit=17,mxitem=20)
      integer :: lrow(mxrow)
      character(len=132) :: row(mxrow),item(mxitem)
c
      i=1
      open(gunit,file='akira.com',status='unknown')
1     read(gunit,'(a132)',end=2) row(i)
      call llen(row(i),lrow(i))
      call split(row(i),item,mxitem)
      if(item(1)(1:1).eq.'#') then
        row(i) = '#'
        lrow(i) = 1
        do j=2,mxitem
          call llen(item(j),litem)
          if(index(item(j),'symm').eq.0.and.index(item(j),'Symm')
     *      .eq.0) then
            row(i)=row(i)(1:lrow(i))//' '//item(j)(1:litem)
            call llen(row(i),lrow(i))
          end if
        end do
        row(i) = row(i)(1:lrow(i))//' NoSymm'
        call llen(row(i),lrow(i))
      end if
      i=i+1
      goto 1
2     continue
      iend=i-1
      rewind(gunit)
      do i = 1,iend
        write(gunit,'(a)') row(i)(1:lrow(i))
      end do
      close(gunit)
c
      end subroutine chkgnosymm
