      subroutine checkpg(pg,coord,ic,natoms,valid,thres)

*-------------------------------------------------------------------
* check whether the point group symbol pg is in accordance with the
* the coordinates of the atoms given in coord
*
* based on subroutines by C. Kind and J. Neugebauer, Uni Erlangen, 2002
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------

      implicit none
      integer natoms,ic(natoms)
      real*8 coord(3,natoms),thres
      logical valid,check_symel
      character*3 pg
      external check_symel

      valid = .false.

*-----------------------------
* check tetraeder point groups
*-----------------------------

      if(pg(1:1).eq. 't') then
        valid=check_symel(natoms,coord,ic,'o1301  ',thres)
        valid=valid.and.check_symel(natoms,coord,ic,'o1303  ',thres)
        if(pg.eq.'td') then
          valid=valid .and.check_symel(natoms,coord,ic,'p1401  ',thres)
        endif
        if(pg.eq.'th') then
          valid=valid .and.check_symel(natoms,coord,ic,'sh     ',thres)
        endif
      end if

      end

*===================================================================

      logical function check_symel(natoms,coord,ic,symmel,thres)

*-------------------------------------------------------------------
* check if set of atoms described by coord and ic is in accordance 
* with the symmetry element symmel
*
* based on subroutines by C. Kind and J. Neugebauer, Uni Erlangen, 2002
* written by J. Neugebauer, Uni Erlangen, 2002
*-------------------------------------------------------------------

      implicit none
      integer natoms,ic(natoms),i,j
      real*8 coord(3,natoms),thres,symco(3),symmat(3,3)
      logical valat,atom_equal
      character*7 symmel
      external atom_equal

      check_symel = .true.
      call getsymelmat(symmel,symmat)
      do i = 1, natoms

*       ---------------------------------------------------------------
*       operate on each atom in the molecule with the symmetry operator
*       ---------------------------------------------------------------

        call matvecmul3d('n',symmat,coord(1,i),symco)
        valat = .false.
        do j = 1, natoms

*         -------------------------------------------------------------
*         check which atom corresponds to the symmetry transformed atom
*         -------------------------------------------------------------

          if(atom_equal(symco,ic(i),coord(1,j),ic(j),thres)) 
     &      valat = .true.
        end do
        check_symel = check_symel .and. valat
        if(.not.check_symel) return
      end do

      end 

*===================================================================

      logical function atom_equal(co1,ic1,co2,ic2,thres)

*----------------------------------------------------------------------
* check if atoms described by coordinates co1, co2 and nuclear charges
* ic1, ic2 are equal
*
* based on subroutines by C. Kind and J. Neugebauer, Uni Erlangen, 2002
* written by J. Neugebauer, 2004
*----------------------------------------------------------------------

      implicit none
      integer ic1,ic2,i
      real*8 co1(3),thres,co2(3)
      logical valid

      atom_equal = (ic1 .eq. ic2)
      do i = 1, 3
        if(abs(co1(i)-co2(i)) .gt. thres) then
          atom_equal = .false.
          return
        end if
      end do

      end
