*********************************************************
* ALL SUBROUTINES AND FUNCTIONS WRITTEN BY J. NEUGEBAUER,
* University Erlangen, 2002, 2003
*********************************************************
      double precision function norm3d(x)
* calculates the norm of a vector in R3
      implicit none
      integer i
      real*8 x(3),sqrt
      intrinsic sqrt
      norm3d=0.0d0
      do i=1,3
        norm3d=norm3d+x(i)*x(i)
      end do
      norm3d=sqrt(norm3d)
      end

      double precision function norm(x,n)
* calculates the norm of a vector in Rn
      implicit none
      integer i,n
      real*8 x(n),sqrt
      intrinsic sqrt
      norm=0.0d0
      do i=1,n
        norm=norm+x(i)*x(i)
      end do
      norm=sqrt(norm)
      end
     
      double precision function scalar3d(x1,x2)
* calculates the scalar product x1*x2
      implicit none
      integer i
      real*8 x1(3),x2(3)
      scalar3d=0.0d0
      do i=1,3
        scalar3d=scalar3d+x1(i)*x2(i)
      end do
      end

      subroutine normalize3d(x)
* normalize vector x
      implicit none
      integer i
      real*8 x(3),norm3d,nnorm
      external norm3d
      nnorm=norm3d(x)
      do i=1,3
        x(i)=x(i)/nnorm
      end do
      end

      double precision function anglxy(x,y)
* calculate angle between two vectors x,y in 3D
      implicit none
      real*8 acos,scalar3d,norm3d,x(3),y(3)
      external norm3d,scalar3d
      intrinsic acos
      anglxy=acos(scalar3d(x,y)/norm3d(x)/norm3d(y))
      end 
 
      subroutine getnormv(x1,x2,x3,xn)
* calculate vector normal to plane (x1,x2,x3)
      implicit none
      integer i
      real*8 x1(3),x2(3),x3(3),xn(3),d13(3),d12(3)
      real*8 scale,thres
      logical zd12(3),zd13(3),zdenom(3)
      parameter(thres=1.0d-6)
      do i=1,3
        zd12(i)=.true.
        zd13(i)=.true.
        d12(i)=x2(i)-x1(i)
        d13(i)=x3(i)-x1(i)
        if(abs(d12(i)).lt.thres) zd12(i)=.false.
        if(abs(d13(i)).lt.thres) zd13(i)=.false.
      end do
      zdenom(1)=abs(d12(2)*d13(3)-d13(2)*d12(3)).gt.thres
      zdenom(2)=abs(d12(1)*d13(3)-d13(1)*d12(3)).gt.thres
      zdenom(3)=abs(d12(1)*d13(2)-d13(1)*d12(2)).gt.thres
      if(.not.zd12(3).and..not.zd13(3)) then
        write(*,*) ' all atoms in xy-plane'
        xn(1)=0.0d0 
        xn(2)=0.0d0
        xn(3)=1.0d0
      else if(.not.zd12(2).and..not.zd13(2)) then
        write(*,*) ' all atoms in xz-plane'
        xn(1)=0.0d0
        xn(2)=1.0d0
        xn(3)=0.0d0
      else if(.not.zd12(1).and..not.zd13(1)) then
        write(*,*) ' all atoms in yz-plane'
        xn(1)=1.0d0
        xn(2)=0.0d0
        xn(3)=0.0d0
      else if(zd12(1).and.zdenom(2)) then
        scale=d13(1)/d12(1)
        xn(2)=1.0d0
        write(*,*) '3: denominater: ',(d13(3)-d12(3)*scale)
        xn(3)=(-1.0d0*xn(2)*(d13(2)-d12(2)*scale))/(d13(3)-d12(3)*scale)
        xn(1)=(-1.0d0*xn(2)*d12(2)-xn(3)*d12(3))/d12(1)
      else if(zd13(3).and.zdenom(1)) then
        scale=d12(3)/d13(3)
        write(*,*) '1: denominater: ',(d13(2)-d12(2)*scale)
        xn(1)=1.0d0
        xn(2)=(-1.0d0*xn(1)*(d12(1)-d13(1)*scale))/(d12(2)-d13(2)*scale)
        xn(3)=(-1.0d0*xn(1)*d13(1)-xn(2)*d13(2))/d13(3)
      else if(zd12(3).and.zdenom(1)) then
        scale=d13(3)/d12(3)
        xn(1)=1.0d0
        write(*,*) '2: denominater: ',(d13(2)-d12(2)*scale)
        xn(2)=(-1.0d0*xn(1)*(d13(1)-d12(1)*scale))/(d13(2)-d12(2)*scale)
        xn(3)=(-1.0d0*xn(1)*d12(1)-xn(2)*d12(2))/d12(3)
      else if(zd13(1).and.zdenom(2)) then
        scale=d12(1)/d13(1)
        xn(2)=1.0d0
        write(*,*) '4: denominater: ',(d12(3)-d13(3)*scale)
        xn(3)=(-1.0d0*xn(2)*(d12(2)-d13(2)*scale))/(d12(3)-d13(3)*scale)
        xn(1)=(-1.0d0*xn(2)*d13(2)-xn(3)*d13(3))/d13(1)
      else if(zd12(2).and.zdenom(3)) then
        scale=d13(2)/d12(2)
        xn(3)=1.0d0
        write(*,*) '5: denominater: ',(d13(1)-d12(1)*scale)
        xn(1)=(-1.0d0*xn(3)*(d13(3)-d12(3)*scale))/(d13(1)-d12(1)*scale)
        xn(2)=(-1.0d0*xn(3)*d12(3)-xn(1)*d12(1))/d12(2)
      else if(zd13(2).and.zdenom(3)) then
        scale=d12(2)/d13(2)
        xn(3)=1.0d0
        write(*,*) '6: denominater: ',(d12(1)-d13(1)*scale)
        xn(1)=(-1.0d0*xn(3)*(d12(3)-d13(3)*scale))/(d12(1)-d13(1)*scale)
        xn(2)=(-1.0d0*xn(3)*d13(3)-xn(1)*d13(1))/d13(2)
      else if(zd13(3).and.zdenom(1)) then
        scale=d12(3)/d13(3)
        write(*,*) '1: denominater: ',(d13(2)-d12(2)*scale)
        xn(1)=1.0d0
        xn(2)=(-1.0d0*xn(1)*(d12(1)-d13(1)*scale))/(d12(2)-d13(2)*scale)
        xn(3)=(-1.0d0*xn(1)*d13(1)-xn(2)*d13(2))/d13(3)
      else
        write(*,*) 'Error calculating normal vector'
      end if
      end

      subroutine vecprod(vec1,vec2,resvec)
***************************************************************
* calculates the vector product resvec(3) = vec1(3) x vec2(3)
* 
* written by J. Neugebauer, Uni Erlangen, 2003
***************************************************************
      implicit none
      integer i
      real*8 vec1(3),vec2(3),resvec(3)
      resvec(1)=vec1(2)*vec2(3)-vec1(3)*vec2(2)
      resvec(2)=vec1(3)*vec2(1)-vec1(1)*vec2(3)
      resvec(3)=vec1(1)*vec2(2)-vec1(2)*vec2(1)
      end 

*-------------------------------------------------------------

      subroutine matmul3d(t1,t2,mat1,mat2,resmat)

*--------------------------------------------------
*     returns product resmat=mat1 * mat2
*
*     written by J. Neugebauer, Uni Erlangen, 2002
*--------------------------------------------------
      implicit none
      integer i,j,k
      real*8 mat1(3,3),mat2(3,3),resmat(3,3)
      character*1 t1,t2
      if(t1.eq.'t'.and.t2.ne.'t') then
        do i=1,3
          do j=1,3
            resmat(i,j)=0.0d0
            do k=1,3
              resmat(i,j)=resmat(i,j)+mat1(k,i)*mat2(k,j)
            end do
          end do
        end do
      else if(t1.eq.'t'.and.t2.eq.'t') then
        do i=1,3
          do j=1,3
            resmat(i,j)=0.0d0
            do k=1,3
              resmat(i,j)=resmat(i,j)+mat1(k,i)*mat2(j,k)
            end do
          end do
        end do
      else if(t1.ne.'t'.and.t2.eq.'t') then
        do i=1,3
          do j=1,3
            resmat(i,j)=0.0d0
            do k=1,3
              resmat(i,j)=resmat(i,j)+mat1(i,k)*mat2(j,k)
            end do
          end do
        end do
      else
        do i=1,3
          do j=1,3
            resmat(i,j)=0.0d0
            do k=1,3
              resmat(i,j)=resmat(i,j)+mat1(i,k)*mat2(k,j)
            end do
          end do
        end do
      end if
      end

*-------------------------------------------------------------

      function dotprod(vec1,vec2,lvec)

*--------------------------------------------------
*     returns dot product vec1 * vec2
*
*     written by J. Neugebauer, Uni Erlangen, 2002
*--------------------------------------------------
      implicit none
      integer lvec,i
      real*8 dotprod,vec1(lvec),vec2(lvec)
      dotprod=0.0d0
      do i=1,lvec
        dotprod=dotprod+vec1(i)*vec2(i)
      end do
      end

*-------------------------------------------------------------

      subroutine matvecmul3d(t,mat,vec,resvec)

*--------------------------------------------------
*     returns product resvec=mat * vec
*
*     written by J. Neugebauer, Uni Erlangen, 2002
*--------------------------------------------------
      implicit none
      integer i,j
      real*8 mat(3,3),vec(3),resvec(3)
      character*1 t
      if(t.eq.'t') then
        do i=1,3
          resvec(i)=0.0d0
          do j=1,3
            resvec(i)=resvec(i)+mat(j,i)*vec(j)
          end do
        end do
      else
        do i=1,3
          resvec(i)=0.0d0
          do j=1,3
            resvec(i)=resvec(i)+mat(i,j)*vec(j)
          end do
        end do
      end if
      end

******************************************************************************
   
      subroutine normalize(vector,nvec)
*
* normalization of <vector> 
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer i,nvec
      real*8 vector(nvec),sum,sqrt,zthres
      parameter(zthres=1.0d-12)
      intrinsic sqrt
   
      sum=0.0d0
      do i=1,nvec
        sum=sum+vector(i)*vector(i)
      end do
      sum=sqrt(sum)
      if(sum.le.zthres) sum=1.0d0
      do i=1,nvec
        vector(i)=vector(i)/sum
      end do
      end 
 
