      SUBROUTINE rslavem40(nbm)

****************************************************
* akira version by J. Neugebauer, Uni Erlangen, 2001
****************************************************
* rslave-routine for ADF single-point calculations
* written by C. Herrmann, Uni Bonn 2004,
* using large parts of rslavem4.r written by
* J. Neugebauer
*
* f90 memory management by J. Neugebauer, 2007
****************************************************

#include "defs.h"
#include "param.h"

      real*8, allocatable :: grad(:),co(:,:),normmodes(:)
      integer, allocatable ::  ilab(:)

      integer info,nn,irc,l,mypid,j,mbuf,iclean,icleanl,i,idmy
      real*8 escf,gsum,mmgrad(nvar)
      integer ldscf,kstep,xprog,lforce,lmfout,nproc,system,lfout,laout
      integer*4 jstepa,nbm,totlmfout
      character*24 date
      character*255 forcename,forceout,mmforceout,adfout,b1
      real*8 cpu,wall,gwall,gcpu
      integer xeq,tmpxeq,icmp,qmnvar,mmnvar,nit
      character*5 nic
      logical found,parafreq,anffast

*-------------------------------------------------------------------
* allocate arrays
*-------------------------------------------------------------------
      allocate(grad(nvar),co(3,natoms),normmodes(nvar*maxnbm))
      allocate(ilab(natoms))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c loop over steps
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      kstep=1
      write(iosl,*) 'rslavem40: calling rslave'
      call flush(iosl)
      call rslave(co,ilab,normmodes,jstepa,iclean)
      write(iosl,*) 'rslavem40: back from rslave'

*-------------------------------------------------------------------
* prepare ADF input; use new set of coordinates
*-------------------------------------------------------------------
      call flush(iosl)
      call rcoord(tempdir(1:lt)//'/coord',nn,co,ilab)
      call flush(iosl)
      call wadf(tempdir(1:lt),natoms,ilab,co,iosl)
      call flush(iosl)
c
      anffast=.false.
      nic='nice '
      call rkeywd(tempdir(1:lt)//'/akira_control','$anffast',anffast)
      if(anffast) then
        nic='     '
        write (iosl,*) 'test calculation: omitting command nice'
      end if
c
*-------------------------------------------------------------------
* remove TAPE21 and t21* files from previous ADF runs 
* (otherwise, ADF will not run properly)
*-------------------------------------------------------------------
         info=system('rm '//tempdir(1:lt)//'/'//'TAPE21 t21*')
*-------------------------------------------------------------------
* do the calculation (ADF)
*-------------------------------------------------------------------
         CALL augnum(adfout,'adfslave.out.',jstepa)
         CALL llen(adfout,laout)
           write(iosl,*) '** rslavem40: running ADF'
           write(iosl,*) 'tempdir(1:lt)//adfout(1:laout) = ', 
     *     tempdir(1:lt)//'/'//adfout(1:laout)
           call flush(iosl)
          write(iosl,*) nic(1:5)//tempdir(1:lt)//'/adfslave.in > '
     *          //tempdir(1:lt)//'/'//adfout(1:laout)
          call flush(iosl)
           info=xprog(nic(1:5)//tempdir(1:lt)//'/adfslave.in',
     *          tempdir(1:lt)//'/'//adfout(1:laout))
*-------------------------------------------------------------------
* check for SCF convergence
*-------------------------------------------------------------------
      write(iosl,*) '** rslavem40: looking for NOT CONVERGED'
      CALL flush(iosl)
      CALL findstr(tempdir(1:lt)//'/'//adfout(1:laout),'NOT CONVERGED',
     *             found)
      write(iosl,*) '** rslavem40: NOT CONVERGED',found
      CALL flush(iosl)
      IF (found) THEN
       CALL mnotify(5,mypid,7,'NOT CONVERGED found',13)
       WRITE (iosl,*)
     * 'NOT CONVERGED found in adf output file ... shutting down node'
       CALL flush(iosl)
       CALL slshutdown(1,tid0,mytid)
      ENDIF
*-------------------------------------------------------------------
* extract gradient information from TAPE21 
*-------------------------------------------------------------------
         CALL augnum(forceout,'grad.out.',jstepa)
         CALL llen(forceout,lfout)
         CALL augnum(mmforceout,'MM.energies.',jstepa)
         CALL llen(mmforceout,lmfout)
        write(iosl,*) nic(1:5)//'$ADFBIN/dmpkf '//tempdir(1:lt)//
     *    '/TAPE21 Geometry GeoOpt Energy > '//
     *    tempdir(1:lt)//'/'//forceout(1:lfout)
        call flush(iosl)
        info=system(nic(1:5)//'$ADFBIN/dmpkf '//tempdir(1:lt)//
     *    '/TAPE21 Geometry GeoOpt Energy > '//
     *    tempdir(1:lt)//'/'//forceout(1:lfout))
*---------------------------------------------------
* read and save gradient
*---------------------------------------------------
      if(mode.eq.40) then
        CALL gadfgrad(natoms,grad,tempdir(1:lt)//'/'
     *                //forceout(1:lfout))
      else if(mode.eq.41) then
	call flush(iosl)
        CALL gadfgrad(lastqmatom,grad,tempdir(1:lt)//'/'
     *                //forceout(1:lfout))
        call gadfmmgrad(natoms,lastqmatom,grad,
     *  tempdir(1:lt)//'/'//'MM.energies')
        info=system('mv '//tempdir(1:lt)//'/'//'MM.energies '//
     *  tempdir(1:lt)//'/'//mmforceout(1:lmfout))
        mmnvar = (natoms-lastqmatom)*3
        qmnvar = lastqmatom*3
      end if
      write(iosl,*) '** rslavem40: gradient read'
       CALL flush(iosl)
      cpu=0.0d0
      wall=0.0d0
      gcpu=0.0d0
      gwall=0.0d0
      call flush(iosl)
      if(info.ne.0) call errex('rslavem40: system command failed')
      call flush(iosl)

*     --------------------------------------------------
*     read and save total bonding energy
*     --------------------------------------------------
      CALL gadfenergy(tempdir(1:lt)//'/'//forceout(1:lfout),escf)

*     ------------------------------------------------------------------
*     send final results
*     ------------------------------------------------------------------
      call sendres(nit,cpu,wall,escf,gcpu,gwall,grad,jstepa,nbm)

      write(iosl,*) '** rslavem40: results sent'

      call flush(iosl)
      call mnotify(5,mypid,9,'results sent',12)

*     ------------------------------------------------------------------
*     deallocate arrays
*     ------------------------------------------------------------------
      deallocate(grad,co,normmodes)
      deallocate(ilab)

      return
      end subroutine rslavem40
